#ifndef __GRAPHICS_H_
#define __GRAPHICS_H_

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <string>
#include <vector>

#define SCREENBPP 32

#define GOPT_WRAP_IN_SPACES (1<<0)

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
#define rmask 0xff000000
#define gmask 0x00ff0000
#define bmask 0x0000ff00
#define amask 0x000000ff
#else
#define rmask 0x000000ff
#define gmask 0x0000ff00
#define bmask 0x00ff0000
#define amask 0xff000000
#endif

struct Fonts
{
	TTF_Font *normal, *small, *quitesmall, *big;
};

struct ViewableRow
{
	ViewableRow();
	ViewableRow(std::string);
	void SetHilight(int start, int length);
	std::string text;
	int hilight_start, hilight_length;
	SDL_Color color;
};

struct ViewableText
{
	std::vector<ViewableRow> rows;
	TTF_Font *font;
	int height;
	int width;
};

void GSlock(SDL_Surface *s);
void GSulock(SDL_Surface *s);

void GDrawPixel(SDL_Surface *surface, int x, int y, Uint32 color);
Uint32 GGetPixel(SDL_Surface *surface, int x, int y);
bool IsPixelColor(SDL_Surface *surface, int x, int y,
		Uint8 r, Uint8 g, Uint8 b);
bool IsBlockSingleColoured(SDL_Surface *surface, int sx, int sy,
		int w, int h, Uint8 r, Uint8 g, Uint8 b);

void GDrawRect(SDL_Surface *surface, int tox, int toy, int w, int h, Uint32 color);
void GDrawAlphaRect(SDL_Surface *surface, int tox, int toy, int w, int h,
		Uint32 color, Uint8 alpha);
void GDrawAlphaRectArea(SDL_Surface *dest_surface, int x, int y, int w, int h,
		int minx, int miny, int areaw, int areah, Uint32 color, Uint8 ca);

void GDrawIMG(SDL_Surface *img, SDL_Surface *toimg, int x, int y);
void GDrawIMG(SDL_Surface *img, SDL_Surface *toimg, int x, int y,
		int w, int h, int x2, int y2);
void GDrawIMGArea(SDL_Surface *dest_surface, SDL_Surface *img, int x, int y,
		int w, int h, int x2, int y2,
		int minx, int miny, int areaw, int areah);
void GDrawIMGArea(SDL_Surface *dest_surface, SDL_Surface *img, int x, int y,
		int minx, int miny, int areaw, int areah);

int GDrawText(SDL_Surface *dest_surface, int x, int y, int textx, int textw,
		TTF_Font *font, std::string &text, SDL_Color color);
int GDrawText(SDL_Surface *dest_surface, int x, int y, int textx, int textw,
		TTF_Font *font, std::string &text, Uint8 r, Uint8 g, Uint8 b);
int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font, 
		std::string &text, SDL_Color color);
/*int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font,
		std::string text, Uint8 r, Uint8 g, Uint8 b);*/
int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font,
		std::string &text, Uint8 r, Uint8 g, Uint8 b);
int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font,
		const char *text, Uint8 r, Uint8 g, Uint8 b);

ViewableText GWrapText(TTF_Font *font, int w, int options, std::string text);

void GDrawMultilineText(SDL_Surface *dest_surface, int textx, int texty, int texth, int textw,
		int screen_x, int screen_y, ViewableText &vtext);
void GDrawMultilineText(SDL_Surface *dest_surface, int texty, int texth,
		int screen_x, int screen_y, ViewableText &vtext);
void GDrawMultilineText(SDL_Surface *dest_surface,
		int screen_x, int screen_y, ViewableText &vtext);

#if 0
class Graphics
{
public:
	Graphics();
	~Graphics();
	SDL_Surface* GetScreen();
	int InitVideo(int w, int h);
	int GetW();
	int GetH();
	void UpdateScreen();
	/*void Slock();
	void Sulock();
	void DrawPixel(int x, int y, Uint32 color);
	void DrawRect(int x, int y, int w, int h, Uint32 color);
	void DrawAlphaRect(int x, int y, int w, int h, Uint32 color, Uint8 alpha);
	void DrawIMG(SDL_Surface *img, int x, int y);
	void DrawIMG(SDL_Surface *img, int x, int y,
			int w, int h, int x2, int y2);
	void ClearScreen();

	int DrawText(int x, int y, int textw, int textx,
			TTF_Font *font, std::string &text, SDL_Color color);
	int DrawText(int x, int y, int textw, int textx,
			TTF_Font *font, std::string &text, Uint8 r, Uint8 g, Uint8 b);
	int DrawText(int x, int y, TTF_Font *font, std::string &text, SDL_Color color);
	int DrawText(int x, int y, TTF_Font *font, std::string text, Uint8 r, Uint8 g, Uint8 b);
	ViewableText WrapText(TTF_Font *font, int w, int options, std::string text);
	void DrawMultilineText(int textx, int texty, int texth, int textw,
			int screen_x, int screen_y, ViewableText &vtext);
	void DrawMultilineText(int texty, int texth, int screen_x, int screen_y, ViewableText &vtext);
	void DrawMultilineText(int screen_x, int screen_y, ViewableText &vtext);*/
private:
	SDL_Surface *screen;
};
#endif

#endif

