#ifndef __MAP_H_
#define __MAP_H_

#include "object.h"
#include "objectimagelib.h"
#include "mapimagelib.h"
#include "coordinates.h"
#include <vector>
#include <string>

class Object;

#define DEFAULTBLOCKW 40
#define DEFAULTBLOCKH 40

#define MAPBLOCK_SHORTID_CHARS 32
#define MAPBLOCK_MOVE_SHORTID_CHARS 32
#define MAPIMAGE_SHORTID_CHARS 32
#define MAPNAMEDPLACE_NAME_CHARS 32

//kentän yksi blokki
struct MapBlock
{
	MapBlock();
	char tile_shortid[MAPBLOCK_SHORTID_CHARS];
	char move_shortid[MAPBLOCK_MOVE_SHORTID_CHARS];
	char y_change_per_10x_right;
	char special_info;
	
	bool hilighted;
};

//kentällä olevan mapimagen info
struct MapImageInfo
{
	MapImageInfo();
	bool IsImageInBlock(int x, int y, MapImageLib *lib);
	char shortid[MAPIMAGE_SHORTID_CHARS];
	int x, y;

	bool hilighted;
};

//nimetty paikka johon voi vaikka viitata jostain
struct MapNamedPlace
{
	char name[MAPNAMEDPLACE_NAME_CHARS];
	XYPlace xy; //ruutuja
	int heading;
};

//talletuksessa käytettävät karsitut edelliset struktit

struct SaveMapBlock
{
	char tile_shortid[MAPBLOCK_SHORTID_CHARS];
	char move_shortid[MAPBLOCK_MOVE_SHORTID_CHARS];
	char y_change_per_10x_right;
	char special_info;
};

struct SaveMapImageInfo
{
	char shortid[MAPIMAGE_SHORTID_CHARS];
	int x, y;
};

class MapFloor
{
public:
	MapFloor();
	~MapFloor();

	void UnAllocate();
	void Allocate(int w, int h);
	
	MapBlock *GetBlock(int x, int y);
	void SetBlock(int x, int y, MapBlock);

	std::vector<MapImageInfo> & GetMapImageInfos();
	void AddMapImageInfo(MapImageInfo info);
	std::vector<int> GetImagesInBlock(int x, int y, MapImageLib *lib);
	void RemoveImages(std::vector<int> imgs);

	std::vector<MapNamedPlace> & GetNamedPlaces();
	void AddNamedPlace(MapNamedPlace place);
	std::vector<int> GetNamedPlacesInBlock(int x, int y);
	void RemoveNamedPlaces(std::vector<int> places);
	MapNamedPlace GetNamedPlace(std::string name);

	std::vector<XYPlace> GetBlockHilights();
	bool IsBlockHilighted(int x, int y);
	void ClearBlockHilights();
	void AddBlockHilight(int x, int y);
	void RemoveBlockHilight(int x, int y);
	void AddBlockHilightBlock(int x, int y, int x2, int y2);

	std::vector<int> GetImageHilights();
	bool IsImageHilighted(int i);
	void ClearImageHilights();
	void AddImageHilight(int i);
	void RemoveImageHilight(int i);

private:
	int mapwidth, mapheight;

	MapBlock **mapblocks;
	std::vector<MapImageInfo> imageinfos;
	std::vector<MapNamedPlace> namedplaces;
};

//tallennuksessa käytettäviä strukteja

struct MapFloorInfo
{
	unsigned int imagecount;
	unsigned int namedplacecount;
	unsigned int nothing1, nothing2, nothing3;
};

struct MapInfo
{
	unsigned int version;
	unsigned int width, height;
	unsigned int blockw, blockh;
	unsigned int floorcount;
	unsigned int objectcount;
	unsigned int nothing1, nothing2, nothing3;
};

class Map
{
public:
	Map();
	~Map();

	int GetW();
	int GetH();
	int GetBlockW();
	int GetBlockH();
	int GetPxW();
	int GetPxH();

	void SetBlockSize(int w, int h);

	int LoadFromFile(std::string filename);
	int WriteToFile(std::string filename);

	void Allocate(int w, int h);
	void Allocate(int w, int h, int bw, int bh);
	void UnAllocate();

	MapFloor* AddFloorTop();
	MapFloor* AddFloorBottom();
	bool RemoveFloorTop();
	bool RemoveFloorBottom();

	std::vector<MapFloor*>& GetFloors();
	unsigned int FloorCount();
	MapFloor* GetFloor(unsigned int i);

	std::vector<Object*> & GetObjects();
	void AddObject(Object *info);
	
private:
	int mapwidth, mapheight;
	int blockw, blockh;
	
	std::vector<MapFloor*> floors;
	std::vector<Object*> objects;
};

#endif

