#ifndef __OBJECTIMAGELIB_H_
#define __OBJECTIMAGELIB_H_

#include "image.h"
#include <vector>
#include <string>

/*

hakemisto kaikelle: data/objectimagelib/

index.txt:n formaatti:
- rivin formaatti: <shortid> : <W>x<H> : <yoffset>
  : <liikkumisanimaation framejen kestot>
  : <muiden animaatioiden framejen kestot>
- jos ei ole liikkumisanimaatiota, liikkumisanimaatioiden framejen
  kestojen tilalle nomove
- esim:
car1 : 64x64 : 0 : 100, 200, 150 : 200, 200, 200 : 0
car1big : 128x128 : 20 : nomove

- muita filuja tulisi tällöin hakemistossa olemaan:

tiedostonimen formaatti: <shortid>.png
car1.png
car1big.png

kuvatiedoston sisältö:

- kaikki käyttämätön tausta tehdään värillä 0,255,0,
  jotta voidaan tunnistaa animaatioiden framejen määrä

- väri 255,0,255 on läpinäkyvä

- 1. rivi: pääkuva joka suuntaan (vierekkäin: ylös, oikealle, alas, vasemmalle)

- 2. rivi: liikkumisanimaatio ylös, framet vierekkäin
- 3. rivi: oikealle
- 4. rivi: alas
- 5. rivi: vasemmalle

- 6. rivi: ensimmäinen lisäanimaatio, framet vierekkäin
- 6+n. rivi: muut lisäanimaatiot

- kuvan koko on siten minimissään (4 * leveys) x ((5 + lisäanimaatiot) * korkeus)

muuta:

- pelissä kuvat asetetaan aina horisontaalisesti keskelle objektin
  x-koordinaattia ja alareuna kohtaan y - yoffset

*/

struct ObjectImageChunk
{
	std::string shortid;
	Way4Image main_image;
	Way4Animation move_animation;
	std::vector<Animation> other_animations;
	int yoffset;
};

class ObjectImageLib
{
public:
	~ObjectImageLib();
	int Load(std::string path);
	void Clear();
	ObjectImageChunk GetChunk(std::string shortid);
private:
	std::vector<ObjectImageChunk> chunks;
};

#endif

