#ifndef __SCRIPT_H_
#define __SCRIPT_H_

#include <vector>
#include <string>
#include <list>
#include "variables.h"
#include "object.h"
#include "map.h"

class Map;
class Object;

struct ScriptRow
{
	int real_rownum;
	std::string rowid;
	std::string command;
	std::vector<std::string> args;
};

class Script;

struct ScriptInstance
{
	Script *script;

	int current_row;
	bool command_continue;
	int command_counter;

	void SetStringVariable(std::string name, std::string value);
	void SetIntVariable(std::string name, int value);
	
	std::string GetStringVariable(std::string name);
	int GetIntVariable(std::string name);

	std::string GetStringValue(std::string arg);
	int GetSingleIntValue(std::string arg);
	int GetIntValue(std::string arg);

	bool DoCondition(std::string condition);

private:
	Variables vars;
};

class Script
{
public:
	Script();
	~Script();
	int LoadFile(std::string file);
	int Init(Object * object, Variables *g, Map *map_, ObjectImageLib * objimglib);
	std::string GetFilename();
	int StartInstanceAtRowID(std::string id, std::vector<std::string> args);
	int RunInstance();
	std::vector<std::string> GetLastReturnValues();
	std::vector<std::string> RunSub(std::string rowid, std::vector<std::string> args);
	friend struct ScriptInstance;
private:
	void BadError(int row, std::string description);
	void BadError(std::string description);
	int FindRowID(std::string id);
	std::string filename;
	bool file_loaded;
	bool init_done;

	std::vector<std::string> lastreturnvalues;

	std::vector<ScriptRow> rows;
	
	std::list<ScriptInstance> instances;

	Variables vars;

	Object * object;
	Map * map;
	ObjectImageLib * imagelib;
	Variables * gvariables;
};

#endif

