#include "graphics.h"
#include "world.h"
#include "objectimagelib.h"
#include "tilelib.h"
#include "mapimagelib.h"
#include "variables.h"
#include <string>
#include "graphics.h"
#include "mapeditor.h"
#include "gui.h"
#include "gameviewer.h"
#include "filesys.h"
#include "gamegui.h"

#define SCREENW 800
#define SCREENH 600

int main(int argc, char **argv)
{
	if(SDL_Init(SDL_INIT_VIDEO) < 0){
		std::cout<<"unable to init sdl"<<std::endl;
		return 1;
	}
	
	std::string mapeditorfilename;
	if(argc==2){
		std::cout<<"two args"<<std::endl;
		mapeditorfilename = argv[1];
		std::cout<<"mapeditorfilename="<<mapeditorfilename<<std::endl;
	}
	
	SDL_Surface *screen;
	Fonts fonts;
	World world;
	ObjectImageLib objectimagelib;
	TileLib tilelib;
	TileLib movetilelib;
	MapImageLib mapimagelib;
	Variables *gvariables;
	MapEditor mapeditor;
	
	fonts.normal = NULL;
	fonts.small = NULL;
	fonts.quitesmall = NULL;
	fonts.big = NULL;
	gvariables = NULL;
	bool ttf_initialized = false;

	int w=SCREENW, h=SCREENH;
	screen = SDL_SetVideoMode(w, h, SCREENBPP, SDL_HWSURFACE | SDL_DOUBLEBUF);
	if(screen==NULL){
		std::cout<<"Unable to set "<<w<<"x"<<h<<" mode"<<std::endl;
		{std::cout<<"failed to initialize"<<std::endl; goto quit;}
	}

	if(objectimagelib.Load("data/objectimagelib")) {std::cout<<"failed to initialize"<<std::endl; goto quit;}

	if(tilelib.Load("data/tilelib")) {std::cout<<"failed to initialize"<<std::endl; goto quit;}
	
	if(movetilelib.Load("data/movetilelib")) {std::cout<<"failed to initialize"<<std::endl; goto quit;}
	
	if(mapimagelib.Load("data/mapimagelib")) {std::cout<<"failed to initialize"<<std::endl; goto quit;}
	
	if(TTF_Init() == -1)
	{
		std::cout<<"TTF_Init error: "<<TTF_GetError();
		{std::cout<<"failed to initialize"<<std::endl; goto quit;}
	}
	ttf_initialized = true;
	
	fonts.normal = TTF_OpenFont("Vera.ttf", 10);
	if(fonts.normal == NULL) {std::cout<<"failed to initialize"<<std::endl; goto quit;}
	fonts.small = TTF_OpenFont("Vera.ttf", 7);
	if(fonts.small == NULL) {std::cout<<"failed to initialize"<<std::endl; goto quit;}
	fonts.quitesmall = TTF_OpenFont("Vera.ttf", 9);
	if(fonts.quitesmall == NULL) {std::cout<<"failed to initialize"<<std::endl; goto quit;}
	fonts.big = TTF_OpenFont("Vera.ttf", 20);
	if(fonts.big == NULL) {std::cout<<"failed to initialize"<<std::endl; goto quit;}

	gvariables = new Variables("data/globalvariables");

	mapeditor.SetThings(screen, &fonts, &objectimagelib, &tilelib,
			&movetilelib, &mapimagelib, mapeditorfilename);

	mapeditor.Main();

quit:
	std::cout<<"quitting"<<std::endl;
	
	if(fonts.normal!=NULL) TTF_CloseFont(fonts.normal);
	if(fonts.small!=NULL) TTF_CloseFont(fonts.small);
	if(fonts.quitesmall!=NULL) TTF_CloseFont(fonts.quitesmall);
	if(fonts.big!=NULL) TTF_CloseFont(fonts.big);

	if(ttf_initialized) TTF_Quit();

	if(gvariables!=NULL) delete gvariables;

	SDL_Quit();
	return 0;
}

