#include "game.h"
#include "graphics.h"
//#include "mapeditor.h"
#include "gui.h"
#include "gameviewer.h"
#include "filesys.h"
#include "gamegui.h"
#include "confparser.h"
#include <iostream>
#include <cmath>
#include <fstream>

#define SCREENW 640
#define SCREENH 480

Game::Game()
{
	initialized_properly = false;
	ttf_initialized = false;
	fonts.normal = NULL;
	fonts.small = NULL;
	fonts.quitesmall = NULL;
	fonts.big = NULL;
	gvariables = NULL;


	ConfParser parser;
	if(!parser.LoadFile("titrascon.conf")){
		std::cout<<"ConfParser failed"<<std::endl;
		return;
	}
	bool use_fullscreen;
	if(!parser.GetBoolValue("use_fullscreen", use_fullscreen)){
		std::cout<<"Could not find config option"<<std::endl;
		return;
	}

	int w=SCREENW, h=SCREENH;
	screen = SDL_SetVideoMode(w, h, SCREENBPP, SDL_HWSURFACE | SDL_DOUBLEBUF
			| (use_fullscreen?SDL_FULLSCREEN:0));
	if(screen==NULL){
		std::cout<<"Unable to set "<<w<<"x"<<h<<" mode"<<std::endl;
		return;
	}

	if(objectimagelib.Load("data/objectimagelib"))
	{std::cout<<"error loading objectimagelib"<<std::endl; return;}

	if(tilelib.Load("data/tilelib")) 
	{std::cout<<"error loading tilelib"<<std::endl; return;}
	
	if(movetilelib.Load("data/movetilelib")) 
	{std::cout<<"error loading movetilelib"<<std::endl; return;}
	
	if(mapimagelib.Load("data/mapimagelib")) 
	{std::cout<<"error loading mapimagelib"<<std::endl; return;}
	
	if(TTF_Init() == -1)
	{
		std::cout<<"TTF_Init error: "<<TTF_GetError();
		return;
	}
	
	ttf_initialized = true;
	
	fonts.normal = TTF_OpenFont("Vera.ttf", 10);
	if(fonts.normal == NULL) 
	{std::cout<<"error opening fonts.normal"<<std::endl; return;}
	fonts.small = TTF_OpenFont("Vera.ttf", 7);
	if(fonts.small == NULL) 
	{std::cout<<"error opening fonts.small"<<std::endl; return;}
	fonts.quitesmall = TTF_OpenFont("Vera.ttf", 9);
	if(fonts.quitesmall == NULL) 
	{std::cout<<"error opening fonts.quitesmall"<<std::endl; return;}
	fonts.big = TTF_OpenFont("Vera.ttf", 20);
	if(fonts.big == NULL) 
	{std::cout<<"error opening fonts.big"<<std::endl; return;}
	//std::cout<<"font pointers: normal="<<(unsigned int)fonts.normal<<", small="<<(unsigned int)fonts.small<<", quitesmall="<<(unsigned int)fonts.quitesmall<<", big="<<(unsigned int)fonts.big<<std::endl;

	gvariables = new Variables("data/globalvariables");

	initialized_properly = true;
}

Game::~Game()
{
	std::cout<<"~Game()"<<std::endl;
	
	std::cout<<"closing fonts"<<std::endl;
	if(fonts.normal!=NULL) TTF_CloseFont(fonts.normal);
	if(fonts.small!=NULL) TTF_CloseFont(fonts.small);
	if(fonts.quitesmall!=NULL) TTF_CloseFont(fonts.quitesmall);
	if(fonts.big!=NULL) TTF_CloseFont(fonts.big);

	std::cout<<"TTF_Quitting"<<std::endl;
	if(ttf_initialized) TTF_Quit();
	else std::cout<<"(was not needed)"<<std::endl;

	std::cout<<"deleting gvariables"<<std::endl;
	if(gvariables!=NULL) delete gvariables;
	else std::cout<<"(was not needed)"<<std::endl;
}

int Game::RunGame(/*std::string mapeditorfilename*/)
{
	std::cout<<"RunGame()"<<std::endl;
	if(!initialized_properly){
		std::cout<<"not initialized properly, returning"<<std::endl;
		return -1;
	}

	/*MapEditor mapeditor(screen, &fonts, &objectimagelib, &tilelib,
			&movetilelib, &mapimagelib, mapeditorfilename);

	if(mapeditorfilename != ""){
		mapeditor.Main();
		return 0;
	}*/

	Gui gui(screen, &fonts, 0, 0, 0);

	GuiLabel *titlelabel = new GuiLabel("tlabel", "=== Titra's Consequences ===", fonts);
	gui.AddComponent(titlelabel, -1, 100);
	
	int menu_y = 140;
	int menu_w = 200;
	int button_h = 25;
	int i = menu_y;
	int y_spacing = 35;
	
	GuiTextButton *button;
	button = new GuiTextButton(menu_w, button_h, "game", "Game (test)", fonts);
	gui.AddComponent(button, -1, i+=y_spacing);
	/*button = new GuiTextButton(menu_w, button_h, "editor", "Map Editor", fonts);
	gui.AddComponent(button, -1, i+=y_spacing);*/
	button = new GuiTextButton(menu_w, button_h, "quit", "Quit", fonts);
	gui.AddComponent(button, -1, i+=y_spacing);

	int mousex=0, mousey=0;
	SDL_GetMouseState(&mousex, &mousey);
	int oldmousex=mousex, oldmousey=mousey;
	bool leftbutton_down = false, rightbutton_down = false;
	int mousemovex, mousemovey;

	bool quitgame = false;
	while(!quitgame){
		SDL_Delay(1000/50);
		
		//SDL:n eventit

		mousemovex = 0;
		mousemovey = 0;
		
		SDL_Event sdlevent;
		while(SDL_PollEvent(&sdlevent) && !quitgame){
			if(sdlevent.type == SDL_QUIT){
				quitgame = true;
			}
			if(sdlevent.type == SDL_KEYUP){
				switch(sdlevent.key.keysym.sym){
					case SDLK_ESCAPE:
						quitgame = true;
						break;
					case SDLK_q:
						quitgame = true;
						break;
					default:
						break;
				}
			}
			if(sdlevent.type == SDL_MOUSEMOTION){
				oldmousex = mousex;
				oldmousey = mousey;
				mousex = sdlevent.motion.x;
				mousey = sdlevent.motion.y;
				mousemovex += mousex - oldmousex;
				mousemovey += mousey - oldmousey;
			}
			if(sdlevent.type == SDL_MOUSEBUTTONDOWN){
				if(sdlevent.button.button == SDL_BUTTON_LEFT){
					leftbutton_down = true;
				}
				if(sdlevent.button.button == SDL_BUTTON_RIGHT){
					rightbutton_down = true;
				}
			}
			if(sdlevent.type == SDL_MOUSEBUTTONUP){
				if(sdlevent.button.button == SDL_BUTTON_LEFT){
					leftbutton_down = false;
				}
				if(sdlevent.button.button == SDL_BUTTON_RIGHT){
					rightbutton_down = false;
				}
			}
		}

		//syötetään guille hiiren tiedot
		gui.Mouse(mousex, mousey, mousemovex, mousemovey, leftbutton_down, rightbutton_down);

		//Gui:n eventit
		
		GuiEvent guievent;
		while(gui.GetEvent(&guievent)){
			if(guievent.type == GUIEVENT_BUTTONCLICK){
				if(guievent.cnames[0]=="game"){
					if(GameLoop()) quitgame = true;
				}
				/*if(guievent.cnames[0]=="editor"){
					if(mapeditor.Main()) quitgame = true;
				}*/
				if(guievent.cnames[0]=="quit"){
					quitgame = true;
				}
			}
		}

		gui.Draw(0, 0, 800, 600);

		SDL_Flip(screen);
	}
	return 0;
}

int Game::GameLoop()
{
	std::cout<<"Game::GameLoop()"<<std::endl;
	bool run = true;
	int retval = 0;

	//tausta varmuuden vuoksi mustaksi

	Uint32 blackcolor = SDL_MapRGB(screen->format, 0, 0, 0);

	GDrawRect(screen, 0, 0, SCREENW, SCREENH, blackcolor);
	SDL_Flip(screen);

	//kartta
	
	//std::string mapfile = "data/maps/testmap0/mapdata";
	std::string mapfile = "data/maps/testmap1/mapdata";
	
	Map map;

	if(map.LoadFromFile(mapfile)){
		std::cout<<"loading map from \""<<mapfile<<"\" failed"<<std::endl;
		return -1;
	}

	//pelaaja

	Player player;
	
	PlaceInfo p;
	p.xy.x = 200;
	p.xy.y = 650;
	p.heading = HEADING_S;
	p.floor = 0;
	player.SetPlaceInfo(p);

	ObjectImageChunk c = objectimagelib.GetChunk("man");
	if(c.shortid == ""){
		std::cout<<"could not get imagechunk \"man\""<<std::endl;
		return -1;
	}
	player.SetImageChunk(c);
	
	map.AddObject(&player);

	//objektit
	
	std::vector<Object*> objects;
	
	Object *object = new Object;
	
	Script *script = new Script();
	if(script->LoadFile("testscript")){
		run = false;
		retval = 1;
	}
	script->Init(object, gvariables, &map, &objectimagelib);

	/*std::vector<std::string> args;
	args.push_back("a");
	args.push_back("b");
	std::vector<std::string> retvals = script->RunSub("testsub", args);
	for(unsigned int i=0; i<retvals.size(); i++){
		std::cout<<"retval: "<<retvals[i]<<std::endl;
	}*/
	
	object->SetScript(script);

	map.AddObject(object);
	objects.push_back(object);

	/*for(int i=0; i<100; i++){
		Object *object = new Object;
		
		PlaceInfo p;
		p.xy.x = (400+i*45)%800;
		if(p.xy.x < 0) p.xy.x = 0 - p.xy.x;
		p.xy.y = (150+i*47)%600;
		if(p.xy.y < 0) p.xy.y = 0 - p.xy.y;
		p.heading = HEADING_S;
		p.floor = 1;
		object->SetPlaceInfo(p);

		object->SetImageChunk(c);

		object->SetCurrentImageID(1);

		map.AddObject(object);
		objects.push_back(object);
	}*/

	//jotain muuttujia

	int view_map_x_offset = 0, view_map_y_offset = 0;
	bool key_up=false, key_down=false, key_left=false, key_right=false;
	Heading playerheading;
	playerheading = HEADING_S;

	//mainloop

	int lastticks = SDL_GetTicks();;
	int wanted_frame_time = 1000/30;
	while(run){
		int ticksnow = SDL_GetTicks();
		if(ticksnow < lastticks) lastticks = ticksnow;
		int lasttime = ticksnow - lastticks;
		int timetosleep = wanted_frame_time - lasttime;
		int lastframetime = lasttime;
		if(timetosleep > 0){
			SDL_Delay(timetosleep);
			lastframetime+=timetosleep;
		}
		lastticks = SDL_GetTicks();
		
		SDL_Event sdlevent;
		while(SDL_PollEvent(&sdlevent) && run){
			if(sdlevent.type == SDL_QUIT){
				run = false;
				retval = 1;
			}
			if(sdlevent.type == SDL_KEYDOWN){
				switch(sdlevent.key.keysym.sym){
				/////////
				case SDLK_UP:
					key_up = true;
					break;
				case SDLK_DOWN:
					key_down = true;
					break;
				case SDLK_LEFT:
					key_left = true;
					break;
				case SDLK_RIGHT:
					key_right = true;
					break;
				/////////
				default:
					break;
				}
			}
			if(sdlevent.type == SDL_KEYUP){
				switch(sdlevent.key.keysym.sym){
				/////////
				case SDLK_UP:
					key_up = false;
					break;
				case SDLK_DOWN:
					key_down = false;
					break;
				case SDLK_LEFT:
					key_left = false;
					break;
				case SDLK_RIGHT:
					key_right = false;
					break;
				/////////
				case SDLK_ESCAPE:
					run = false;
					break;
				case SDLK_q:
					run = false;
					break;
				default:
					break;
				}
			}
		}

		//tehdään juttuja eventtien mukaan
		//std::cout<<"-- player --"<<std::endl;
		//TODO: öö toimiikohan tämä nyt ihan noilla tarkkuuksilla?
		int vert_change = 0, horiz_change = 0;
		int px_per_s = 200;
		if(key_up)    vert_change  -= (lastframetime * px_per_s) / 1000;
		if(key_down)  vert_change  += (lastframetime * px_per_s) / 1000;
		if(key_left)  horiz_change -= (lastframetime * px_per_s) / 1000;
		if(key_right) horiz_change += (lastframetime * px_per_s) / 1000;
		
		if(vert_change != 0 || horiz_change != 0){
			if(vert_change < 0){
				if(horiz_change < 0) playerheading = HEADING_NW;
				else if(horiz_change > 0) playerheading = HEADING_NE;
				else playerheading = HEADING_N;
			}
			else if(vert_change > 0){
				if(horiz_change < 0) playerheading = HEADING_SW;
				else if(horiz_change > 0) playerheading = HEADING_SE;
				else playerheading = HEADING_S;
			}
			else if(horiz_change < 0) playerheading = HEADING_W;
			else if(horiz_change > 0) playerheading = HEADING_E;
			player.SetCurrentImageID(1);
		}
		else{
			player.SetCurrentImageID(0);
		}

		player.GetPlaceInfo()->heading = playerheading;
		
		MapFloor *currentfloor = map.GetFloor(player.GetPlaceInfo()->floor);

		//bool was_on_anyfloortile = currentfloor->

		player.GetPlaceInfo()->xy.y += vert_change;
		player.GetPlaceInfo()->xy.x += horiz_change;

		//std::cout<<"--/player --"<<std::endl;

		//otetaan nykyinen aika

		int timenow_ms = SDL_GetTicks();

		//suoritetaan objektien taskeja

		for(unsigned int i=0; i<objects.size(); i++){
			/*std::cout<<"object "<<i<<": "<<objects[i]->GetPlaceInfo()->xy.x<<","
					<<objects[i]->GetPlaceInfo()->xy.y<<std::endl;*/
			Task *task = objects[i]->GetTask();
			switch(task->type){
			case TASKTYPE_NO_TASK:
			{
				//no task set, continuing script to get task again
				//std::cout<<"---- running script ----"<<std::endl;

				int a = objects[i]->GetScript()->RunInstance();
				if(a == 3){
					std::cout<<"script error"<<std::endl;
					retval = 1;
					goto clean_and_quit;
				}
				else if(a == 2){
					std::cout<<"Script ran out of instances, this shouldn't happen."
							<<std::endl<<"If script has to do nothing, it has to be"
							" set to do the TASKTYPE_DO_NOTHING_FOREVER task."
							<<std::endl;
					retval = 1;
					goto clean_and_quit;
				}
				//std::cout<<"-- end running script --"<<std::endl;
				break;
			}
			case TASKTYPE_DO_NOTHING_FOREVER:
				//doing nothing, so we do nothing 8)
				break;
			case TASKTYPE_WAIT:
				if(task->starttime == -1){
					task->starttime = timenow_ms;
				}
				else if(timenow_ms > task->starttime + task->length){
					task->Clear();
				}
				break;
			case TASKTYPE_MOVE:
			{
				int move_waittime = wanted_frame_time;
				if(task->starttime == -1){
					task->starttime = timenow_ms;
					task->lasttime = timenow_ms;
					task->startplace = objects[i]->GetPlaceInfo()->xy;
					//jos paikka on sama, ei tehdä mitään
					if(task->startplace.x == task->targetplace.x
							&& task->startplace.y == task->targetplace.y){
						std::cout<<"move: distance 0, cancelling"<<std::endl;
						task->Clear();
						break;
					}
				}

				while(task->lasttime + move_waittime < timenow_ms){
					task->lasttime += move_waittime;

					//std::cout<<"--- move "<<task->lasttime<<" ---"<<std::endl;

					/*std::cout<<"start: "<<task->startplace.x<<","
							<<task->startplace.y<<" end: "
							<<task->targetplace.x<<","
							<<task->targetplace.y
							<<" speed = "<<task->speed<<std::endl;*/
					
					double all_x_change = task->targetplace.x - task->startplace.x;
					double all_y_change = task->targetplace.y - task->startplace.y;
					double distance = sqrt(pow(all_x_change, 2) + pow(all_y_change, 2));
					double move_per_ms = (double)task->speed / 1000.0;
					/*std::cout<<"all_x_change = "<<all_x_change<<" all_y_change="<<all_y_change
							<<std::endl<<" distance="<<distance
							<<" move_per_ms = "<<move_per_ms<<std::endl;*/
					double time_moved = (double)(task->lasttime - task->starttime);
					double distance_moved = move_per_ms * time_moved;
					bool endofmove = false;
					if(distance_moved >= distance){
						endofmove = true;
						distance_moved = distance;
					}
					double moved = distance_moved / distance;
					/*std::cout<<" time_moved = "<<time_moved<<" distance_moved = "
							<<distance_moved<<" moved = "<<moved<<std::endl;*/
					double x_change = all_x_change * moved;
					double y_change = all_y_change * moved;
					//std::cout<<"x_change = "<<x_change<<" y_change = "<<y_change<<std::endl;
					XYPlace place_now;
					place_now.x = task->startplace.x + (int)x_change;
					place_now.y = task->startplace.y + (int)y_change;
					//std::cout<<"place_now: "<<place_now.x<<","<<place_now.y<<std::endl;
					objects[i]->GetPlaceInfo()->xy = place_now;
					//std::cout<<"--- /move ---"<<std::endl;
					if(endofmove){
						//std::cout<<"end of move"<<std::endl;
						task->Clear();
						break;
					}
				}
				break;
			}
			default:
				std::cout<<"ERROR: Task type "<<task->type<<" not recognized"<<std::endl;
				retval = 1;
				goto clean_and_quit;
			}
		}
		
		//rullaillaan animaatioita

		//mapimaget
		mapimagelib.RollAnims(timenow_ms);
		
		//objektien imagechunkit
		std::vector<Object*> objects = map.GetObjects();
		{//vc6
		for(unsigned int i=0; i < objects.size(); i++){
			objects[i]->RollAnim(timenow_ms);
		}
		}//vc6

		//siirretään ruutu kivasti

		int view_w=SCREENW, view_h=SCREENH;

		XYPlace playerplace = player.GetPlaceInfo()->xy;
		view_map_x_offset = playerplace.x - view_w/2;
		view_map_y_offset = playerplace.y - view_h/2;
		if(view_w < map.GetPxW()){
			if(view_map_x_offset < 0) view_map_x_offset = 0;
			if(view_map_x_offset + view_w > map.GetPxW())
				view_map_x_offset = map.GetPxW() - view_w;
		}
		if(view_h < map.GetPxH()){
			if(view_map_y_offset < 0) view_map_y_offset = 0;
			if(view_map_y_offset + view_h > map.GetPxH())
				view_map_y_offset = map.GetPxH() - view_h;
		}


		//piirrellään jutut
		
		DrawGame(screen,
				0, 0, view_w, view_h,
				view_map_x_offset, view_map_y_offset,
				&map, &objectimagelib,
				&tilelib, &mapimagelib,
				fonts, 0, -1);

		SDL_Flip(screen);
	}

clean_and_quit:
	
	//vapautellaan objektien skriptit ja objektit itse
	{//vc6
	for(unsigned int i=0; i<objects.size(); i++){
		delete objects[i]->GetScript();
		delete objects[i];
	}
	}//vc6
	
	//tyhjätään ruutu
	GDrawRect(screen, 0, 0, SCREENW, SCREENH, blackcolor);
	SDL_Flip(screen);
	
	return retval;
}

