#include "gameviewer.h"
#include <iostream>
#include <string.h>
#include <stdio.h>

enum{
	WHAT_IMAGE,
	WHAT_OBJECT
};

struct ImgOrObject{ int what; unsigned int num; };

void BlockPlaceOnScreen(int block_map_x, int block_map_y,
		int mapfirstx, int mapfirsty,
		int view_map_dist_x, int view_map_dist_y,
		int screen_x, int screen_y,
		int block_w, int block_h,
		int &block_screen_x, int &block_screen_y)
{
	//blokin x näkyvällä kartanpalalla
	int x = block_map_x - mapfirstx;
	int y = block_map_y - mapfirsty;

	//ruudun x ja y pikseleinä näkymällä
	int block_view_x = x * block_w - view_map_dist_x;
	int block_view_y = y * block_h - view_map_dist_y;

	//ruudun x ja y pikseleinä näytöllä
	block_screen_x = screen_x + block_view_x;
	block_screen_y = screen_y + block_view_y;
}

void DrawGame(SDL_Surface *surface,
		int screen_x, int screen_y,
		int view_w, int view_h,
		int view_map_x_offset, int view_map_y_offset,
		Map *map, ObjectImageLib *objectimagelib,
		TileLib *tilelib, MapImageLib *mapimagelib,
		Fonts fonts, int whattodraw, int floor_to_draw)
{
	Uint32 color;
	bool drawgrid = whattodraw & DGDRAW_GRID;
	//bool drawonlythisfloor = whattodraw & DGDRAW_ONLYTHISFLOOR;
	bool drawnamedplaces = whattodraw & DGDRAW_NAMEDPLACES;
	
	//otetaan kartalta muuttujia
	int map_w = map->GetW();
	int map_h = map->GetH();
	/*int map_w_px = map->GetPxW();
	int map_h_px = map->GetPxH();*/
	int block_w = map->GetBlockW();
	int block_h = map->GetBlockH();

	//tyhjätään tausta (TODO: optimoi)
	color = SDL_MapRGB(surface->format, 30,15,60);
	GDrawRect(surface, screen_x, screen_y, view_w, view_h, color);
	/*color = SDL_MapRGB(surface->format, 0,0,0);
	GDrawAlphaRectArea(surface, screen_x - view_map_x_offset,
			screen_y - view_map_y_offset, map_w_px, map_h_px,
			screen_x, screen_y, view_w, view_h, color, 255);*/

	//lasketaan näkymän leveyden viemä määrä ruutuja
	int view_w_blocks = view_w / block_w + 2;
	
	//lasketaan näkymän korkeuden viemä määrä ruutuja
	int view_h_blocks = view_h / block_h + 2;
	
	//lasketaan näkymän ylä- ja vasemman reunan etäisyys pikseleinä kartan
	//ensimmäisistä vastaavista ruuduista
	int view_map_dist_x = view_map_x_offset % block_w;
	int view_map_dist_y = view_map_y_offset % block_h;
	
	//lasketaan ensimmäinen kartalta käsittelyyn otettava x ja y
	int mapfirstx = view_map_x_offset / block_w;
	int mapfirsty = view_map_y_offset / block_h;
	
	//kerrokset ja objektit
	
	std::vector<MapFloor*> floors = map->GetFloors();
	std::vector<Object*> objects = map->GetObjects();
	unsigned int objects_added_count = 0;
	
	//loopataan kerrokset läpi (tai ainoastaan yksi jos niin on haluttu)
	
	unsigned int from=0, to=floors.size()-1;
	if(floor_to_draw != -1){
		from = floor_to_draw;
		to = floor_to_draw;
	}
	for(unsigned int fi = from; fi <= to; fi++){
		MapFloor *floor = floors[fi];

		//piirretään tilet
		
		for(int y = 0; y < view_h_blocks; y++){
			for(int x = 0; x < view_w_blocks; x++){
				int block_map_x = mapfirstx + x;
				int block_map_y = mapfirsty + y;

				if(block_map_x < 0 || block_map_y < 0) continue;
				if(block_map_x >= map_w || block_map_y >= map_h) continue;

				int block_screen_x, block_screen_y;
				BlockPlaceOnScreen(block_map_x, block_map_y,
						mapfirstx, mapfirsty,
						view_map_dist_x, view_map_dist_y,
						screen_x, screen_y,
						block_w, block_h,
						block_screen_x, block_screen_y);

				MapBlock *block = floor->GetBlock(block_map_x, block_map_y);
						
				std::string tile_shortid = block->tile_shortid;
				if(tile_shortid != ""){
					SDL_Surface *tileimage = tilelib->GetImage(tile_shortid);
					if(tileimage != NULL){
						GDrawIMGArea(surface, tileimage,
								block_screen_x, block_screen_y,
								screen_x, screen_y,
								view_w, view_h);
					}
					else{
						//tileä ei löydy kirjastosta, piirretään punainen neliö
						Uint32 color;
						color = SDL_MapRGB(surface->format, 100, 0, 0);
						GDrawAlphaRectArea(surface, block_screen_x, block_screen_y,
								block_w, block_h,
								screen_x, screen_y, view_w, view_h,
								color, 255);
					}
				}
				else{
					//ei ole tileä, piirretään musta neliö jos on alin kerros
					if(fi==from){
						Uint32 color;
						color = SDL_MapRGB(surface->format, 0, 0, 0);
						GDrawAlphaRectArea(surface, block_screen_x, block_screen_y,
								block_w, block_h,
								screen_x, screen_y, view_w, view_h,
								color, 255);
					}
				}
				
				//piirretään hilighti jos tarvitsee
				if(block->hilighted){
					Uint32 color = SDL_MapRGB(surface->format, 255, 0, 255);
					GDrawAlphaRectArea(surface, block_screen_x, block_screen_y,
							block_w, block_h,
							screen_x, screen_y, view_w, view_h,
							color, 64);
					}
				}
		}

		//mapimageinfot
		
		std::vector<MapImageInfo> mapimageinfos = floor->GetMapImageInfos();

		//keksitään järjestys jossa ne piirretään

		std::vector<ImgOrObject> draw_order;
		{//vc6
		for(unsigned int i=0; i<mapimageinfos.size() + 1; i++){
			int map_backswapy = 0;
			bool lastloop = (i == mapimageinfos.size());
			if(!lastloop){
				MapImageInfo imageinfo = mapimageinfos[i];
				MapImage *mapimage = mapimagelib->GetImage(imageinfo.shortid);
				int map_y = imageinfo.y * block_h;
				int backswapy = mapimage->GetBackSwapY();
				map_backswapy = map_y + backswapy;
			}
			//std::cout<<"loop "<<i<<" "<<(lastloop?"is lastloop":"")<<std::endl;

			//lisäillään objektit jotka on tässä kerroksessa ja kuvaa ennen

			std::vector<ImgOrObject> draw_order_temp_unsorted;

			for(unsigned int j=0; j<objects.size(); j++){
				PlaceInfo *p = objects[j]->GetPlaceInfo();
				if(p->floor != fi) continue;
				if(p->xy.y < map_backswapy || lastloop){
					//std::cout<<"checking object "<<j<<"... ";
					bool found = false;
					for(unsigned int k=0; k<draw_order.size(); k++){
						if(draw_order[k].what == WHAT_OBJECT
								&& draw_order[k].num == j){
							found = true;
							break;
						}
					}
					//std::cout<<(found?"found, not adding":"not found, adding")<<std::endl;
					if(!found){
						ImgOrObject ioo;
						ioo.what = WHAT_OBJECT;
						ioo.num = j;
						draw_order_temp_unsorted.push_back(ioo);
						objects_added_count++;
					}
				}
			}
			
			while(!draw_order_temp_unsorted.empty()){
				int smallest_y = -1, smallest_y_index = -1;
				bool first_added = false;
				for(unsigned int j=0; j<draw_order_temp_unsorted.size(); j++){
					int num = draw_order_temp_unsorted[j].num;
					//std::cout<<"num = "<<num<<std::endl;
					PlaceInfo *p = objects[num]->GetPlaceInfo();
					if(p->xy.y < smallest_y || !first_added){
						smallest_y = p->xy.y;
						smallest_y_index = j;
						first_added = true;
					}
				}
				//std::cout<<"adding object "<<draw_order_temp_unsorted[smallest_y_index].num
				//		<<" to draw_order"<<std::endl;
				draw_order.push_back(draw_order_temp_unsorted[smallest_y_index]);
				draw_order_temp_unsorted.erase(
						draw_order_temp_unsorted.begin() + smallest_y_index, 
						draw_order_temp_unsorted.begin() + smallest_y_index + 1);
			}
			
			if(!lastloop){
				ImgOrObject ioo;
				ioo.what = WHAT_IMAGE;
				ioo.num = i;
				draw_order.push_back(ioo);
			}
		}
		}//vc6
		//piirretään mapimageiden pohjat joka tapauksessa ensin
		{//vc6
		for(unsigned int i = 0; i < mapimageinfos.size(); i++){
			//kaivellaan info MapImagelle
			MapImageInfo info = mapimageinfos[i];
			
			//kaivellaan kirjastosta infon mukaan MapImage ja sen
			//pohjan kuva tällä hetkellä
			MapImage *image = mapimagelib->GetImage(info.shortid);
			SDL_Surface *under = image->GetUnderAnim().GetCurrentImage();
			
			int block_screen_x, block_screen_y;
			BlockPlaceOnScreen(info.x, info.y,
					mapfirstx, mapfirsty,
					view_map_dist_x, view_map_dist_y,
					screen_x, screen_y,
					block_w, block_h,
					block_screen_x, block_screen_y);

			GDrawIMGArea(surface, under, block_screen_x, block_screen_y,
					screen_x, screen_y, view_w, view_h); 
		}
		}//vc6
		//piirretään objektit ja mapimageiden päällykset järjestyksessä
		{//vc6
		//std::cout<<"draw_order:"<<std::endl;
		for(unsigned int i=0; i<draw_order.size(); i++){
			//std::cout<<draw_order[i].what<<", "<<draw_order[i].num<<std::endl;
			int num = draw_order[i].num;
			if(draw_order[i].what == WHAT_IMAGE){
				//kaivellaan info MapImagelle
				MapImageInfo info = mapimageinfos[num];
				
				//kaivellaan kirjastosta infon mukaan MapImage ja sen
				//over-kuva tällä hetkellä
				MapImage *image = mapimagelib->GetImage(info.shortid);
				SDL_Surface *over = image->GetOverAnim().GetCurrentImage();
				
				int block_screen_x, block_screen_y;
				BlockPlaceOnScreen(info.x, info.y,
						mapfirstx, mapfirsty,
						view_map_dist_x, view_map_dist_y,
						screen_x, screen_y,
						block_w, block_h,
						block_screen_x, block_screen_y);

				GDrawIMGArea(surface, over, block_screen_x, block_screen_y,
						screen_x, screen_y, view_w, view_h); 

				if(info.hilighted){
					Uint32 color = SDL_MapRGB(surface->format, 255, 0, 255);
					GDrawAlphaRectArea(surface, block_screen_x, block_screen_y,
							over->w, over->h,
							screen_x, screen_y, view_w, view_h,
							color, 64);
				}
			}
			else if(draw_order[i].what == WHAT_OBJECT){
				PlaceInfo *p = objects[num]->GetPlaceInfo();
				SDL_Surface *image = objects[num]->GetCurrentImage();
				int yoffset = objects[num]->GetImageChunk()->yoffset;
				if(image==NULL){
					//std::cout<<"WARNING: object image NULL"<<std::endl;
					continue;
				}
				GDrawIMGArea(surface, image,
						screen_x - view_map_x_offset + p->xy.x - image->w/2,
						screen_y - view_map_y_offset + p->xy.y + yoffset - image->h,
						screen_x, screen_y,
						view_w, view_h);
			}
		}
		}//vc6
		//piirretään floor-id:t jos on käsketty
		
		//piirretään nimetyt paikat (MapNamedPlace) jos on käsketty
		
		if(drawnamedplaces){
			std::vector<MapNamedPlace> namedplaces = floor->GetNamedPlaces();
			for(unsigned int i=0; i<namedplaces.size(); i++){
				MapNamedPlace place = namedplaces[i];

				int block_screen_x, block_screen_y;
				BlockPlaceOnScreen(place.xy.x, place.xy.y,
						mapfirstx, mapfirsty,
						view_map_dist_x, view_map_dist_y,
						screen_x, screen_y,
						block_w, block_h,
						block_screen_x, block_screen_y);

				char namedplacetext[MAPNAMEDPLACE_NAME_CHARS + 16];
				sprintf(namedplacetext, "(h=%i) %s",
						place.heading, place.name);

				int textw, texth;
				TTF_SizeText(fonts.quitesmall, namedplacetext, &textw, &texth);

				if(block_screen_x >= screen_x && block_screen_y >= screen_y
						&& block_screen_x + textw < screen_x + view_w
						&& block_screen_y + texth < screen_y + view_h){
					color = SDL_MapRGB(surface->format, 100, 0, 0);
					GDrawAlphaRectArea(surface, block_screen_x,
							block_screen_y + TTF_FontHeight(fonts.small)*2,
							textw, texth,
							screen_x, screen_y, view_w, view_h,
							color, 255);
					GDrawText(surface, block_screen_x,
							block_screen_y + TTF_FontHeight(fonts.small)*2,
							fonts.quitesmall, namedplacetext, 255,255,255);
				}
			}
			int textw, texth;
			TTF_SizeText(fonts.normal, "Drawing NamedPlaces", &textw, &texth);
			color = SDL_MapRGB(surface->format, 0, 0, 0);
			GDrawAlphaRectArea(surface, screen_x, screen_y,
					textw, texth,
					screen_x, screen_y, view_w, view_h,
					color, 255);
			GDrawText(surface, screen_x, screen_y,
					fonts.normal, "Drawing NamedPlaces", 255,100,100);
		}
	}//floor-loop

	if(objects_added_count != objects.size()){
		std::cout<<"WARNING: different count of objects added than in object-list"<<std::endl;
	}
	
	//piirretään ruudukko jos on käsketty
	if(drawgrid){
		color = SDL_MapRGB(surface->format, 255, 255, 255);
		for(int y = -view_map_y_offset; y < view_h
				&& y + view_map_y_offset < map_h * block_h + block_h; y += block_h){
			if(y<0 || y>=view_h) continue;
			GDrawAlphaRect(surface, screen_x, screen_y + y, view_w, 1, color, 64);
		}
		for(int x = -view_map_x_offset; x < view_w 
				&& x + view_map_x_offset < map_w * block_w + block_w; x += block_w){
			if(x<0 || x>=view_w) continue;
			GDrawAlphaRect(surface, screen_x + x, screen_y, 1, view_h, color, 64);
		}
	}
}


