#include "graphics.h"
#include <iostream>
#include "string.h"

ViewableRow::ViewableRow()
{
	hilight_length = 0;
	color.r = 255;
	color.g = 255;
	color.b = 255;
}

ViewableRow::ViewableRow(std::string s)
{
	hilight_length = 0;
	color.r = 255;
	color.g = 255;
	color.b = 255;
	text = s;
}

void ViewableRow::SetHilight(int start, int length)
{
	hilight_start = start;
	hilight_length = length;
}


void GSlock(SDL_Surface *s)
{
  if ( SDL_MUSTLOCK(s) )
  {
	if ( SDL_LockSurface(s) < 0 )
	{
	  return;
	}
  }
}

void GSulock(SDL_Surface *s)
{
  if ( SDL_MUSTLOCK(s) )
  {
	SDL_UnlockSurface(s);
  }
}

void GDrawPixel(SDL_Surface *surface, int x, int y, Uint32 color)
{
	if(SCREENBPP==16){
		Uint16 *bufp;
		bufp = (Uint16 *)surface->pixels + y*surface->pitch/2 + x;
		*bufp = color;
	}
	else if(SCREENBPP==32){
		Uint32 *bufp;
		bufp = (Uint32 *)surface->pixels + y*surface->pitch/4 + x;
		*bufp = color;
	}
}

Uint32 GGetPixel(SDL_Surface *surface, int x, int y)
{
	return *((Uint32 *)surface->pixels + y*surface->pitch/4 + x);
}

bool IsPixelColor(SDL_Surface *surface, int x, int y,
		Uint8 r, Uint8 g, Uint8 b)
{
	Uint32 color = SDL_MapRGB(surface->format, r, g, b);
	return GGetPixel(surface, x, y) == color;
}

bool IsBlockSingleColoured(SDL_Surface *surface, int sx, int sy,
		int w, int h, Uint8 r, Uint8 g, Uint8 b)
{
	Uint32 color = SDL_MapRGB(surface->format, r, g, b);
	for(int y = 0; y < h; y++){
		for(int x = 0; x < w; x++){
			if(GGetPixel(surface, sx, sy) != color) return false;
		}
	}
	return true;
}


void GDrawRect(SDL_Surface *surface, int tox, int toy, int w, int h, Uint32 color)
{
	//onkohan tämä vähän optimaalisempi? :P
	if(w<=0 || h<=0) return;
	Uint32 *bufp, *bufp2, *bufp3; 
	bufp = (Uint32 *)surface->pixels + (toy * surface->pitch / 4) + tox;
	bufp2 = bufp;
	for(int x=0; x<w; x++){
		*bufp = color;
		bufp++;
	}
	for(int y=1; y<h; y++){
		bufp3 = (Uint32 *)surface->pixels + ((toy + y) * surface->pitch / 4) + tox;
		memcpy(bufp3, bufp2, w*sizeof(Uint32));
	}
	/*Uint32 *bufp;
	for(int y=0; y<h; y++){
		bufp = (Uint32 *)surface->pixels + ((toy + y) * surface->pitch / 4) + tox;
		for(int x=0; x<w; x++){
			*bufp = color;
			bufp++;
		}
	}*/
}

void GDrawAlphaRect(SDL_Surface *surface, int tox, int toy, int w, int h,
		Uint32 color, Uint8 ca)
{
	if(ca==0) return;
	if(ca==255) GDrawRect(surface, tox, toy, w, h, color);
	//std::cout<<"GDrawAlphaRect(SDL_Surface *surface, int tox="<<tox<<", int toy="<<toy<<", int w="<<w<<", int h="<<h<<", Uint32 color, Uint8 ca)"<<std::endl;
	int cr, cg, cb;
	//int ca;
	int cai;
	cr = (color>>0) & 0xff;
	cg = (color>>8) & 0xff;
	cb = (color>>16) & 0xff;
	//ca = (color>>24) & 0xff;
	cai = 255 - ca;
	//std::cout<<"ca="<<ca<<" cai="<<cai<<std::endl;
	int r, g, b;
	Uint32 *bufp;
	for(int y=0; y<h; y++){
		bufp = (Uint32 *)surface->pixels + ((toy + y) * surface->pitch / 4) + tox;
		for(int x=0; x<w; x++){
			r = (*bufp>>0) & 0xff;
			g = (*bufp>>8) & 0xff;
			b = (*bufp>>16) & 0xff;
			r = (r*cai + cr*ca)/256;
			g = (g*cai + cg*ca)/256;
			b = (b*cai + cb*ca)/256;
			*bufp = (r<<0) + (g<<8) + (b<<16)/* + (a<<24)*/;
			bufp++;
		}
	}
}

void GDrawAlphaRectArea(SDL_Surface *dest_surface, int x, int y, int w, int h,
		int minx, int miny, int areaw, int areah, Uint32 color, Uint8 ca)
{
	//std::cout<<"GDrawAlphaRectArea(SDL_Surface *surface, int x="<<x<<", int y="<<y<<", int w="<<w<<", int h="<<h<<", Uint32 color, Uint8 ca)"<<std::endl;
	if(w<0 || h<0) return;

	//tutkitaan paljonko reunat menevät yli

	int tooup, toolow, tooleft, tooright;
	if((tooup = miny - y) < 0) tooup = 0;
	if((toolow = y + h - miny - areah) < 0) toolow = 0;
	if((tooleft = minx - x) < 0) tooleft = 0;
	if((tooright = x + w - minx - areaw) < 0) tooright = 0;
	
	//jos kuvasta näkyy ylipäätänsä mitään
	if(tooup < h && toolow < h
			&& tooleft < w && tooright < w){
		GDrawAlphaRect(dest_surface,
				x + tooleft,
				y + tooup,
				w - tooleft - tooright,
				h - tooup - toolow,
				color, ca);
	}
}

void GDrawIMG(SDL_Surface *dest_surface, SDL_Surface *img, int x, int y)
{
	SDL_Rect dest;
	dest.x = x;
	dest.y = y;
	SDL_BlitSurface(img, NULL, dest_surface, &dest);
}

void GDrawIMG(SDL_Surface *dest_surface, SDL_Surface *img, int x, int y,
		int w, int h, int x2, int y2)
{
	/*std::cout<<"GDrawIMG(SDL_Surface*, SDL_Surface*, "
			<<x<<", "<<y<<", "<<w<<", "<<h<<", "<<x2<<", "<<y2<<")"<<std::endl;*/
	SDL_Rect dest2;
	dest2.x = x2;
	dest2.y = y2;
	dest2.w = w;
	dest2.h = h;
	SDL_Rect dest;
	dest.x = x;
	dest.y = y;
	SDL_BlitSurface(img, &dest2, dest_surface, &dest);
}

void GDrawIMGArea(SDL_Surface *dest_surface, SDL_Surface *img, int x, int y,
		int w, int h, int x2, int y2,
		int minx, int miny, int areaw, int areah)
{
	if(w<0 || h<0) return;

	//tutkitaan paljonko reunat menevät yli

	int tooup, toolow, tooleft, tooright;
	if((tooup = miny - y) < 0) tooup = 0;
	if((toolow = y + h - miny - areah) < 0) toolow = 0;
	if((tooleft = minx - x) < 0) tooleft = 0;
	if((tooright = x + w - minx - areaw) < 0) tooright = 0;

	//jos kuvasta näkyy ylipäätänsä mitään
	if(tooup < h && toolow < h
			&& tooleft < w && tooright < w){
		GDrawIMG(dest_surface, img,
				x + tooleft,
				y + tooup,
				w - tooleft - tooright,
				h - tooup - toolow,
				x2 + tooleft,
				y2 + tooup);
	}
}

void GDrawIMGArea(SDL_Surface *dest_surface, SDL_Surface *img, int x, int y,
		int minx, int miny, int areaw, int areah)
{
	GDrawIMGArea(dest_surface, img, x, y,
			img->w, img->h, 0, 0,
			minx, miny, areaw, areah);
}

/////////////

int GDrawText(SDL_Surface *dest_surface, int x, int y, int textx, int textw,
		TTF_Font *font, std::string &text, SDL_Color color)
{
	if(text == "") return 0;
	SDL_Surface *text_surface;
	text_surface = TTF_RenderText_Solid(font, text.c_str(), color);
	if(text_surface == NULL)
	{
		std::cout<<"error rendering text"<<std::endl;
		std::cout<<"text is \""<<text<<"\", color is "<<(int)color.r<<","<<(int)color.g<<","<<(int)color.b<<std::endl;
		return -1;
	}

	int w = text_surface->w;
	int h = text_surface->h;

	if(textw != -1) w = textw;

	if(x==-1) x = dest_surface->w/2 - w/2;
	if(y==-1) y = dest_surface->h/2 - h/2;
	
	GDrawIMG(dest_surface, text_surface, x, y, w, h, textx, 0);

	SDL_FreeSurface(text_surface);
	return 0;
}

int GDrawText(SDL_Surface *dest_surface, int x, int y, int textx, int textw,
		TTF_Font *font, std::string &text, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Color color = { r, g, b };
	return GDrawText(dest_surface, x, y, textx, textw, font, text, color);
}

int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font, 
		std::string &text, SDL_Color color)
{
	return GDrawText(dest_surface, x, y, 0, -1, font, text, color);
}

/*int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font,
		std::string text, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Color color = { r, g, b };
	return GDrawText(dest_surface, x, y, 0, -1, font, text, color);
}*/

int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font,
		std::string &text, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Color color = { r, g, b };
	return GDrawText(dest_surface, x, y, 0, -1, font, text, color);
}

int GDrawText(SDL_Surface *dest_surface, int x, int y, TTF_Font *font,
		const char *text, Uint8 r, Uint8 g, Uint8 b)
{
	std::string stext = text;
	SDL_Color color = { r, g, b };
	return GDrawText(dest_surface, x, y, 0, -1, font, stext, color);
}

ViewableText GWrapText(TTF_Font *font, int w, int options, std::string text)
{
	bool wrap_at_spaces = options & GOPT_WRAP_IN_SPACES;
	std::vector<std::string> rows;
	ViewableText vtext;
	vtext.font = font;
	std::string row;
	int rowmaxwidth_wanted = w;
	//int maxheight = view_h;
	int rowmaxwidth = 0;
	int heightnow = 0;
	int fontheight = TTF_FontHeight(font);
	for(unsigned int i=0; i<text.size(); i++){
		if(text[i]!='\n') row += text[i];
		if(text[i]=='\n' || i == text.size()-1){
			//std::cout<<"newline found, row is \""<<row<<"\""<<std::endl;
			int w, h;

			//mitataan leveys
			if(TTF_SizeText(font, row.c_str(), &w, &h)) return vtext;
			
			//lisätään rivi, tarvittaessa pilkkoen.
			
			while(row.size() > 0){
				unsigned int fitting_size = 0; //leikkauskohta
				bool too_long_word = false;
				
				//haetaan kohta josta pätkäistä
				if(wrap_at_spaces){
					//std::cout<<"finding last fitting space"<<std::endl;
					bool whole_did_fit = true;
					for(unsigned int i=0; i < row.size(); i++){
						//std::cout<<"b "<<i<<std::endl;
						if(TTF_SizeText(font, row.substr(0, i).c_str(), &w, &h)) return vtext;
						if(w > rowmaxwidth_wanted){
							whole_did_fit = false;
							break;
						}
						if(row[i] != ' ') continue;
						fitting_size = i+1;
						//std::cout<<fitting_size<<" fits"<<std::endl;
					}
					if(whole_did_fit) fitting_size = row.size();
					if(fitting_size == 0){
						//std::cout<<"too long word"<<std::endl;
						too_long_word = true;
					}
				}
				if(!wrap_at_spaces || too_long_word){
					for(unsigned int i=0; i < row.size(); i++){
						if(TTF_SizeText(font, row.substr(0, i).c_str(), &w, &h)) return vtext;
						if(w > rowmaxwidth_wanted) break;
						fitting_size = i;
					}
				}

				if(TTF_SizeText(font, row.substr(0, fitting_size).c_str(),
						&w, &h)) return vtext;
				//jos rivin leveys on suurempi kuin isoin tiedetty, asetetaan
				//isoimmaksi tämän rivin leveys
				if(w > rowmaxwidth) rowmaxwidth = w;

				heightnow += fontheight;
				ViewableRow vrow;
				vrow.text = row.substr(0, fitting_size);
				//std::cout<<"added row \""<<vrow.text<<"\""<<std::endl;
				vtext.rows.push_back(vrow);
				row = row.substr(fitting_size);
				//std::cout<<"row is now \""<<row<<"\""<<std::endl;
				//std::cout<<"row length is now "<<row.size()<<std::endl;
			}
			row = "";
			continue;
		}
	}
	vtext.height = heightnow;
	vtext.width = rowmaxwidth;
	return vtext;
}

/*
 * texty: ensimmäinen näytettävä rivi
 * texth: näytettävän kohdan korkeus pikseleinä
 * screen_x,y: näytön kohta johon teksti piirretään
 * vtext: teksti
 */
void GDrawMultilineText(SDL_Surface *dest_surface, int textx, int texty, int texth, int textw,
		int screen_x, int screen_y, ViewableText &vtext)
{
	//TODO: hilightit
	
	/*Uint32 color = SDL_MapRGB(GetScreen()->format, 20,20,40);
	DrawRect(screen_x, screen_y, view_w, view_h, color);*/
	
	int fontheight = TTF_FontHeight(vtext.font);
	unsigned int rows_to_show = texth / fontheight;
	int first_row = texty / fontheight;
	if(rows_to_show > vtext.rows.size()) rows_to_show = vtext.rows.size();
	
	for(unsigned int i=0; i<rows_to_show; i++){
		unsigned int ci = first_row + i;
		if(ci >= vtext.rows.size()) break;
		GDrawText(dest_surface, screen_x, screen_y + fontheight * i, textx, textw,
				vtext.font, vtext.rows[ci].text, vtext.rows[ci].color);
	}
}

void GDrawMultilineText(SDL_Surface *dest_surface, int texty, int texth,
		int screen_x, int screen_y, ViewableText &vtext)
{
	GDrawMultilineText(dest_surface, 0, texty, texth, -1, screen_x, screen_y, vtext);
}

void GDrawMultilineText(SDL_Surface *dest_surface,
		int screen_x, int screen_y, ViewableText &vtext)
{
	GDrawMultilineText(dest_surface, 0, 0, vtext.height, -1, screen_x, screen_y, vtext);
}
//###################################################################

#if 0
Graphics::Graphics()
{
	std::cout<<"graphics constructor"<<std::endl;

	screen = NULL;
}

Graphics::~Graphics()
{
	std::cout<<"graphics destructor"<<std::endl;
}

SDL_Surface* Graphics::GetScreen()
{
	return screen;
}

int Graphics::InitVideo(int w, int h)
{
	std::cout<<"initvideo"<<std::endl;
	screen = SDL_SetVideoMode(w, h, SCREENBPP, SDL_HWSURFACE | SDL_DOUBLEBUF);
	if(screen==NULL){
		std::cout<<"Unable to set "<<w<<"x"<<h<<" mode"<<std::endl;
		return -1;
	}
	return 0;
}

int Graphics::GetW()
{
	if(screen==NULL) return 0;
	return screen->w;
}

int Graphics::GetH()
{
	if(screen==NULL) return 0;
	return screen->h;
}

void Graphics::UpdateScreen()
{
	SDL_Flip(screen);
}

/*void Graphics::Slock()
{
	GSlock(screen);
}

void Graphics::Sulock()
{
	GSulock(screen);
}

void Graphics::DrawPixel(int x, int y, Uint32 color)
{
	Uint16 *bufp;
	bufp = (Uint16 *)screen->pixels + y*screen->pitch/2 + x;
	*bufp = color;
}

void Graphics::DrawRect(int x, int y, int w, int h, Uint32 color)
{
	GDrawRect(screen, x, y, w, h, color);
}

void Graphics::DrawAlphaRect(int x, int y, int w, int h, Uint32 color, Uint8 alpha)
{
	GDrawAlphaRect(screen, x, y, w, h, color, alpha);
}

void Graphics::DrawIMG(SDL_Surface *img, int x, int y)
{
	GDrawIMG(img, screen, x, y);
}

void Graphics::DrawIMG(SDL_Surface *img, int x, int y,
		int w, int h, int x2, int y2)
{
	GDrawIMG(img, screen, x, y, w, h, x2, y2);
}

void Graphics::ClearScreen()
{
	Uint32 color = SDL_MapRGB(screen->format, 0, 0, 0);
	DrawRect(0, 0, screen->w, screen->h, color);
}*/

#endif

#if 0
int Graphics::DrawText(int x, int y, int textx, int textw,
		TTF_Font *font, std::string &text, SDL_Color color)
{
	if(text == "") return -1;
	SDL_Surface *text_surface;
	text_surface = TTF_RenderText_Solid(font, text.c_str(), color);
	if(text_surface == NULL)
	{
		std::cout<<"error rendering text"<<std::endl;
		return -1;
	}

	int w = text_surface->w;
	int h = text_surface->h;

	if(textw != -1) w = textw;

	if(x==-1) x = screen->w/2 - w/2;
	if(y==-1) y = screen->h/2 - h/2;
	
	DrawIMG(text_surface, x, y, w, h, textx, 0);

	SDL_FreeSurface(text_surface);
	return 0;
}

int Graphics::DrawText(int x, int y, int textx, int textw,
		TTF_Font *font, std::string &text, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Color color = { r, g, b };
	return DrawText(x, y, textx, textw, font, text, color);
}

int Graphics::DrawText(int x, int y, TTF_Font *font, std::string &text, SDL_Color color)
{
	return DrawText(x, y, 0, -1, font, text, color);
}

int Graphics::DrawText(int x, int y, TTF_Font *font, std::string text, Uint8 r, Uint8 g, Uint8 b)
{
	SDL_Color color = { r, g, b };
	return DrawText(x, y, 0, -1, font, text, color);
}

/*int Graphics::TTF_SizeText(TTF_Font *font, std::string text.c_str(), int *w, int *h)
{
	return TTF_SizeText(font, text.c_str(), w, h);
}*/

/*int Graphics::TTF_FontHeight(TTF_Font *font)
{
	return TTF_TTF_FontHeight(font);
}*/

ViewableText Graphics::WrapText(TTF_Font *font, int w, int options, std::string text)
{
	bool wrap_at_spaces = options & GOPT_WRAP_IN_SPACES;
	std::vector<std::string> rows;
	ViewableText vtext;
	vtext.font = font;
	std::string row;
	int rowmaxwidth_wanted = w;
	//int maxheight = view_h;
	int rowmaxwidth = 0;
	int heightnow = 0;
	int fontheight = TTF_FontHeight(font);
	for(unsigned int i=0; i<text.size(); i++){
		if(text[i]!='\n') row += text[i];
		if(text[i]=='\n' || i == text.size()-1){
			//std::cout<<"newline found, row is \""<<row<<"\""<<std::endl;
			int w, h;

			//mitataan leveys
			if(TTF_SizeText(font, row.c_str(), &w, &h)) return vtext;
			
			//lisätään rivi, tarvittaessa pilkkoen.
			
			while(row.size() > 0){
				unsigned int fitting_size = 0; //leikkauskohta
				bool too_long_word = false;
				
				//haetaan kohta josta pätkäistä
				if(wrap_at_spaces){
					//std::cout<<"finding last fitting space"<<std::endl;
					bool whole_did_fit = true;
					for(unsigned int i=0; i < row.size(); i++){
						//std::cout<<"b "<<i<<std::endl;
						if(TTF_SizeText(font, row.substr(0, i).c_str(), &w, &h)) return vtext;
						if(w > rowmaxwidth_wanted){
							whole_did_fit = false;
							break;
						}
						if(row[i] != ' ') continue;
						fitting_size = i+1;
						//std::cout<<fitting_size<<" fits"<<std::endl;
					}
					if(whole_did_fit) fitting_size = row.size();
					if(fitting_size == 0){
						//std::cout<<"too long word"<<std::endl;
						too_long_word = true;
					}
				}
				if(!wrap_at_spaces || too_long_word){
					for(unsigned int i=0; i < row.size(); i++){
						if(TTF_SizeText(font, row.substr(0, i).c_str(), &w, &h)) return vtext;
						if(w > rowmaxwidth_wanted) break;
						fitting_size = i;
					}
				}

				if(TTF_SizeText(font, row.substr(0, fitting_size).c_str(),
						&w, &h)) return vtext;
				//jos rivin leveys on suurempi kuin isoin tiedetty, asetetaan
				//isoimmaksi tämän rivin leveys
				if(w > rowmaxwidth) rowmaxwidth = w;

				heightnow += fontheight;
				ViewableRow vrow;
				vrow.text = row.substr(0, fitting_size);
				//std::cout<<"added row \""<<vrow.text<<"\""<<std::endl;
				vtext.rows.push_back(vrow);
				row = row.substr(fitting_size);
				//std::cout<<"row is now \""<<row<<"\""<<std::endl;
				//std::cout<<"row length is now "<<row.size()<<std::endl;
			}
			row = "";
			continue;
		}
	}
	vtext.height = heightnow;
	vtext.width = rowmaxwidth;
	return vtext;
}

/*
 * texty: ensimmäinen näytettävä rivi
 * texth: näytettävän kohdan korkeus pikseleinä
 * screen_x,y: näytön kohta johon teksti piirretään
 * vtext: teksti
 */
void Graphics::DrawMultilineText(int textx, int texty, int texth, int textw,
		int screen_x, int screen_y, ViewableText &vtext)
{
	//TODO: hilightit
	
	/*Uint32 color = SDL_MapRGB(GetScreen()->format, 20,20,40);
	DrawRect(screen_x, screen_y, view_w, view_h, color);*/
	
	int fontheight = TTF_FontHeight(vtext.font);
	unsigned int rows_to_show = texth / fontheight;
	int first_row = texty / fontheight;
	if(rows_to_show > vtext.rows.size()) rows_to_show = vtext.rows.size();
	
	for(unsigned int i=0; i<rows_to_show; i++){
		unsigned int ci = first_row + i;
		if(ci >= vtext.rows.size()) break;
		DrawText(screen_x, screen_y + fontheight * i,
				textx, textw,
				vtext.font, vtext.rows[ci].text, vtext.rows[ci].color);
	}
}

void Graphics::DrawMultilineText(int texty, int texth, int screen_x, int screen_y,
		ViewableText &vtext)
{
	DrawMultilineText(0, texty, texth, -1, screen_x, screen_y, vtext);
}

void Graphics::DrawMultilineText(int screen_x, int screen_y, ViewableText &vtext)
{
	DrawMultilineText(0, 0, vtext.height, -1, screen_x, screen_y, vtext);
}
#endif

