#include "gui.h"
#include <iostream>

GuiEvent::GuiEvent()
{
}

GuiEvent::GuiEvent(std::string cname, GuiEventType type_)
{
	type = type_;
	AddName(cname);
}

void GuiEvent::AddName(std::string name)
{
	cnames.push_back(name);
}

///////////////////////////////

GuiKey::GuiKey()
{
}

GuiKey::GuiKey(SDLKey sdlk_, bool shiftstate_)
{
	SetSDLK(sdlk_, shiftstate_);
}

void GuiKey::SetSDLK(SDLKey sdlk_, bool shiftstate_)
{
	sdlk = sdlk_;
	shiftstate = shiftstate_;
}

SDLKey GuiKey::GetSDLK()
{
	return sdlk;
}

char GuiKey::GetChar()
{
	if(sdlk >= SDLK_a && sdlk <= SDLK_z){
		if(shiftstate==false) return 'a' + (sdlk - SDLK_a);
		if(shiftstate==true ) return 'A' + (sdlk - SDLK_a);
	}
	switch(sdlk){
		case SDLK_SPACE:
			return ' ';
		case SDLK_BACKSPACE:
			return '\b';
		case SDLK_MINUS:
			if(!shiftstate) return '-';
			else return '_';
		case SDLK_PERIOD:
			if(!shiftstate) return '.';
			else return ':';
		case SDLK_COMMA:
			if(!shiftstate) return ',';
			else return ';';
		case SDLK_PLUS:
			if(!shiftstate) return '+';
			else return '?';
		case SDLK_RETURN:
			return '\n';
		/*case SDLK_UNDERSCORE:
			return '_';*/
		default:
			break;
	}
	if(sdlk >= SDLK_0 && sdlk <= SDLK_9){
		if(shiftstate){
			switch(sdlk){
			case SDLK_1:
				return '!';
			case SDLK_2:
				return '"';
			case SDLK_3:
				return '#';
			case SDLK_4:
				return '$';
			case SDLK_5:
				return '%';
			case SDLK_6:
				return '&';
			case SDLK_7:
				return '/';
			case SDLK_8:
				return '(';
			case SDLK_9:
				return ')';
			case SDLK_0:
				return '=';
			default:
				break;
			}
		}
		return '0' + (sdlk - SDLK_0);
	}
	return -1;
}

//#############################################################

GuiComponent::GuiComponent(int w, int h, std::string name, Fonts fonts)
{
	//std::cout<<"GuiComponent::GuiComponent("<<w<<", "<<h<<", \""<<name<<"\")"<<std::endl;
	SetProperties(w, h, name, fonts);
	delete_me = false;
	disabled = false;
}

GuiComponent::~GuiComponent()
{
}

void GuiComponent::SetBasePlace(int x, int y)
{
	basex = x;
	basey = y;
	screen_x = basex + rel_x;
	screen_y = basey + rel_y;
}

void GuiComponent::SetRelPlace(int x, int y)
{
	rel_x = x;
	rel_y = y;
	screen_x = basex + rel_x;
	screen_y = basey + rel_y;
}

void GuiComponent::SetSize(int w, int h)
{
	cw = w;
	ch = h;
}

void GuiComponent::GetRelPlace(int &x, int &y)
{
	x = rel_x;
	y = rel_y;
}

void GuiComponent::GetSize(int &w, int &h)
{
	
	w = cw;
	h = ch;
}

void GuiComponent::SetName(std::string name)
{
	cname = name;
}

std::string GuiComponent::GetName()
{
	return cname;
}

void GuiComponent::SetProperties(int w, int h, std::string name, Fonts fonts_)
{
	cw = w;
	ch = h;
	cname = name;
	fonts = fonts_;
}

void GuiComponent::Delete()
{
	delete_me = true;
}

bool GuiComponent::Deleted()
{
	return delete_me;
}

void GuiComponent::SetDisabled(bool disabled_)
{
	disabled = disabled_;
	if(disabled){
		GuiEvent e(cname, GUIEVENT_REQUEST_DRAWBACKGROUND);
		events.push_back(e);
	}
}

void GuiComponent::Disable()
{
	SetDisabled(true);
}

void GuiComponent::Enable()
{
	SetDisabled(false);
}

bool GuiComponent::IsDisabled()
{
	return disabled;
}

void GuiComponent::SetSurface(SDL_Surface *s)
{
	surface = s;
}

SDL_Surface* GuiComponent::GetSurface()
{
	return surface;
}

/*TTF_Font *GuiComponent::GetDefaultFont()
{
	return fonts.normal;
}*/

bool GuiComponent::GetEvent(GuiEvent *event)
{
	if(events.size() > 0){
		*event = events.front();
		events.pop_front();
		return true;
	}
	return false;
}

void GuiComponent::Draw(int x, int y, int w, int h, bool is_selected)
{
	Uint32 color = SDL_MapRGB(surface->format, 255, 0, 0);
	GDrawRect(surface, screen_x+x, screen_y+y, w, h, color);
}

bool GuiComponent::Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton)
{
	/*//if is on it
	if(x>=0 && y>=0 && x<cw && x<ch) return true;*/
	return false;
}

bool GuiComponent::Keyboard(GuiKey key)
{
	return false;
}

bool GuiComponent::SDLK(SDLKey key, bool down)
{
	return false;
}

///////////////////////////////////////////////////////////////////////

GuiContainer::GuiContainer(int w, int h, std::string name, std::string title_,
		Uint8 r, Uint8 g, Uint8 b, Fonts fonts)
	: GuiComponent(0, 0, name, fonts)
{
	selected_component = -1;
	bgcolor_r = r;
	bgcolor_g = g;
	bgcolor_b = b;
	title = title_;
	cw = w;
	if(title == "") titleh = 0;
	else titleh = TTF_FontHeight(fonts.normal);
	ch = h + titleh;
	titlegrabbed = false;
	bg_drawn = false;
}

GuiContainer::~GuiContainer()
{
	for(unsigned int i=0; i<components.size(); i++){
		delete components[i];
	}
}

void GuiContainer::SetSize(int w, int h_)
{
	int h = h_ + titleh;
	GuiComponent::SetSize(w, h);
}

void GuiContainer::SetBasePlace(int x, int y)
{
	GuiComponent::SetBasePlace(x, y);
	for(unsigned int i=0; i<components.size(); i++){
		components[i]->SetBasePlace(screen_x, screen_y);
	}
}

void GuiContainer::SetRelPlace(int x, int y)
{
	GuiComponent::SetRelPlace(x, y);
	for(unsigned int i=0; i<components.size(); i++){
		components[i]->SetBasePlace(screen_x, screen_y);
	}
}

void GuiContainer::SetSurface(SDL_Surface *s)
{
	GuiComponent::SetSurface(s);
	for(unsigned int i=0; i<components.size(); i++){
		components[i]->SetSurface(s);
	}
}

void GuiContainer::Draw(int dx, int dy, int dw, int dh, bool is_selected)
{
	Uint32 bgcolor = SDL_MapRGB(surface->format,
			bgcolor_r, bgcolor_g, bgcolor_b);
	//titleviritelmä ja tausta
	if(title!=""){
		GDrawRect(surface, screen_x, screen_y+titleh, cw, ch-titleh, bgcolor);
		Uint32 titlecolor = bgcolor + 0x22222222;
		if(is_selected) titlecolor += 0x44444444;
		GDrawRect(surface, screen_x, screen_y, cw, titleh, titlecolor);
		GDrawText(surface, screen_x, screen_y, fonts.normal, title, 255,255,255);
	}
	else{
		if(!bg_drawn){
			bg_drawn = true;
			GDrawRect(surface, screen_x, screen_y, cw, ch, bgcolor);
		}
	}

	{//vc6
	//katsotaan jos komponentteja pitää poistaa
	for(unsigned int i=0; i<components.size(); i++){
		if(components[i]->Deleted()){
			int x, y, w, h;
			components[i]->GetRelPlace(x, y);
			components[i]->GetSize(w, h);
			GDrawRect(surface, basex + x, basey + y, w, h, bgcolor);
			if((int)i == selected_component) selected_component = -1;
			delete components[i];
			components.erase(components.begin()+i, components.begin()+i+1);
			i = 0; //alusta
		}
	}
	}//vc6
	//piirrellään
	{//vc6
	for(unsigned int i=0; i<components.size(); i++){
		if(components[i]->IsDisabled()) continue;
		int x, y, w, h;
		components[i]->GetRelPlace(x, y);
		components[i]->GetSize(w, h);
		if(x < 0) x = 0;
		if(x + w >= cw) x = cw - w;
		if(y < 0) y = 0;
		if(y + h >= ch) y = ch - h;
		components[i]->SetRelPlace(x, y);
		/*std::cout<<"GuiContainer \""<<cname<<"\" drawing \""<<components[i]->GetName()
				<<"\" (x="<<x<<" y="<<y<<" w="<<w<<" h="<<h<<")?"<<std::endl;*/
		if(x + w >= dx && x <= dx + dw && y + h >= dy && y <= dy + dh){
			//std::cout<<"yes"<<std::endl;
			components[i]->Draw(0, 0, w, h, ((int)i==selected_component)); //TODO
		}
		else{
			/*std::cout<<"GuiContainer \""<<cname<<"\" not drawing \""<<components[i]->GetName()
					<<"\""<<std::endl;*/
		}
	}
	}//vc6
}

bool GuiContainer::Mouse(int mx, int my, int movex, int movey, bool lbutton, bool rbutton)
{
	bool is_on_area = (mx >= 0 && my >= 0 && mx < cw && my < ch);

	if(!is_on_area && (lbutton || rbutton)) selected_component = -1;

	//otsikosta raahailu
	if(titleh>0){
		if(my >= 0 && my < titleh && mx >= 0 && mx < cw && !titlegrabbed && lbutton){
			titlegrabbed = true;
			return true;
		}
		if(titlegrabbed){
			if(!lbutton){
				titlegrabbed = false;
				return true;
			}
			int new_rel_x = rel_x + movex;
			int new_rel_y = rel_y + movey;
			//std::cout<<"new_rel_x="<<new_rel_x<<" new_rel_y="<<new_rel_y<<std::endl;
			SetRelPlace(new_rel_x, new_rel_y);
			GuiEvent e(cname, GUIEVENT_REQUEST_DRAWBACKGROUND);
			events.push_back(e);
			return true;
		}
	}
	
	//annetaan hiiren tiedot komponentille
	bool did = false;
	//aloitetaan uusimmasta = päällimmäisestä ja ei tehdä uusiksi
	for(int i=components.size()-1; i>=0; i--){
		if(components[i]->IsDisabled()) continue;
		int x, y, w, h;
		components[i]->GetRelPlace(x, y);
		components[i]->GetSize(w, h);
		int mx2 = mx - x;
		int my2 = my - y;
		if(!did){
			if(mx2 >= 0 && my2 >= 0 && mx2 < w && my2 < h && (lbutton || rbutton)){
				selected_component = i;
			}
			if(components[i]->Mouse(mx2, my2, movex, movey, lbutton, rbutton)){
				did = true;
				mx = x + mx2;
				my = y + my2;
			}
		}
	}
	if(lbutton && !did) selected_component = -1;

	/*if(mx < 0) mx = 0;
	if(my < 0) my = 0;
	if(mx >= surface->w) mx = surface->w-1;
	if(my >= surface->h) my = surface->h-1;*/

	return is_on_area;
}

bool GuiContainer::Keyboard(GuiKey key)
{
	if(selected_component != -1){
		return components[selected_component]->Keyboard(key);
	}
	return false;
}

bool GuiContainer::SDLK(SDLKey key, bool down)
{
	bool did = false;
	for(int i=components.size()-1; i>=0; i--){
		if(components[i]->IsDisabled()) continue;
		if(components[i]->SDLK(key, down)) did = true;
	}
	return did;
}

void GuiContainer::AddComponent(GuiComponent *c, int x, int y)
{
	/*std::cout<<"GuiContainer \""<<cname<<"\" adding GuiComponent \""<<c->GetName()<<"\""
			<<std::endl;*/
	
	y = y + titleh;

	int w, h;
	c->GetSize(w, h);
	
	if(x==-1) x = cw/2 - w/2;
	if(y==-1) y = ch/2 - h/2;

	//TODO: jotain tarkistuksia paikalle
	
	c->SetBasePlace(screen_x, screen_y);
	c->SetRelPlace(x, y);
	
	c->SetSurface(surface);
	components.push_back(c);

	selected_component = components.size()-1;
}

bool GuiContainer::GetEvent(GuiEvent *event)
{
	//onko omia eventtejä?
	if(events.size() > 0){
		*event = events.front();
		events.pop_front();
		//std::cout<<"container \""<<cname<<"\" returning event type "<<event->type<<", event list size was "<<events.size()+1<<std::endl;
		return true;
	}
	//ei, otetaan sitten komponenttien
	for(int i=components.size()-1; i>=0; i--){
		/*std::cout<<"container \""<<cname<<"\" checking if component named "
				<<components[i]->GetName()
				<<" has events"<<std::endl;*/
		if(components[i]->GetEvent(event)){
			//std::cout<<"container \""<<cname<<"\" got event type "<<event->type
			//	<<" from "<<event->cnames[0]<<std::endl;
			if(event->type == GUIEVENT_REQUEST_DRAWBACKGROUND){
				bg_drawn = false;
			}
			if(cname != "") event->AddName(cname);
			return true;
		}
	}
	return false;
}

////////

GuiLabel::~GuiLabel()
{
}

GuiLabel::GuiLabel(std::string name, std::string text_, Fonts f)
	: GuiComponent(0, 0, name, f)
{
	SetText(text_);
	TTF_SizeText(fonts.normal, text.c_str(), &cw, &ch);
}

void GuiLabel::SetText(std::string text_)
{
	text = text_;
	TTF_SizeText(fonts.normal, text.c_str(), &cw, &ch);
	GuiEvent e(cname, GUIEVENT_REQUEST_DRAWBACKGROUND);
	events.push_back(e);
}

void GuiLabel::Draw(int x, int y, int w, int h, bool is_selected)
{
	//std::cout<<"GuiLabel::Draw() (cname=\""<<cname<<"\""<<std::endl;
	GDrawText(surface, screen_x, screen_y, fonts.normal, text, 255,255,255);
}

/////////

GuiMultilineLabel::GuiMultilineLabel(std::string name, ViewableText &vtext_, Fonts f)
	: GuiComponent(0, 0, name, f)
{
	vtext = vtext_;
	ch = vtext.height;
	cw = vtext.width;
}

GuiMultilineLabel::~GuiMultilineLabel()
{
}

void GuiMultilineLabel::Draw(int x, int y, int w, int h, bool is_selected)
{
	GDrawMultilineText(surface, screen_x, screen_y, vtext);
}

void GuiMultilineLabel::SetText(ViewableText &vtext_)
{
	vtext = vtext_;
}

////////

GuiTextView::GuiTextView(int w, int h, std::string name, ViewableText *vtext_, Fonts f)
	: GuiComponent(0, 0, name, f)
{
	cw = w;
	ch = h;
	text_scrollpos_x = 0;
	text_scrollpos_y = 0;
	vtext = vtext_;
}

GuiTextView::~GuiTextView()
{
}

void GuiTextView::Draw(int x, int y, int w, int h, bool is_selected)
{
	//TODO: uudestaan!
	Uint32 textbgcolor = SDL_MapRGB(surface->format, 20,60,50);
	GDrawRect(surface, screen_x, screen_y, cw, ch, textbgcolor);
	/*Uint32 scrollbgcolor = SDL_MapRGB(surface->format, 10,30,25);
	Uint32 scrollblockcolor = SDL_MapRGB(surface->format, 40,120,100);
	int scrollbarsize = 20;
	int textx = screen_x;
	int texty = screen_y;
	int textw = cw - scrollbarsize;
	int texth = ch - scrollbarsize;
	int scrollbarx = textx + textw;
	int scrollbary = texty + texth;
	double scroll_amount_h = (double)text_scrollpos_x / (double)(vtext->width - textw);
	double scroll_amount_v = (double)text_scrollpos_y / (double)(vtext->height - texth);

	int b_scrollblock_w = (textw * textw) / vtext->width;
	if(b_scrollblock_w > textw) b_scrollblock_w = textw;
	int b_scrollblock_h = scrollbarsize;
	int r_scrollblock_w = scrollbarsize;
	int r_scrollblock_h = (texth * texth) / vtext->height;
	if(r_scrollblock_h > texth) r_scrollblock_h = texth;
	
	int scrollblock_x = (int)((double)(textw - b_scrollblock_w) * scroll_amount_h);
	int scrollblock_y = (int)((double)(texth - r_scrollblock_h) * scroll_amount_v);
	if(scrollblock_x + b_scrollblock_w > textw) scrollblock_x = textw - b_scrollblock_w;
	if(scrollblock_y + r_scrollblock_h > texth) scrollblock_y = texth - r_scrollblock_h;
	
	GDrawRect(surface, screen_x, screen_y, textw, texth, textbgcolor);
	GDrawMultilineText(surface, text_scrollpos_x, text_scrollpos_y, texth, textw, screen_x, screen_y, *vtext);
	GDrawRect(surface, scrollbarx, screen_y, scrollbarsize, texth, scrollbgcolor);
	GDrawRect(surface, screen_x, scrollbary, textw, scrollbarsize, scrollbgcolor);
	GDrawRect(surface, screen_x + scrollblock_x, scrollbary,
			b_scrollblock_w, b_scrollblock_h, scrollblockcolor);
	GDrawRect(surface, scrollbarx, screen_y + scrollblock_y,
			r_scrollblock_w, r_scrollblock_h, scrollblockcolor);*/
}

bool GuiTextView::Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton)
{
	bool justpressed = false;
	if(lbutton && !pressed && x>=0 && y>=0 && x<cw && y<ch){
		pressed = true;
		justpressed = true;
		dragstart_x = x;
		dragstart_y = y;
		text_oldpos_x = text_scrollpos_x;
		text_oldpos_y = text_scrollpos_y;
	}
	if(!lbutton && pressed){
		pressed = false;
		pressed_on_v_scrollbar = false;
		pressed_on_h_scrollbar = false;
	}
	if(pressed){
		//TODO: uudestaan!
		/*int scrollbarsize = 20;
		int textx = 0;
		int texty = 0;
		int textw = cw - scrollbarsize;
		int texth = ch - scrollbarsize;
		int scrollbarx = textx + textw;
		int scrollbary = texty + texth;
		double scroll_amount_h = (double)text_scrollpos_x / (double)(vtext->width - textw);
		double scroll_amount_v = (double)text_scrollpos_y / (double)(vtext->height - texth);

		int b_scrollblock_w = (textw * textw) / vtext->width;
		if(b_scrollblock_w > textw) b_scrollblock_w = textw;
		//int b_scrollblock_h = scrollbarsize;
		//int r_scrollblock_w = scrollbarsize;
		int r_scrollblock_h = (texth * texth) / vtext->height;
		if(r_scrollblock_h > texth) r_scrollblock_h = texth;

		int scrollblock_x = (int)((double)(textw - b_scrollblock_w) * scroll_amount_h);
		int scrollblock_y = (int)((double)(texth - r_scrollblock_h) * scroll_amount_v);
		if(scrollblock_x > textw - b_scrollblock_w) scrollblock_x = textw - b_scrollblock_w;
		if(scrollblock_y > texth - r_scrollblock_h) scrollblock_y = texth - r_scrollblock_h;

		int scrollx = dragstart_x - x;
		int scrolly = dragstart_y - y;

		if(pressed_on_v_scrollbar || (textw < vtext->width
				&& x >= scrollbarx && x < scrollbarx + scrollbarsize
				&& y >= scrollblock_y
				&& y < scrollblock_y + r_scrollblock_h)){
			double scroll_pixel_per_scrollbar_pixel_v = (double)(vtext->height - texth) /
					(double)(texth - r_scrollblock_h);
			text_scrollpos_y = text_oldpos_y - (int)((double)scrolly
					* scroll_pixel_per_scrollbar_pixel_v);
			if(justpressed) pressed_on_v_scrollbar = true;
		}
		if(pressed_on_h_scrollbar || (texth < vtext->height
				&& y >= scrollbary && y < scrollbary + scrollbarsize
				&& x >= scrollblock_x
				&& x < scrollblock_x + b_scrollblock_w)){
			double scroll_pixel_per_scrollbar_pixel_h = (double)(vtext->width - textw) /
					(double)(textw - b_scrollblock_w);
			text_scrollpos_x = text_oldpos_x - (int)((double)scrollx
					* scroll_pixel_per_scrollbar_pixel_h);
			if(justpressed) pressed_on_h_scrollbar = true;
		}
		if(text_scrollpos_x > vtext->width - cw + scrollbarsize)
			text_scrollpos_x = vtext->width - cw + scrollbarsize;
		if(text_scrollpos_y > vtext->height - ch + scrollbarsize)
			text_scrollpos_y = vtext->height - ch + scrollbarsize;
		if(text_scrollpos_x < 0) text_scrollpos_x = 0;
		if(text_scrollpos_y < 0) text_scrollpos_y = 0;*/
		return true;
	}
	if(x>=0 && y>=0 && x<cw && y<ch) return true;
	return false;
}

////////

GuiTextButton::GuiTextButton(int w, int h, std::string name, std::string label_, Fonts f)
	: GuiComponent(w, h, name, f)
{
	pressed = false;
	label = label_;
}

void GuiTextButton::SetLabel(std::string text)
{
	label = text;
}

GuiTextButton::~GuiTextButton()
{
}

void GuiTextButton::Draw(int x, int y, int w, int h, bool is_selected)
{ 
	Uint32 color;
	if(!pressed) color = SDL_MapRGB(surface->format, 70,40,80);
	else         color = SDL_MapRGB(surface->format, 45,30,50);
	int tw, th;
	TTF_SizeText(fonts.normal, label.c_str(), &tw, &th);
	int tx = cw/2 - tw/2;
	int ty = ch/2 - th/2;
	GDrawRect(surface, screen_x, screen_y, cw, ch, color);
	GDrawText(surface, screen_x+tx, screen_y+ty, fonts.normal, label, 255,255,255);
}

bool GuiTextButton::Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton)
{
	if(x>=0 && y>=0 && x<cw && y<ch){
		if(!pressed && lbutton) pressed = true;
		else if(!lbutton && pressed){
			pressed = false;
			GuiEvent e(cname, GUIEVENT_BUTTONCLICK);
			events.push_back(e);
		}
		return true;
	}
	else{
		pressed = false;
	}
	return false;
}

////////////////

GuiImage::GuiImage(std::string name, SDL_Surface *image_, Fonts fonts)
	: GuiComponent(0, 0, name, fonts)
{
	pressed = false;
	image = image_;
	SetSize(image->w, image->h);
}

GuiImage::~GuiImage()
{
}

void GuiImage::Draw(int x, int y, int w, int h, bool is_selected)
{
	//std::cout<<"GuiImage::Draw("<<x<<", "<<y<<", "<<w<<", "<<h<<", "<<is_selected<<")"<<std::endl;
	GDrawIMG(surface, image, screen_x+x, screen_y+y, w, h, x, y);
	if(pressed){
		Uint32 color;
		color = SDL_MapRGB(surface->format, 0, 0, 0);
		GDrawRect(surface, screen_x, screen_y, cw, 1, color);
		GDrawRect(surface, screen_x, screen_y+1, 1, ch-2, color);
		color = SDL_MapRGB(surface->format, 128, 128, 128);
		GDrawRect(surface, screen_x+cw-1, screen_y+1, 1, ch-2, color);
		GDrawRect(surface, screen_x, screen_y+ch-1, cw, 1, color);
	}
	/*if(pressed){
		Uint32 color = SDL_MapRGB(surface->format, 255, 255, 255);
		GDrawAlphaRect(surface, screen_x+x, screen_y+y, w, h, color, 32);
	}*/
}

bool GuiImage::Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton)
{
	if(x>=0 && y>=0 && x<cw && y<ch){
		if(!pressed && lbutton) pressed = true;
		else if(!lbutton && pressed){
			pressed = false;
			GuiEvent e(cname, GUIEVENT_IMAGECLICK);
			events.push_back(e);
		}
		return true;
	}
	else{
		pressed = false;
		GuiEvent e(cname, GUIEVENT_REQUEST_DRAWBACKGROUND);
		events.push_back(e);
	}
	return false;
}

////////////////

GuiScrollbar::~GuiScrollbar()
{
}

GuiScrollbar::GuiScrollbar(int w, int h, std::string name, GuiScrollbarType type_,
		int range, int barlength, int position, Fonts fonts)
		: GuiComponent(w, h, name, fonts)
{
	type = type_;
	pressed = false;
	SetProperties(range, barlength, position);
}

void GuiScrollbar::SetProperties(int range_, int barlength_, int position_)
{
	range = range_;
	barlength = barlength_;
	position = position_;
}

void GuiScrollbar::SetRange(int range_)
{
	range = range_;
}

void GuiScrollbar::SetBarLength(int barlength_)
{
	barlength = barlength_;
}

void GuiScrollbar::SetPosition(int position_)
{
	position = position_;
}

int GuiScrollbar::GetPosition()
{
	return position;
}

void GuiScrollbar::Draw(int x, int y, int w, int h, bool is_selected)
{
	Uint32 color;
	int bx, by, bw, bh;
	double screen_real_ratio;
	CalcBar(bx, by, bw, bh, screen_real_ratio);

	color = SDL_MapRGB(surface->format, 35,60,40);
	if(type == GUISCROLLBAR_VERT){
		GDrawRect(surface, screen_x, screen_y, bw, by, color);
		GDrawRect(surface, screen_x+bx, screen_y+by+bh, bw, ch - by - bh, color);
	}
	else{
		GDrawRect(surface, screen_x, screen_y, bx, bh, color);
		GDrawRect(surface, screen_x+bx+bw, screen_y+by, cw - bx - bw, bh, color);
	}

	color = SDL_MapRGB(surface->format, 130,170,150);
	GDrawRect(surface, screen_x+bx, screen_y+by, bw, bh, color);
}

bool GuiScrollbar::Mouse(int mx, int my, int movex, int movey, bool lbutton, bool rbutton)
{
	/*std::cout<<"GuiScrollbar::Mouse("<<mx<<","<<my<<","<<movex<<","<<movey<<","	
		<<lbutton<<","<<rbutton<<")"<<std::endl;*/
	int bx, by, bw, bh;
	double screen_real_ratio;
	CalcBar(bx, by, bw, bh, screen_real_ratio);

	bool on_bar = (mx >= bx && my >= by
			&& mx < bx + bw && my < by + bh);
	//std::cout<<"on_bar="<<on_bar<<std::endl;

	if(on_bar && !pressed && lbutton){
		pressed = true;
		mstartx = mx;
		mstarty = my;
		startposition = position;
		return true;
	}
	if(pressed && lbutton){
		//siirrä palkkia
		if(type == GUISCROLLBAR_VERT){
			position = startposition + (int)((double)(my - mstarty) / screen_real_ratio);
		}
		else{
			position = startposition + (int)((double)(mx - mstartx) / screen_real_ratio);
		}
		if(position < 0) position = 0;
		if(position >= range - barlength) position = range - barlength;
		//std::cout<<"would move bar to position="<<position<<std::endl;
		return true;
	}
	if(pressed && !lbutton){
		pressed = false;
	}
	return false;
}

void GuiScrollbar::CalcBar(int &x, int &y, int &w, int &h, double &screen_real_ratio)
{
	x = 0;
	y = 0;
	w = 20;
	h = 20;

	int range_on_screen;
	if(type == GUISCROLLBAR_VERT){
		range_on_screen = ch;
	}
	else{
		range_on_screen = cw;
	}

	screen_real_ratio = ((double)range_on_screen / (double)range);

	int barlength_on_screen = (int)(screen_real_ratio * (double)barlength);
	int position_on_screen = (int)(screen_real_ratio * (double)position);
	
	if(type == GUISCROLLBAR_VERT){
		h = barlength_on_screen;
		y = position_on_screen;
	}
	else{
		w = barlength_on_screen;
		x = position_on_screen;
	}
}

////////////////

GuiScrollable::GuiScrollable(int w, int h, std::string name, GuiComponent *c, Fonts fonts)
	: GuiComponent(w, h, name, fonts)
{
	drawsurface = NULL;
	SetComponent(c);
}

GuiScrollable::GuiScrollable(int w, int h, std::string name, Fonts fonts)
	: GuiComponent(w, h, name, fonts)
{
	drawsurface = NULL;
}

GuiScrollable::~GuiScrollable()
{
	delete component;
	delete scrollbar_vert;
	delete scrollbar_horiz;
	if(drawsurface!=NULL) SDL_FreeSurface(drawsurface);
}

void GuiScrollable::SetBasePlace(int x, int y)
{
	GuiComponent::SetBasePlace(x, y);
	scrollbar_vert->SetBasePlace(screen_x, screen_y);
	scrollbar_horiz->SetBasePlace(screen_x, screen_y);
}

void GuiScrollable::SetRelPlace(int x, int y)
{
	GuiComponent::SetRelPlace(x, y);
	scrollbar_vert->SetBasePlace(screen_x, screen_y);
	scrollbar_horiz->SetBasePlace(screen_x, screen_y);
}

void GuiScrollable::SetSurface(SDL_Surface *s)
{
	GuiComponent::SetSurface(s);
	scrollbar_vert->SetSurface(s);
	scrollbar_horiz->SetSurface(s);
}

void GuiScrollable::Draw(int x, int y, int w, int h, bool is_selected)
{
	//std::cout<<"GuiScrollable::Draw(int x="<<x<<", int y="<<y<<", int w="<<w<<", int h="<<h<<", bool is_selected="<<is_selected<<")"<<std::endl;
	if(!w_fits || !h_fits){
		Uint32 color = SDL_MapRGB(surface->format, 30, 10, 20);
		GDrawRect(surface, screen_x+x, screen_y+y, w, h, color);
	}
	
	position_y = scrollbar_vert->GetPosition();
	position_x = scrollbar_horiz->GetPosition();

	component->Draw(position_x, position_y, com_w - position_x, com_h - position_y, false);
	GDrawIMG(surface, drawsurface, screen_x, screen_y, cw-20, ch-20, position_x, position_y);

	scrollbar_vert->Draw(0, 0, 20, ch-20, false);
	scrollbar_horiz->Draw(0, 0, cw-20, 20, false);

	if(is_selected){
		Uint32 color = SDL_MapRGB(surface->format, 120, 40, 80);
		GDrawRect(surface, screen_x, screen_y, cw, 1, color);
		GDrawRect(surface, screen_x, screen_y+1, 1, ch-2, color);
		GDrawRect(surface, screen_x+cw-1, screen_y+1, 1, ch-2, color);
		GDrawRect(surface, screen_x, screen_y+ch-1, cw, 1, color);
	}
}

bool GuiScrollable::Mouse(int mx, int my, int movex, int movey, bool lbutton, bool rbutton)
{
	bool on_area = (mx>=0 && my>=0 && mx<cw && my<ch);
	bool on_component_area = (mx>=0 && my>=0 && mx<cw-20 && my<ch-20);

	if(scrollbar_vert->Mouse(mx - cw + 20, my, movex, movey, lbutton, rbutton)) return true;
	if(scrollbar_horiz->Mouse(mx, my - ch + 20, movex, movey, lbutton, rbutton)) return true;
	
	if(on_component_area){
		if(component->Mouse(position_x + mx, position_y + my, movex, movey, lbutton, rbutton)){
			return true;
		}
	}

	return on_area && lbutton;
}

bool GuiScrollable::GetEvent(GuiEvent *event)
{
	//onko omia eventtejä?
	if(GuiComponent::GetEvent(event)) return true;
	//ei, otetaan sitten komponentin, jos sillä on
	if(component->GetEvent(event)){
		if(cname != "") event->AddName(cname);
		return true;
	}
	return false;
}

void GuiScrollable::SetComponent(GuiComponent *c)
{
	component = c;
	component->GetSize(com_w, com_h);
	if(drawsurface!=NULL) SDL_FreeSurface(drawsurface);
	drawsurface = SDL_CreateRGBSurface(SDL_SWSURFACE, com_w, com_h, SCREENBPP,
			rmask, gmask, bmask, amask);
	component->SetSurface(drawsurface);
	component->SetBasePlace(0, 0);
	component->SetRelPlace(0, 0);
	if(cw-20 >= com_w) w_fits = true;
	else w_fits = false;
	if(ch-20 >= com_h) h_fits = true;
	else h_fits = false;
	position_x = 0;
	position_y = 0;

	int range_vert = com_h, barlength_vert = ch - 20, position_vert = 0;
	int range_horiz = com_w, barlength_horiz = cw - 20, position_horiz = 0;
	if(range_vert < barlength_vert) range_vert = barlength_vert;
	if(range_horiz < barlength_horiz) range_horiz = barlength_horiz;

	scrollbar_vert = new GuiScrollbar(20, ch-20, "vertbar", GUISCROLLBAR_VERT,
			range_vert, barlength_vert, position_vert, fonts);
	scrollbar_horiz = new GuiScrollbar(cw-20, 20, "horizbar", GUISCROLLBAR_HORIZ,
			range_horiz, barlength_horiz, position_horiz, fonts);
	
	scrollbar_vert->SetRelPlace(cw-20, 0);
	scrollbar_horiz->SetRelPlace(0, ch-20);
}

GuiComponent* GuiScrollable::GetComponent()
{
	return component;
}

//////////

GuiImageList::GuiImageList(int w, int h, std::string name, Fonts f,
		std::vector<ImageWithName> images)
	: GuiScrollable(w, h, name, f)
{
	int hborder = 2;
	int wborder = 2;

	int images_combined_w = cw - 20;

	int current_y = hborder;
	int row_current_x = hborder;
	int rowbefore_height = 0;
	{//vc6
	for(unsigned int i=0; i<images.size(); i++){
		//mahtuuko riville yksi vielä?
		if(row_current_x + images[i].s->w < images_combined_w){
			//mahtuu
		}
		else{
			if(row_current_x > images_combined_w) images_combined_w = row_current_x;
			current_y += rowbefore_height + hborder;
			row_current_x = wborder*2;
			rowbefore_height = 0;
		}
		row_current_x += images[i].s->w + wborder;
		if(images[i].s->h > rowbefore_height) rowbefore_height = images[i].s->h;
	}
	}//vc6
	int images_combined_h = current_y + rowbefore_height + hborder;

	GuiContainer *c = new GuiContainer(images_combined_w, images_combined_h,
			"", "", 100, 100, 100, f);

	current_y = hborder;
	row_current_x = hborder;
	rowbefore_height = 0;
	{//vc6
	for(unsigned int i=0; i<images.size(); i++){
		GuiImage *guiimg = new GuiImage(images[i].n, images[i].s, fonts);
		//mahtuuko riville yksi vielä?
		if(row_current_x + images[i].s->w < images_combined_w){
			//mahtuu
		}
		else{
			current_y += rowbefore_height + hborder;
			row_current_x = wborder;
			rowbefore_height = 0;
		}
		c->AddComponent(guiimg, row_current_x, current_y);
		row_current_x += images[i].s->w + wborder;
		if(images[i].s->h > rowbefore_height) rowbefore_height = images[i].s->h;
	}
	}//vc6
	GuiScrollable::SetComponent(c);
}

GuiImageList::~GuiImageList()
{
}

///////////////

GuiGameViewer::GuiGameViewer(int w, int h, std::string name, int a, Map *m,
		ObjectImageLib *olib, TileLib *tilelib_,
		MapImageLib *mapimagelib_, Fonts f)
	: GuiComponent(w, h, name, f)
{
	whattodraw = a;
	floortodraw = -1;
	SetMap(m);
	objectimagelib = olib;
	tilelib = tilelib_;
	mapimagelib = mapimagelib_;
	view_map_x_offset = 0;
	view_map_y_offset = 0;
	rpressed = false;
	lpressed = false;
	lshift = false;
	rshift = false;
	lctrl = false;
	rctrl = false;
	lastbx = -1;
	lastby = -1;
	lasthoverbx = -1;
	lasthoverby = -1;
	/*firsthilight.x = -1;
	firsthilight.y = -1;*/
}

void GuiGameViewer::Draw(int dx, int dy, int dw, int dh, bool is_selected)
{
	DrawGame(surface,
			screen_x, screen_y,
			cw, ch,
			view_map_x_offset, view_map_y_offset,
			map, objectimagelib,
			tilelib, mapimagelib,
			fonts, whattodraw, floortodraw);
}

bool GuiGameViewer::Mouse(int mx, int my, int movex, int movey, bool lbutton, bool rbutton)
{
	bool in_area = (mx >= 0 && my >= 0 && mx < cw && my < ch);
	
	//oikea painike
	
	if(rbutton && !rpressed && in_area){
		rpressed = true;
		rstartx = mx;
		rstarty = my;
		GuiEvent e(cname, GUIEVENT_REQUEST_HIDEMOUSE);
		events.push_back(e);
	}
	else if(!rbutton && rpressed){
		rpressed = false;
		GuiEvent e(cname, GUIEVENT_REQUEST_SHOWMOUSE);
		events.push_back(e);
	}
	if(rpressed){
		view_map_x_offset -= movex;
		view_map_y_offset -= movey;
		GuiEvent e(cname, GUIEVENT_REQUEST_WARPMOUSE);
		e.x = basex + rel_x + rstartx;
		e.y = basey + rel_y + rstarty;
		events.push_back(e);
		return true;
	}
	
	//vasen painike
	
	if(!in_area) return false;
	
	if(!lbutton && lpressed){
		lpressed = false;
		lasthoverbx = -1;
		lasthoverby = -1;
		return true;
	}

	if(lbutton && !lpressed && in_area){
		lpressed = true;
		lmapstartx = view_map_x_offset + mx;
		lmapstarty = view_map_y_offset + my;
		lastbx = -1;
		lastby = -1;
		lasthoverbx = -1;
		lasthoverby = -1;
		int bx, by;
		if(GetBlockBaseInPxPlace(lmapstartx, lmapstarty, bx, by)){
			/*if((lctrl || rctrl) && IsTileHilighted(bx, by)){
				ctrl_on_hilighted = true;
			}
			else{
				ctrl_on_hilighted = false;
			}*/
			/*if(((!lshift && !rshift) || (firsthilight.x==-1 && firsthilight.y==-1))){
				firsthilight.x = bx;
				firsthilight.y = by;
				if(!lctrl && !rctrl) ClearTileHilights();
				AddTileHilight(bx, by);
				SortTileHilights();
			}*/
		}
	}

	if(lbutton && lpressed){
		int bx, by;
		if(!GetBlockBaseInPxPlace(view_map_x_offset + mx, view_map_y_offset + my, bx, by)){
			if(lmapstartx <= map->GetW() * map->GetBlockW()
					&& lmapstarty <= map->GetH() * map->GetBlockH()) return false;
			bx = map->GetW()-1;
			by = map->GetH()-1;
		}
		/*if(lctrl || rctrl){
			if(ctrl_on_hilighted){
				RemoveTileHilight(bx, by);
			}
			else{
				AddTileHilight(bx, by);
				SortTileHilights();
			}
			return true;
		}
		if(lshift || rshift){
			ClearTileHilights();
			AddTileHilightBlock(firsthilight.x, firsthilight.y, bx, by);
			return true;
		}*/
		if(bx != lastbx || by != lastby){
			/*ClearTileHilights();
			AddTileHilight(bx, by);*/
			GuiEvent e(cname, GUIEVENT_MAPVIEWER_BLOCKCLICK);
			e.x = bx;
			e.y = by;
			events.push_back(e);
			lastbx = bx;
			lastby = by;
		}
		return true;
	}
	
	if(in_area){
		int bx, by;
		if(GetBlockBaseInPxPlace(view_map_x_offset + mx, view_map_y_offset + my, bx, by)){
			if(bx != lasthoverbx || by != lasthoverby){
				GuiEvent e(cname, GUIEVENT_MAPVIEWER_BLOCKHOVER);
				e.x = bx;
				e.y = by;
				events.push_back(e);
				lasthoverbx = bx;
				lasthoverby = by;
			}
		}
	}

	return false;
}

bool GuiGameViewer::SDLK(SDLKey key, bool down)
{
	switch(key){
		case SDLK_LSHIFT:
			lshift = down;
			return true;
		case SDLK_RSHIFT:
			rshift = down;
			return true;
		case SDLK_LCTRL:
			lctrl = down;
			return true;
		case SDLK_RCTRL:
			rctrl = down;
			return true;
		default:
			return false;
	}
}

void GuiGameViewer::SetMap(Map *m)
{
	map = m;
}

void GuiGameViewer::SetMapOffset(int x, int y)
{
	view_map_x_offset = x;
	view_map_y_offset = y;
}

void GuiGameViewer::AddMapOffset(int xchange, int ychange)
{
	view_map_x_offset += xchange;
	view_map_y_offset += ychange;
}

void GuiGameViewer::SetWhatToDraw(int a)
{
	whattodraw = a;
}

void GuiGameViewer::SetFloorToDraw(int floortodraw_)
{
	floortodraw = floortodraw_;
}

bool GuiGameViewer::GetBlockBaseInPxPlace(int on_map_x, int on_map_y, int &bx, int &by)
{
	int map_w = map->GetW();
	int map_h = map->GetH();
	int block_w = map->GetBlockW();
	int block_h = map->GetBlockH();
	
	//if(on_map_x < 0 || on_map_y < 0) return false;
	
	bx = on_map_x / block_w;
	by = on_map_y / block_h;

	//std::cout<<"GetBlockBaseInPxPlace("<<on_map_x<<", "<<on_map_y<<", "<<bx<<", "<<by<<")";
	
	if(bx >= 0 && by >= 0 && bx < map_w && by < map_h){
		//std::cout<<" -> true"<<std::endl;
		return true;
	}
	//std::cout<<" -> false"<<std::endl;
	return false;
}

void GuiGameViewer::RollAnims(int timenow_ms)
{
	//std::cout<<"GuiGameViewer::RollAnims("<<timenow_ms<<")"<<std::endl;
	
	//mapimaget
	mapimagelib->RollAnims(timenow_ms);
	
	//objektien imagechunkit
	std::vector<Object*> objects = map->GetObjects();
	for(unsigned int i=0; i < objects.size(); i++){
		objects[i]->RollAnim(timenow_ms);
	}
}
///////////

GuiMessageBox::~GuiMessageBox()
{
}

GuiMessageBox::GuiMessageBox(std::string name, std::string title,
		std::string message, Fonts f)
		: GuiContainer(0, 0, name, title, 50, 25, 10, f)
{
	cw = 500;
	int border = 5;

	ViewableText vtext = GWrapText(fonts.normal, cw-border*2,
			GOPT_WRAP_IN_SPACES, message);
	GuiMultilineLabel *messagelabel = new GuiMultilineLabel("messagelabel_", vtext, fonts);

	GuiTextButton *button = new GuiTextButton(50, 20, "okbutton", "OK", fonts);

	int mw, mh;
	messagelabel->GetSize(mw, mh);

	int bw, bh;
	button->GetSize(bw, bh);

	int w = mw;
	w += border*2;
	int h = mh + bh + border*3;

	SetSize(w, h);

	AddComponent(messagelabel, border, border);
	AddComponent(button, -1, mh + border*2);
}

bool GuiMessageBox::Keyboard(GuiKey key)
{
	char c = key.GetChar();
	if(c=='\n'){
		GuiEvent e("okbutton", GUIEVENT_BUTTONCLICK);
		e.AddName(cname);
		events.push_back(e);
		return true;
	}
	return false;
}

bool GuiMessageBox::GetEvent(GuiEvent *event)
{
	GuiEvent guievent;
	while(GuiContainer::GetEvent(&guievent)){
		if(guievent.type == GUIEVENT_BUTTONCLICK){
			if(guievent.cnames[0] == "okbutton"){
				Delete();
			}
		}
		*event = guievent;
		return true;
	}
	return false;
}


///////////

GuiAskLine::~GuiAskLine()
{
}

GuiAskLine::GuiAskLine(std::string name, std::string title, std::string message, Fonts f)
		: GuiContainer(0, 0, name, title, 25, 10, 50, f)
{
	line = "";
	
	int w = 500;
	int border = 5;

	ViewableText messagevtext = GWrapText(fonts.normal, w-border*2,
			GOPT_WRAP_IN_SPACES, message);
	GuiMultilineLabel *messagelabel = new GuiMultilineLabel("messagelabel_",
			messagevtext, fonts);

	linelabel = new GuiLabel("linelabel_", "> ", fonts);

	GuiTextButton *okbutton = new GuiTextButton(50, 20, "okbutton", "OK", fonts);
	GuiTextButton *cabutton = new GuiTextButton(50, 20, "cabutton", "Cancel", fonts);

	int mw, mh;
	messagelabel->GetSize(mw, mh);

	int lh = TTF_FontHeight(fonts.normal);

	int h = mh + lh + 20 + border*4;

	SetSize(w, h);

	AddComponent(messagelabel, border, border);
	AddComponent(linelabel, border, mh + border*2);
	AddComponent(okbutton, 380, mh + lh + border*3);
	AddComponent(cabutton, 440, mh + lh + border*3);
}

void GuiAskLine::SetLine(std::string text)
{
	line = text;
	linelabel->SetText("> "+line);
}

bool GuiAskLine::Keyboard(GuiKey key)
{
	char c = key.GetChar();
	if(c!=-1){
		if(c=='\b'){
			//std::cout<<"backspace"<<std::endl;
			line = line.substr(0, line.size()-1);
		}
		else if(c=='\n'){
			GuiEvent e("okbutton", GUIEVENT_BUTTONCLICK);
			e.AddName(cname);
			events.push_back(e);
			return true;
		}
		else{
			//std::cout<<"adding '"<<c<<"'"<<std::endl;
			line += c;
		}
		linelabel->SetText("> "+line);
		return true;
	}
	return false;
}

bool GuiAskLine::GetEvent(GuiEvent *event)
{
	GuiEvent guievent;
	while(GuiContainer::GetEvent(&guievent)){
		if(guievent.type == GUIEVENT_BUTTONCLICK){
			if(guievent.cnames[0] == "okbutton"){
				GuiEvent e(cname, GUIEVENT_ASKLINE_RETURN);
				e.text = line;
				*event = e;
				Delete();
				return true;
			}
			if(guievent.cnames[0] == "cabutton"){
				GuiEvent e(cname, GUIEVENT_ASKLINE_CANCEL);
				*event = e;
				Delete();
				return true;
			}
		}
		*event = guievent;
		return true;
	}
	return false;
}

//#############################################################

Gui::Gui(SDL_Surface *surface, Fonts *fonts_, Uint8 r, Uint8 g, Uint8 b)
{
	fonts = fonts_;
	basecontainer = new GuiContainer(surface->w, surface->h,
			"", "", r, g, b, (*fonts_));
	basecontainer->SetSurface(surface);
	basecontainer->SetBasePlace(0, 0);
	basecontainer->SetRelPlace(0, 0);
}

Gui::~Gui()
{
	SDL_Surface *s = basecontainer->GetSurface();
	Uint32 color = SDL_MapRGB(s->format, 0, 0, 0);
	GDrawRect(s, 0, 0, s->w, s->h, color);
	delete basecontainer;
}

void Gui::Draw(int x, int y, int w, int h)
{
	basecontainer->Draw(x, y, w, h, true);
}

bool Gui::Mouse(int x, int y, int movex, int movey, bool lbutton, bool rbutton)
{
	return basecontainer->Mouse(x, y, movex, movey, lbutton, rbutton);
}

bool Gui::Keyboard(GuiKey k)
{
	return basecontainer->Keyboard(k);
}

bool Gui::SDLK(SDLKey key, bool down)
{
	return basecontainer->SDLK(key, down);
}

void Gui::AddComponent(GuiComponent* c, int x, int y)
{
	basecontainer->AddComponent(c, x, y);
}

bool Gui::GetEvent(GuiEvent *event)
{
	if(basecontainer->GetEvent(event)){
		return true;
	}
	return false;
}

void Gui::StartMessageBox(std::string name, std::string title, std::string message)
{
	GuiMessageBox *mb = new GuiMessageBox(name, title, message, (*fonts));
	basecontainer->AddComponent(mb, -1, -1);
}

void Gui::StartAskIntValue(std::string name, std::string title, std::string message)
{
	GuiAskLine *aline = new GuiAskLine(name, title, message, (*fonts));
	basecontainer->AddComponent(aline, -1, -1);
}

