#include "image.h"
#include <iostream>

Way4Image::Way4Image()
{
	lastvalidheading = HEADING_UNDEFINED;
	for(int i=0; i<4; i++) images[i] = NULL;
}

Way4Image::~Way4Image()
{
}

SDL_Surface* Way4Image::GetImage(Heading heading)
{
	//std::cout<<"Way4Image::GetImage("<<heading<<")"<<std::endl;
	//std::cout<<"image pointers are "<<(int)images[0]<<" "<<(int)images[1]<<" "<<(int)images[2]<<" "<<(int)images[3]<<std::endl;
	int headingval = heading.Get4WVal();
	if(headingval != -1) return images[headingval];
	else return NULL;
}

void Way4Image::SetImages(SDL_Surface **images_)
{
	for(int i=0; i<4; i++){
		//std::cout<<"setting image pointer "<<i<<" to "<<(int)images_[i]<<std::endl;
		images[i] = images_[i];
	}
}

void Way4Image::FreeImages()
{
	for(int i=0; i<4; i++){
		if(images[i]!=NULL) SDL_FreeSurface(images[i]);
	}
}

//////////

AnimationImage::AnimationImage()
{
	image = NULL;
}

void AnimationImage::FreeImage()
{
	if(image == NULL) SDL_FreeSurface(image);
}

//////////

AnimationBase::AnimationBase()
{
	ToStart();
}

void AnimationBase::ToStart()
{
	currentimage = 0;
	lastframetime_ms = -1;
}

void AnimationBase::CycleCurrentImage()
{
	currentimage++;
	if(currentimage >= imagebases.size()) currentimage = 0;
}

void AnimationBase::RollFrames(int timenow_ms)
{
	if(imagebases.size()==0){
		currentimage = 0;
		return;
	}
	//jos edellisen framen aika on rutkasti alle nykyisen ajan,
	//korjataan tilanne
	if(lastframetime_ms < timenow_ms - 10000){
		lastframetime_ms = timenow_ms;
		return;
	}
	//jos nykyinen aika on alle edellisen ajan, jotain on käynyt hassusti,
	//joten korjataan tilanne
	if(timenow_ms < lastframetime_ms){
		lastframetime_ms = timenow_ms;
		return;
	}
	int frameduration_ms = imagebases[currentimage].duration_ms;
	//jos on kulunut riittävästi aikaa, loopataan frameja eteen päin
	while(timenow_ms > (lastframetime_ms + frameduration_ms)){
		lastframetime_ms = lastframetime_ms + frameduration_ms;
		CycleCurrentImage();
		frameduration_ms = imagebases[currentimage].duration_ms;
	}
}

//////

SDL_Surface* Way4Animation::GetCurrentImage(Heading heading)
{
	if(currentimage >= images.size()) return NULL;
	return images[currentimage].image.GetImage(heading);
}

void Way4Animation::AddImage(AnimationWay4Image image)
{
	images.push_back(image);
	imagebases.push_back(image);
}

void Way4Animation::FreeImages()
{
	for(unsigned int i=0; i<images.size(); i++){
		images[i].image.FreeImages();
	}
	images.clear();
}

//////

SDL_Surface* Animation::GetCurrentImage()
{
	if(currentimage >= images.size()) return NULL;
	return images[currentimage].image;
}

void Animation::AddImage(AnimationImage image)
{
	images.push_back(image);
	imagebases.push_back(image);
}

void Animation::FreeImages()
{
	for(unsigned int i=0; i<images.size(); i++){
		SDL_FreeSurface(images[i].image);
	}
	images.clear();
}

