#include "../include/map.h"
#include "object.h"
#include "mapimagelib.h"
#include <fstream>
#include <iostream>

MapBlock::MapBlock()
{
	{//vc6
	for(unsigned int i=0; i<MAPBLOCK_SHORTID_CHARS; i++){
		tile_shortid[i] = 0;
	}
	}//vc6
	{//vc6
	for(unsigned int i=0; i<MAPBLOCK_MOVE_SHORTID_CHARS; i++){
		move_shortid[i] = 0;
	}
	}//vc6
	y_change_per_10x_right = 0;
	special_info = 0;

	hilighted = false;
}

//#################################################################

MapImageInfo::MapImageInfo()
{
	{//vc6
	for(unsigned int i=0; i<MAPIMAGE_SHORTID_CHARS; i++){
		shortid[i] = 0;
	}
	}//vc6
	x=-1;
	y=-1;
	
	hilighted = false;
}

bool MapImageInfo::IsImageInBlock(int bx, int by, MapImageLib *lib)
{
	int w, h;
	lib->GetImage(shortid)->GetSizeTiles(w, h);
	return (x <= bx && y <= by && x + w > bx && y + h > by);
}

//#################################################################

MapFloor::MapFloor()
{
	mapblocks = NULL;
}

MapFloor::~MapFloor()
{
	UnAllocate();
}

void MapFloor::UnAllocate()
{
	std::cout<<"MapFloor::UnAllocate()"<<std::endl;
	if(mapblocks != NULL){
		for(int y=0; y<mapheight; y++){
			free(mapblocks[y]);
		}
		free(mapblocks);
		mapblocks = NULL;
		mapwidth = 0;
		mapheight = 0;
	}
	imageinfos.clear();
}

void MapFloor::Allocate(int w, int h)
{
	std::cout<<"MapFloor::Allocate("<<w<<","<<h<<")"<<std::endl;
	if(mapblocks != NULL) UnAllocate();
	mapblocks = (MapBlock**)malloc(sizeof(MapBlock*) * h);
	for(int y=0; y<h; y++){
		mapblocks[y] = (MapBlock*)malloc(sizeof(MapBlock) * w);
		for(int x=0; x<w; x++){
			MapBlock b;
			mapblocks[y][x] = b;
		}
	}
	mapwidth = w;
	mapheight = h;
}

MapBlock *MapFloor::GetBlock(int x, int y)
{
	return &mapblocks[y][x];
}

void MapFloor::SetBlock(int x, int y, MapBlock mapblock)
{
	mapblocks[y][x] = mapblock;
}

std::vector<MapImageInfo> & MapFloor::GetMapImageInfos()
{
	return imageinfos;
}

void MapFloor::AddMapImageInfo(MapImageInfo info)
{
	imageinfos.push_back(info);
}

std::vector<int> MapFloor::GetImagesInBlock(int x, int y, MapImageLib *lib)
{
	std::vector<int> imgs;
	for(unsigned int i=0; i<imageinfos.size(); i++){
		if(imageinfos[i].IsImageInBlock(x, y, lib)){
			imgs.push_back(i);
		}
	}
	return imgs;
}

void MapFloor::RemoveImages(std::vector<int> imgs)
{
	for(unsigned int i=0; i<imgs.size(); i++){
		imageinfos.erase(imageinfos.begin()+imgs[i]);
		for(unsigned int a=i; a<imgs.size(); a++){
			imgs[a]--;
		}
	}
}

std::vector<MapNamedPlace> & MapFloor::GetNamedPlaces()
{
	return namedplaces;
}

void MapFloor::AddNamedPlace(MapNamedPlace place)
{
	namedplaces.push_back(place);
}

std::vector<int> MapFloor::GetNamedPlacesInBlock(int x, int y)
{
	std::vector<int> places;
	for(unsigned int i=0; i<namedplaces.size(); i++){
		MapNamedPlace p = namedplaces[i];
		if(p.xy.x == x && p.xy.y == y) places.push_back(i);
	}
	return places;
}

void MapFloor::RemoveNamedPlaces(std::vector<int> places)
{
	for(unsigned int i=0; i<places.size(); i++){
		namedplaces.erase(namedplaces.begin()+places[i]);
		for(unsigned int a=i; a<places.size(); a++){
			places[a]--;
		}
	}
}

MapNamedPlace MapFloor::GetNamedPlace(std::string name)
{
	for(unsigned int i=0; i<namedplaces.size(); i++){
		if((std::string)namedplaces[i].name == name){
			return namedplaces[i];
		}
	}
	MapNamedPlace p;
	p.name[0] = 0;
	p.xy.x = -1;
	p.xy.y = -1;
	p.heading = HEADING_UNDEFINED;
	return p;
}

std::vector<XYPlace> MapFloor::GetBlockHilights()
{
	std::vector<XYPlace> hls;
	for(int y=0; y<mapheight; y++){
		for(int x=0; x<mapwidth; x++){
			if(mapblocks[y][x].hilighted){
				XYPlace p;
				p.y = y;
				p.x = x;
				hls.push_back(p);
			}
		}
	}
	return hls;
}

bool MapFloor::IsBlockHilighted(int x, int y)
{
	return mapblocks[y][x].hilighted;
}

void MapFloor::ClearBlockHilights()
{
	for(int y=0; y<mapheight; y++){
		for(int x=0; x<mapwidth; x++){
			mapblocks[y][x].hilighted = false;
		}
	}
}

void MapFloor::AddBlockHilight(int x, int y)
{
	mapblocks[y][x].hilighted = true;
}

void MapFloor::RemoveBlockHilight(int x, int y)
{
	mapblocks[y][x].hilighted = false;
}

void MapFloor::AddBlockHilightBlock(int x01, int y01, int x02, int y02)
{
	int x1 = x01 < x02 ? x01 : x02;
	int x2 = x01 > x02 ? x01 : x02;
	int y1 = y01 < y02 ? y01 : y02;
	int y2 = y01 > y02 ? y01 : y02;
	for(int y=y1; y<=y2; y++){
		for(int x=x1; x<=x2; x++){
			AddBlockHilight(x, y);
		}
	}
}

std::vector<int> MapFloor::GetImageHilights()
{
	std::vector<int> hls;
	for(unsigned int i=0; i<imageinfos.size(); i++){
		if(imageinfos[i].hilighted) hls.push_back(i);
	}
	return hls;
}

bool MapFloor::IsImageHilighted(int i)
{
	return imageinfos[i].hilighted;
}

void MapFloor::ClearImageHilights()
{
	for(unsigned int i=0; i<imageinfos.size(); i++){
		imageinfos[i].hilighted = false;
	}
}

void MapFloor::AddImageHilight(int i)
{
	imageinfos[i].hilighted = true;
}

void MapFloor::RemoveImageHilight(int i)
{
	imageinfos[i].hilighted = false;
}

//#################################################################

Map::Map()
{
	mapwidth = 0;
	mapheight = 0;
	blockw = 0;
	blockh = 0;
}

Map::~Map()
{
	UnAllocate();
}

int Map::GetW()
{
	return mapwidth;
}

int Map::GetH()
{
	return mapheight;
}

int Map::GetBlockW()
{
	return blockw;
}

int Map::GetBlockH()
{
	return blockh;
}

int Map::GetPxW()
{
	return mapwidth * blockw;
}

int Map::GetPxH()
{
	return mapheight * blockh;
}

void Map::SetBlockSize(int w, int h)
{
	blockw = w;
	blockh = h;
}

void Map::Allocate(int w, int h)
{
	std::cout<<"Map::Allocate("<<w<<","<<h<<")"<<std::endl;
	mapwidth = w;
	mapheight = h;
}

void Map::Allocate(int w, int h, int bw, int bh)
{
	Allocate(w, h);
	SetBlockSize(bw, bh);
}

void Map::UnAllocate()
{
	for(unsigned int i=0; i<floors.size(); i++){
		floors[i]->UnAllocate();
		delete floors[i];
	}
	floors.clear();
	objects.clear();
}

MapFloor* Map::AddFloorTop()
{
	MapFloor *floor = new MapFloor;
	floor->Allocate(mapwidth, mapheight);
	floors.push_back(floor);
	return floors[floors.size()-1];
}

MapFloor* Map::AddFloorBottom()
{
	MapFloor *floor = new MapFloor;
	floor->Allocate(mapwidth, mapheight);
	floors.insert(floors.begin(), floor);
	return floors[0];
}

bool Map::RemoveFloorTop()
{
	if(floors.size()<=1) return false;
	(*(floors.end()-1))->UnAllocate();
	floors.erase(floors.end()-1);
	return true;
}

bool Map::RemoveFloorBottom()
{
	if(floors.size()<=1) return false;
	(*floors.begin())->UnAllocate();
	floors.erase(floors.begin());
	return true;
}

std::vector<MapFloor*>& Map::GetFloors()
{
	return floors;
}

unsigned int Map::FloorCount()
{
	return floors.size();
}

MapFloor* Map::GetFloor(unsigned int i)
{
	if(i >= floors.size()) return NULL;
	return floors[i];
}


int Map::LoadFromFile(std::string filename)
{
	std::cout<<"loading map from file "<<filename<<std::endl;
	
	std::fstream file(filename.c_str(), std::fstream::in);

	if(!file.good()){
		std::cout<<"failed to open file "<<filename<<" for reading"<<std::endl;
		return 1;
	}
	
	//MapInfo-strukti
	MapInfo mapinfo;
	file.read((char*)&mapinfo, sizeof(MapInfo));

	if(mapinfo.version == 1){
		std::cout<<"not compatible with version 1 maps"<<std::endl;
		return 1;
	}

	Allocate(mapinfo.width, mapinfo.height, mapinfo.blockw, mapinfo.blockh);

	for(unsigned int i=0; i<mapinfo.floorcount; i++){
		MapFloor *floor = AddFloorTop();
		MapFloorInfo mfi;
		file.read((char*)&mfi, sizeof(MapFloorInfo));
		
		{for(unsigned int y=0; y<mapinfo.height; y++){
			for(unsigned int x=0; x<mapinfo.width; x++){
				MapBlock mapblock;
				file.read((char*)&mapblock, sizeof(SaveMapBlock));
				if(mapinfo.version == 2) mapblock.move_shortid[0] = 0;
				floor->SetBlock(x, y, mapblock);
			}
		}}

		//image-taulukko (MapImageInfo-strukteja)
		{for(unsigned int i=0; i<mfi.imagecount; i++){
			MapImageInfo info;
			file.read((char*)&info, sizeof(SaveMapImageInfo));
			floor->AddMapImageInfo(info);
		}}

		//MapNamedPlaceja
		{for(unsigned int i=0; i<mfi.namedplacecount; i++){
			MapNamedPlace place;
			file.read((char*)&place, sizeof(MapNamedPlace));
			floor->AddNamedPlace(place);
		}}
	}

	file.close();

	return 0;
}

int Map::WriteToFile(std::string filename)
{
	std::cout<<"writing map to file "<<filename<<std::endl;

	std::fstream file(filename.c_str(), std::fstream::out);
	
	if(!file.good()){
		std::cout<<"failed to open file "<<filename<<" for writing"<<std::endl;
		return 1;
	}
	
	//MapInfo-strukti
	MapInfo mapinfo;
	mapinfo.version = 3;
	mapinfo.width  = mapwidth;
	mapinfo.height = mapheight;
	mapinfo.blockw = blockw;
	mapinfo.blockh = blockh;
	mapinfo.floorcount = floors.size();
	mapinfo.nothing1 = 0;
	mapinfo.nothing2 = 0;
	mapinfo.nothing3 = 0;

	file.write((char*)&mapinfo, sizeof(MapInfo));

	for(unsigned int i=0; i<mapinfo.floorcount; i++){
		MapFloor *floor = floors[i];
		MapFloorInfo mfi;
		mfi.imagecount = floor->GetMapImageInfos().size();
		mfi.namedplacecount = floor->GetNamedPlaces().size();
		mfi.nothing1 = 0;
		mfi.nothing2 = 0;
		mfi.nothing3 = 0;
		
		file.write((char*)&mfi, sizeof(MapFloorInfo));
		
		{for(unsigned int y=0; y<mapinfo.height; y++){
			for(unsigned int x=0; x<mapinfo.width; x++){
				file.write((char*)floor->GetBlock(x, y), sizeof(SaveMapBlock));
			}
		}}

		//image-taulukko (MapImageInfo-strukteja)
		{for(unsigned int i=0; i<mfi.imagecount; i++){
			file.write((char*)&floor->GetMapImageInfos()[i], sizeof(SaveMapImageInfo));
		}}

		//MapNamedPlaceja
		{for(unsigned int i=0; i<mfi.namedplacecount; i++){
			file.write((char*)&floor->GetNamedPlaces()[i], sizeof(MapNamedPlace));
		}}
	}

	file.close();

	return 0;
}

std::vector<Object*> & Map::GetObjects()
{
	return objects;
}

void Map::AddObject(Object *object)
{
	objects.push_back(object);
}


