#include "mapeditor.h"
#include "gameviewer.h"
#include "gui.h"
#include <iostream>
#include <string.h>
#include <stdio.h>

/*
 * tiilien organisointi:
 * "<maatyyppi>,<lisäinfo>,<num>.png"
 * - maatyyppi:
 *   - esim. "grass", "rock"
 * - lisäinfo:
 *   - "normal"
 *   - "ct_<toinen-maatyyppi>"
 *     - ct = change to, maatyypin muuttuminen
 *   - joku muu juttu, esim threshold (kynnys) tai whatever
 * - num:
 *   - järjestysnumero, alkaa nollasta
 *
 * esim. "grass,ct_water,0.png"
 *       "darkgbrick,normal,0.png"
 */

MapEditor::MapEditor()
{
	initialized = false;
}

MapEditor::MapEditor(SDL_Surface *s, Fonts *f, ObjectImageLib *o, TileLib *tilelib_,
		TileLib *movetilelib_, MapImageLib *mapimagelib_, std::string filename_)
{
	screen = s;
	fonts = f;
	objectimagelib = o;
	tilelib = tilelib_;
	movetilelib = movetilelib_;
	mapimagelib = mapimagelib_;
	filename = filename_;
	initialized = true;
}

void MapEditor::SetThings(SDL_Surface *s, Fonts *f, ObjectImageLib *o, TileLib *tilelib_,
		TileLib *movetilelib_, MapImageLib *mapimagelib_, std::string filename_)
{
	screen = s;
	fonts = f;
	objectimagelib = o;
	tilelib = tilelib_;
	movetilelib = movetilelib_;
	mapimagelib = mapimagelib_;
	filename = filename_;
	initialized = true;
}

int MapEditor::Main()
{
	std::cout<<"MapEditor::Main(\""<<filename<<"\")"<<std::endl;
	if(!initialized){
		std::cout<<"MapEditor::Main(): not initialized"<<std::endl;
		return 1;
	}
	bool run = true;

	Map map;
	if(map.LoadFromFile(filename)){
		std::cout<<"error loading map from "<<filename<<std::endl;
		std::cout<<"creating new"<<std::endl;
		
		//tehdään joku zucka kenttä

		int w = 30, h = 30;
		map.Allocate(w, h, DEFAULTBLOCKW, DEFAULTBLOCKH);
		MapFloor *floor0 = map.AddFloorTop();
		MapFloor *floor1 = map.AddFloorTop();

		MapBlock b;

		sprintf(b.tile_shortid, "water,normal,0");
		{for(int y=0; y<3; y++){
			for(int x=0; x<w; x++){
				floor0->SetBlock(x, y, b);
			}
		}}
		sprintf(b.tile_shortid, "lightgroundbrick,normal,0");
		{for(int y=3; y<12; y++){
			for(int x=0; x<w; x++){
				floor0->SetBlock(x, y, b);
			}
		}}
		sprintf(b.tile_shortid, "grass,normal,0");
		{for(int y=12; y<h; y++){
			for(int x=0; x<w; x++){
				floor0->SetBlock(x, y, b);
			}
		}}

		MapImageInfo info;
		sprintf(info.shortid, "fountain0");
		
		info.x = 5;
		info.y = 3;
		floor0->AddMapImageInfo(info);
	}

	int mousex=0, mousey=0;
	SDL_GetMouseState(&mousex, &mousey);
	int oldmousex=mousex, oldmousey=mousey;
	bool leftbutton_down = false, rightbutton_down = false;
	bool lshift = false, rshift = false;
	bool lctrl = false, rctrl = false;

	Gui gui(screen, fonts, 0, 0, 0);
	
	std::string infotext = "== Map Editor ==\n"
			"right button + move = move map\n"
			"shift and ctrl for selecting";
	ViewableText infovtext = GWrapText(fonts->normal, 300,
			GOPT_WRAP_IN_SPACES, infotext);
	GuiMultilineLabel *infolabel = new GuiMultilineLabel("infolabel", infovtext, (*fonts));
	gui.AddComponent(infolabel, 0, 0);

	GuiTextButton *button;
	button = new GuiTextButton(95, 20, "moib", "Test Button", (*fonts));
	gui.AddComponent(button, 230, 5);
	button = new GuiTextButton(45, 20, "save", "Save", (*fonts));
	gui.AddComponent(button, 230, 30);
	button = new GuiTextButton(45, 20, "quit", "Quit", (*fonts));
	gui.AddComponent(button, 280, 30);
	button = new GuiTextButton(45, 20, "togglegrid", "Grid", (*fonts));
	gui.AddComponent(button, 330, 5);
	GuiTextButton *tfloorb;
	tfloorb = new GuiTextButton(145, 20, "toggleshowallfloors", "Showing All Floors", (*fonts));
	gui.AddComponent(tfloorb, 330, 30);

	int rmargin_width = 150;
	
	int i=-15;

	GuiLabel *curfloorlabel = new GuiLabel("curfloorlabel", "<current floor>", (*fonts));
	gui.AddComponent(curfloorlabel, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "floorup", "Floor+", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "floordown", "Floor-", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width/2-5, 12, "flooraddtop", "Add Top", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width/2-5, 12, "flooraddbot", "Add Bot", (*fonts));
	gui.AddComponent(button, 800-rmargin_width+rmargin_width/2, i);
	button = new GuiTextButton(rmargin_width/2-5, 12, "floorremtop", "Rem Top", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width/2-5, 12, "floorrembot", "Rem Bot", (*fonts));
	gui.AddComponent(button, 800-rmargin_width+rmargin_width/2, i);
	i+=3;
	GuiLabel *othermodelabel = new GuiLabel("othermodelabel", "<other mode>", (*fonts));
	gui.AddComponent(othermodelabel, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "othermode_add", "Add", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "othermode_remove", "Remove", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "othermode_select", "Select", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	i+=3;
	GuiLabel *editmodelabel = new GuiLabel("editmodelabel", "<edit mode>", (*fonts));
	gui.AddComponent(editmodelabel, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "editmode_tiles", "Tiles", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "editmode_movetiles", "Movetiles", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "editmode_mapimages", "MapImages", (*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	button = new GuiTextButton(rmargin_width, 12, "editmode_namedplaces","NamedPlaces",(*fonts));
	gui.AddComponent(button, 800-rmargin_width, i+=13);
	i+=3;
	GuiLabel *seltilelabel = new GuiLabel("seltilelabel", "<selected tile>", (*fonts));
	gui.AddComponent(seltilelabel, 800-rmargin_width, i+=13);
	GuiLabel *selmapilabel = new GuiLabel("selmapilabel", "<selected mapimage>", (*fonts));
	gui.AddComponent(selmapilabel, 800-rmargin_width, i+=13);
	
	bool tfloorblabel_updated = false;
	bool editmodelabel_updated = false;
	bool othermodelabel_updated = false;
	bool curfloorlabel_updated = false;
	bool seltilelabel_updated = false;
	bool selmapilabel_updated = false;

enum{
	EDITMODE_NOTSET,
	EDITMODE_TILES,
	EDITMODE_MOVETILES,
	EDITMODE_MAPIMAGES,
	EDITMODE_NAMEDPLACES
};
enum{
	OTHERMODE_ADD,
	OTHERMODE_REMOVE,
	OTHERMODE_SELECT
};
	int editmode = EDITMODE_NOTSET;
	int othermode = OTHERMODE_ADD;

	unsigned int current_floor_i = 0;
	bool showingallfloors = true;

	std::string selected_tileid;
	std::string selected_movetileid;
	std::string selected_mapimageid;

	XYPlace selectstartplace;
	selectstartplace.x = -1;
	selectstartplace.y = -1;
	XYPlace selectendplace;
	bool select_started = false;
	bool select_finalized = false;
	
	///////////////////////////////////////////////////////////////
	
	i += 20;
	int listheightpx = 600 - i;
	int listi = i;
	
	std::vector<ImageWithName> images;
		
	//tiles
	
	std::vector<std::string> tileids = tilelib->ListIDs();
	
	images.clear();
	{for(unsigned int i=0; i<tileids.size(); i++){
		ImageWithName image;
		image.n = tileids[i];
		image.s = tilelib->GetImage(tileids[i]);
		images.push_back(image);
	}}

	GuiImageList *tilelist;
	tilelist = new GuiImageList(rmargin_width, listheightpx, "tilelist", (*fonts), images);
	gui.AddComponent(tilelist, 800-rmargin_width, listi);
	tilelist->Disable();

	//movetiles
	
	std::vector<std::string> movetileids = movetilelib->ListIDs();
	
	images.clear();
	{for(unsigned int i=0; i<movetileids.size(); i++){
		ImageWithName image;
		image.n = movetileids[i];
		image.s = movetilelib->GetImage(movetileids[i]);
		images.push_back(image);
	}}

	GuiImageList *movetilelist;
	movetilelist = new GuiImageList(rmargin_width, listheightpx, "movetilelist", (*fonts), images);
	gui.AddComponent(movetilelist, 800-rmargin_width, listi);
	movetilelist->Disable();

	//mapimages
	
	std::vector<std::string> mapimageids = mapimagelib->ListIDs();
	
	images.clear();
	{for(unsigned int i=0; i<mapimageids.size(); i++){
		ImageWithName image;
		image.n = mapimageids[i];
		image.s = mapimagelib->GetImage(mapimageids[i])->GetUnderAnim().GetCurrentImage();
		GDrawIMG(image.s, mapimagelib->GetImage(mapimageids[i])
				->GetOverAnim().GetCurrentImage(), 0, 0);
		images.push_back(image);
	}}
	
	GuiImageList *mapimagelist;
	mapimagelist = new GuiImageList(rmargin_width,listheightpx, "mapimagelist", (*fonts), images);
	gui.AddComponent(mapimagelist, 800-rmargin_width, listi);
	mapimagelist->Disable();
	

	/////////////////////////////////////////////////////////////////////

	int gv_whattodraw = 0;
	GuiGameViewer *viewer;
	viewer = new GuiGameViewer(800-rmargin_width-4, 543, "mapviewer", gv_whattodraw, &map,
			objectimagelib, tilelib, mapimagelib, (*fonts));
	gui.AddComponent(viewer, 2, 55);
	
	////////////////////////////////////////////////////////////////////
	
	bool asked_save = false;
	int adding_namedplace_state = 0;
	MapNamedPlace temp_namedplace;

	////////////////////////////////////////////////////////////////

	int lastticks = SDL_GetTicks();;
	int wanted_time = 1000/30;
	while(run){
		//frameraten ylläpito
		int ticksnow = SDL_GetTicks();
		if(ticksnow < lastticks) lastticks = ticksnow;
		int lasttime = ticksnow - lastticks;
		int timetosleep = wanted_time - lasttime;
		if(timetosleep > 0) SDL_Delay(timetosleep);
		lastticks = SDL_GetTicks();

		//jotain juttuja

		MapFloor *current_floor = map.GetFloor(current_floor_i);
		
		if(!tfloorblabel_updated){
			tfloorblabel_updated = true;
		}
		if(!editmodelabel_updated){
			switch(editmode){
			case EDITMODE_NOTSET:
				editmodelabel->SetText("EDITMODE_NOTSET");
				break;
			case EDITMODE_TILES:
				editmodelabel->SetText("EDITMODE_TILES");
				break;
			case EDITMODE_MOVETILES:
				editmodelabel->SetText("EDITMODE_TILES");
				break;
			case EDITMODE_MAPIMAGES:
				editmodelabel->SetText("EDITMODE_MAPIMAGES");
				break;
			case EDITMODE_NAMEDPLACES:
				editmodelabel->SetText("EDITMODE_NAMEDPLACES");
				break;
			};
			editmodelabel_updated = true;
		}
		if(!othermodelabel_updated){
			switch(othermode){
			case OTHERMODE_ADD:
				othermodelabel->SetText("OTHERMODE_ADD");
				break;
			case OTHERMODE_REMOVE:
				othermodelabel->SetText("OTHERMODE_REMOVE");
				break;
			case OTHERMODE_SELECT:
				othermodelabel->SetText("OTHERMODE_SELECT");
				break;
			}
			othermodelabel_updated = true;
		}
		if(!curfloorlabel_updated){
			char c[100];
			c[0] = 0;
			for(unsigned int i=0; i<map.FloorCount(); i++){
				if(showingallfloors || i==current_floor_i)
					sprintf(c+strlen(c), "(");
				if(i!=current_floor_i) sprintf(c+strlen(c), "%i", i);
				else sprintf(c+strlen(c), "[%i]", i);
				if(showingallfloors || i==current_floor_i)
					sprintf(c+strlen(c), ")");
				sprintf(c+strlen(c), " ");
			}
			//sprintf(c, "Editing floor %i", current_floor_i);
			curfloorlabel->SetText(c);
			curfloorlabel_updated = true;
		}
		if(!seltilelabel_updated){
			seltilelabel->SetText("Tile: "+selected_tileid);
			seltilelabel_updated = true;
		}
		if(!selmapilabel_updated){
			selmapilabel->SetText("MapImage: "+selected_mapimageid);
			selmapilabel_updated = true;
		}

		//SDL:n eventit
		
		int mousemovex = 0;
		int mousemovey = 0;
		
		SDL_Event sdlevent;
		while(SDL_PollEvent(&sdlevent) && run){
			if(sdlevent.type == SDL_QUIT){
				return -1;
			}
			if(sdlevent.type == SDL_KEYDOWN){
				gui.SDLK(sdlevent.key.keysym.sym, true);
				switch(sdlevent.key.keysym.sym){
					case SDLK_LSHIFT:
						lshift = true;
						break;
					case SDLK_RSHIFT:
						rshift = true;
						break;
					case SDLK_LCTRL:
						lctrl = true;
						break;
					case SDLK_RCTRL:
						rctrl = true;
						break;
					default:
						break;
				}
				bool did = false;
				GuiKey k(sdlevent.key.keysym.sym, lshift || rshift);
				did = gui.Keyboard(k);
			}
			if(sdlevent.type == SDL_KEYUP){
				gui.SDLK(sdlevent.key.keysym.sym, false);
				switch(sdlevent.key.keysym.sym){
					case SDLK_ESCAPE:
						run = false;
						break;
					case SDLK_LSHIFT:
						lshift = false;
						break;
					case SDLK_RSHIFT:
						rshift = false;
						break;
					case SDLK_LCTRL:
						lctrl = false;
						break;
					case SDLK_RCTRL:
						rctrl = false;
						break;
					/*case SDLK_q:
						run = false;
						break;
					case SDLK_s:
						std::cout<<"saving map to "<<filename<<std::endl;
						if(map.WriteToFile(filename)){
							std::cout<<"error saving map to "<<filename<<std::endl;
						}
						break;*/
					default:
						break;
				}
			}
			if(sdlevent.type == SDL_MOUSEMOTION){
				oldmousex = mousex;
				oldmousey = mousey;
				mousex = sdlevent.motion.x;
				mousey = sdlevent.motion.y;
				mousemovex += mousex - oldmousex;
				mousemovey += mousey - oldmousey;
			}
			if(sdlevent.type == SDL_MOUSEBUTTONDOWN){
				if(sdlevent.button.button == SDL_BUTTON_LEFT){
					leftbutton_down = true;
				}
				if(sdlevent.button.button == SDL_BUTTON_RIGHT){
				rightbutton_down = true;
				}
			}
			if(sdlevent.type == SDL_MOUSEBUTTONUP){
				if(sdlevent.button.button == SDL_BUTTON_LEFT){
					leftbutton_down = false;
				}
				if(sdlevent.button.button == SDL_BUTTON_RIGHT){
					rightbutton_down = false;
				}
			}
		}
		
		//tehdään juttuja eventtien mukaan
		
		//syötetään guille hiiren tiedot
		gui.Mouse(mousex, mousey, mousemovex, mousemovey, leftbutton_down, rightbutton_down);
		
		//Gui:n eventit
		
		GuiEvent guievent;
		while(gui.GetEvent(&guievent)){
			if(guievent.type == GUIEVENT_BUTTONCLICK){
				if(guievent.cnames[0]=="save" && !asked_save){
					GuiAskLine *aline = new GuiAskLine("asksavefilename", "Filename",
							"Give filename", (*fonts));
					aline->SetLine(filename);
					gui.AddComponent(aline, -1, -1);
					asked_save = true;
				}
				if(guievent.cnames[0]=="quit"){
					run = false;
				}
				if(guievent.cnames[0]=="moib"){
					GuiAskLine *aline = new GuiAskLine("aline1", "Button pressed!",
							"What have you just done?", (*fonts));
					gui.AddComponent(aline, -1, -1);
				}
				if(guievent.cnames[0]=="togglegrid"){
					gv_whattodraw ^= DGDRAW_GRID;
					viewer->SetWhatToDraw(gv_whattodraw);
				}
				if(guievent.cnames[0]=="toggleshowallfloors"){
					if(showingallfloors){
						showingallfloors = false;
						tfloorb->SetLabel("Showing This Floor");
						curfloorlabel_updated = false;
					}
					else{
						showingallfloors = true;
						tfloorb->SetLabel("Showing All Floors");
						curfloorlabel_updated = false;
					}
				}
				if(guievent.cnames[0].substr(0,9)=="editmode_"){
					tilelist->Disable();
					movetilelist->Disable();
					mapimagelist->Disable();

					gv_whattodraw &= ~DGDRAW_NAMEDPLACES;

					std::string smode = guievent.cnames[0].substr(9);

					if(smode=="tiles"){
						editmode = EDITMODE_TILES;
						tilelist->Enable();
						editmodelabel_updated = false;
					}
					else if(smode=="movetiles"){
						editmode = EDITMODE_MOVETILES;
						movetilelist->Enable();
						editmodelabel_updated = false;
					}
					else if(smode=="mapimages"){
						editmode = EDITMODE_MAPIMAGES;
						mapimagelist->Enable();
						editmodelabel_updated = false;
					}
					else if(smode=="namedplaces"){
						editmode = EDITMODE_NAMEDPLACES;
						gv_whattodraw |= DGDRAW_NAMEDPLACES;
						editmodelabel_updated = false;
					}

					viewer->SetWhatToDraw(gv_whattodraw);
				}
				if(guievent.cnames[0].substr(0,10)=="othermode_"){
					std::string smode = guievent.cnames[0].substr(10);

					if(smode=="add"){
						othermode = OTHERMODE_ADD;
						othermodelabel_updated = false;
					}
					else if(smode=="remove"){
						othermode = OTHERMODE_REMOVE;
						othermodelabel_updated = false;
					}
					else if(smode=="select"){
						othermode = OTHERMODE_SELECT;
						othermodelabel_updated = false;
						select_started = false;
						current_floor->ClearBlockHilights();
					}
				}
				if(guievent.cnames[0]=="floorup"){
					if(current_floor_i < map.FloorCount() - 1){
						current_floor_i++;
						current_floor->ClearBlockHilights();
						curfloorlabel_updated = false;
					}
				}
				if(guievent.cnames[0]=="floordown"){
					if(current_floor_i > 0){
						current_floor_i--;
						current_floor->ClearBlockHilights();
						curfloorlabel_updated = false;
					}
				}
				if(guievent.cnames[0]=="flooraddtop"){
					map.AddFloorTop();
					curfloorlabel_updated = false;
				}
				if(guievent.cnames[0]=="flooraddbot"){
					map.AddFloorBottom();
					current_floor_i++;
					curfloorlabel_updated = false;
				}
				if(guievent.cnames[0]=="floorremtop"){
					if(map.RemoveFloorTop()){
						curfloorlabel_updated = false;
					}
				}
				if(guievent.cnames[0]=="floorrembot"){
					if(map.RemoveFloorBottom()){
						current_floor_i--;
						curfloorlabel_updated = false;
					}
				}
			}
			
			if(guievent.type == GUIEVENT_IMAGECLICK){
				if(guievent.cnames[1]=="tilelist"){
					if(othermode == OTHERMODE_SELECT){
						for(int y=0; y<map.GetH(); y++){
							for(int x=0; x<map.GetW(); x++){
								MapBlock *b = current_floor->GetBlock(x, y);
								if(b->hilighted){
									sprintf(b->tile_shortid, "%s", guievent.cnames[0].c_str());
								}
							}
						}
					}
					else{
						selected_tileid = guievent.cnames[0];
						seltilelabel_updated = false;
						othermode = OTHERMODE_ADD;
						othermodelabel_updated = false;
					}
				}
				if(guievent.cnames[1]=="mapimagelist"){
					selected_mapimageid = guievent.cnames[0];
					seltilelabel_updated = false;
					othermode = OTHERMODE_ADD;
					othermodelabel_updated = false;
				}
			}
			else if(guievent.type == GUIEVENT_MAPVIEWER_BLOCKCLICK){
				std::cout<<"map block "<<guievent.x<<","<<guievent.y<<" clicked"<<std::endl;
				MapBlock *b = current_floor->GetBlock(guievent.x, guievent.y);
				if(othermode == OTHERMODE_SELECT){
					if(lshift || rshift){
						if(!select_started){
							select_started = true;
							selectstartplace.x = guievent.x;
							selectstartplace.y = guievent.y;
							select_finalized = false;
							current_floor->AddBlockHilight(guievent.x, guievent.y);
						}
						else{
							current_floor->ClearBlockHilights();
							selectendplace.x = guievent.x;
							selectendplace.y = guievent.y;
							select_finalized = true;
							int x1 = (selectstartplace.x<selectendplace.x?
									selectstartplace.x:selectendplace.x);
							int x2 = (selectstartplace.x>selectendplace.x?
									selectstartplace.x:selectendplace.x);
							int y1 = (selectstartplace.y<selectendplace.y?
									selectstartplace.y:selectendplace.y);
							int y2 = (selectstartplace.y>selectendplace.y?
									selectstartplace.y:selectendplace.y);
							for(int y=y1; y<=y2; y++){
								for(int x=x1; x<=x2; x++){
									current_floor->AddBlockHilight(x, y);
								}
							}
						}
					}
					else if(lctrl || rctrl){
						current_floor->AddBlockHilight(guievent.x, guievent.y);
						select_started = false;
					}
					else{
						current_floor->ClearBlockHilights();
						select_started = true;
						selectstartplace.x = guievent.x;
						selectstartplace.y = guievent.y;
						select_finalized = false;
						current_floor->AddBlockHilight(guievent.x, guievent.y);
					}
				}
				else{
					current_floor->ClearBlockHilights();
					switch(editmode){
					case EDITMODE_TILES:
						/*
						switch(othermode){
						case OTHERMODE_ADD:
							break;
						case OTHERMODE_REMOVE:
							break;
						}
						*/
						switch(othermode){
						case OTHERMODE_ADD:
							sprintf(b->tile_shortid, "%s", selected_tileid.c_str());
							break;
						case OTHERMODE_REMOVE:
							b->tile_shortid[0] = 0;
							break;
						}
						break;
					case EDITMODE_MOVETILES:
						switch(othermode){
						case OTHERMODE_ADD:
							sprintf(b->move_shortid, "%s", selected_movetileid.c_str());
							break;
						case OTHERMODE_REMOVE:
							b->move_shortid[0] = 0;
							break;
						}
						break;
					case EDITMODE_MAPIMAGES:
						switch(othermode){
						case OTHERMODE_ADD:
							if(selected_mapimageid!=""){
								MapImageInfo info;
								sprintf(info.shortid, "%s", selected_mapimageid.c_str());
								info.x = guievent.x;
								info.y = guievent.y;
								std::cout<<"adding"<<std::endl;
								current_floor->AddMapImageInfo(info);
								std::cout<<"added"<<std::endl;
							}
							break;
						case OTHERMODE_REMOVE:
							std::vector<int> imgs;
							imgs = current_floor->GetImagesInBlock(guievent.x, guievent.y, mapimagelib);
							current_floor->RemoveImages(imgs);
							break;
						}
						break;
					case EDITMODE_NAMEDPLACES:
						switch(othermode){
						case OTHERMODE_ADD:
							if(adding_namedplace_state == 0){
								temp_namedplace.xy.x = guievent.x;
								temp_namedplace.xy.y = guievent.y;
								GuiAskLine *aline = new GuiAskLine("asknamedplaceheading",
										"Add MapNamedPlace", "Give heading", (*fonts));
								gui.AddComponent(aline, -1, -1);
								adding_namedplace_state = 1;
							}
							break;
						case OTHERMODE_REMOVE:
							std::vector<int> places;
							places = current_floor->GetNamedPlacesInBlock(guievent.x, guievent.y);
							current_floor->RemoveNamedPlaces(places);
							break;
						}
					}
				}
			}
			else if(guievent.type == GUIEVENT_MAPVIEWER_BLOCKHOVER){
				//std::cout<<"map block "<<guievent.x<<","<<guievent.y<<" hovered"<<std::endl;
				//MapBlock *b = current_floor->GetBlock(guievent.x, guievent.y);
				if(othermode == OTHERMODE_SELECT){
				}
				else{
					current_floor->ClearBlockHilights();
					switch(editmode){
					case EDITMODE_TILES:
						current_floor->AddBlockHilight(guievent.x, guievent.y);
						switch(othermode){
						case OTHERMODE_ADD:
							break;
						case OTHERMODE_REMOVE:
							break;
						}
						break;
					case EDITMODE_MOVETILES:
						current_floor->AddBlockHilight(guievent.x, guievent.y);
						switch(othermode){
						case OTHERMODE_ADD:
							break;
						case OTHERMODE_REMOVE:
							break;
						}
						break;
					case EDITMODE_MAPIMAGES:
						switch(othermode){
						case OTHERMODE_ADD:
							if(selected_mapimageid!=""){
								MapImage *image = mapimagelib->GetImage(selected_mapimageid.c_str());
								int wt, ht;
								image->GetSizeTiles(wt, ht);
								for(int y=0; y<ht; y++){
									for(int x=0; x<wt; x++){
										int y2 = guievent.y + y;
										int x2 = guievent.x + x;
										current_floor->AddBlockHilight(x2, y2);
									}
								}
							}
							break;
						case OTHERMODE_REMOVE:
						{
							std::vector<int> imgs = current_floor->GetImagesInBlock(guievent.x,
									guievent.y, mapimagelib);
							std::vector<MapImageInfo> infos = current_floor->GetMapImageInfos();
							for(unsigned int i=0; i<imgs.size(); i++){
								int a = imgs[i];
								MapImage *image = mapimagelib->GetImage(infos[a].shortid);
								int wt, ht;
								image->GetSizeTiles(wt, ht);
								for(int y=0; y<ht; y++){
									for(int x=0; x<wt; x++){
										int y2 = infos[a].y + y;
										int x2 = infos[a].x + x;
										current_floor->AddBlockHilight(x2, y2);
									}
								}
							}
							break;
						}
						}
						break;
					case EDITMODE_NAMEDPLACES:
						switch(othermode){
						case OTHERMODE_ADD:
							break;
						case OTHERMODE_REMOVE:
							break;
						}
					}
				}
			}
			else if(guievent.type == GUIEVENT_REQUEST_HIDEMOUSE){
				SDL_ShowCursor(SDL_DISABLE);
			}
			else if(guievent.type == GUIEVENT_REQUEST_SHOWMOUSE){
				SDL_ShowCursor(SDL_ENABLE);
			}
			else if(guievent.type == GUIEVENT_REQUEST_WARPMOUSE){
				mousex = guievent.x;
				mousey = guievent.y;
				SDL_WarpMouse(mousex, mousey);
			}
			else if(guievent.type == GUIEVENT_ASKLINE_RETURN){
				std::cout<<"askline "<<guievent.cnames[0]<<" returned '"
						<<guievent.text<<"'"<<std::endl;
				if(guievent.cnames[0]=="aline1"){
					gui.StartMessageBox("somebox", "TSOMG", "You wrote \""+guievent.text+"\"");
				}
				if(guievent.cnames[0]=="asksavefilename"){
					asked_save = false;
					filename = guievent.text;
					std::cout<<"saving map to "<<filename<<std::endl;
					if(map.WriteToFile(filename)){
						std::cout<<"error saving map to "<<filename<<std::endl;
						gui.StartMessageBox("", "Save", "Save failed");
					}
					else{
						std::cout<<"saved map to "<<filename<<std::endl;
						gui.StartMessageBox("", "Save", "Saved");
					}
				}
				if(guievent.cnames[0]=="asknamedplaceheading"
						&& adding_namedplace_state == 1){
					temp_namedplace.heading = atoi(guievent.text.c_str());
					GuiAskLine *aline = new GuiAskLine("asknamedplacename",
							"Add MapNamedPlace", "Give name", (*fonts));
					gui.AddComponent(aline, -1, -1);
					adding_namedplace_state = 2;
				}
				if(guievent.cnames[0]=="asknamedplacename"
						&& adding_namedplace_state == 2){
					adding_namedplace_state = 0;
					sprintf(temp_namedplace.name,
							"%s", guievent.text.c_str());
					current_floor->AddNamedPlace(temp_namedplace);
				}
			}
			else if(guievent.type == GUIEVENT_ASKLINE_CANCEL){
				if(guievent.cnames[0]=="asksavefilename"){
					asked_save = false;
				}
				std::string a("asknamedplace");
				if(guievent.cnames[0].substr(0, a.size()) == a){
					adding_namedplace_state = 0;
				}
			}
		}
		
		viewer->RollAnims(SDL_GetTicks());

		//piirretään jutut
		if(showingallfloors){
			viewer->SetFloorToDraw(-1);
		}
		else{
			viewer->SetFloorToDraw(current_floor_i);
		}
		gui.Draw(0, 0, 800, 600);
		
		SDL_Flip(screen);
		
		//break;
	}
	return 0;
}

