#include "mapimagelib.h"
#include "../include/map.h"
#include "graphics.h"
#include "commonfunctions.h"
#include <iostream>
#include <fstream>
#include <SDL/SDL_image.h>

MapImage::~MapImage()
{
}

MapImage::MapImage()
{
	anims_loaded = false;
	w_tiles = -1;
	h_tiles = -1;
}

MapImage::MapImage(std::string shortid_, Animation under, Animation over,
		int backswapy)
{
	anims_loaded = false;
	w_tiles = -1;
	h_tiles = -1;
	SetAll(shortid_, under, over, backswapy);
}

bool MapImage::SetAll(std::string shortid_, Animation under,
		Animation over, int backswapy_)
{
	shortid = shortid_;
	backswapy = backswapy_;
	return SetAnims(under, over);
}

void MapImage::FreeAll()
{
	FreeAnims();
}

std::string MapImage::GetShortID()
{
	return shortid;
}

bool MapImage::SetAnims(Animation under, Animation over)
{
	under_anim = under;
	over_anim = over;
	w_px = over_anim.GetCurrentImage()->w;
	h_px = over_anim.GetCurrentImage()->h;
	if(w_px % DEFAULTBLOCKW != 0 || h_px % DEFAULTBLOCKH != 0 || w_px == 0 || h_px == 0){
		std::cout<<"MapImage::SetAnimsation(): invalid size"<<std::endl;
		FreeAnims();
		return false;
	}
	w_tiles = w_px / DEFAULTBLOCKW;
	h_tiles = h_px / DEFAULTBLOCKH;
	anims_loaded = true;
	return true;
}

Animation& MapImage::GetUnderAnim()
{
	return under_anim;
}

Animation& MapImage::GetOverAnim()
{
	return over_anim;
}

int MapImage::GetBackSwapY()
{
	return backswapy;
}

void MapImage::FreeAnims()
{
	under_anim.FreeImages();
	over_anim.FreeImages();
	anims_loaded = false;
}

void MapImage::RollAnims(int timenow_ms)
{
	under_anim.RollFrames(timenow_ms);
	over_anim.RollFrames(timenow_ms);
}

void MapImage::GetSizeTiles(int &w, int &h)
{
	w = w_tiles;
	h = h_tiles;
}

//////

MapImageLib::~MapImageLib()
{
	Clear();
}

int MapImageLib::Load(std::string path)
{
	std::cout<<"MapImageLib::Load(\""<<path<<"\")"<<std::endl;
	
	std::string indexfname = path + "/index.txt";
	std::fstream ifile(indexfname.c_str());
	if(!ifile.good()){
		std::cout<<"failed to open "<<indexfname<<std::endl;
		return -1;
	}
	while(!ifile.eof()){
		char cline[1024];
		ifile.getline(cline, 1024);
		std::string line = trim(cline);
		if(line == "") continue;
		if(line[0]=='#') continue;
		
		//haetaan riviltä kaikki tiedot
		
		std::cout<<"line: \""<<line<<"\""<<std::endl;
		Strfnd f(line);
		std::string shortid = trim(f.next(" "));
		std::string size_s = trim(f.next(" "));
		std::string backswapy_s = trim(f.next(":"));
		f.next(" ");

		int backswapy = atoi(backswapy_s.c_str());

		Strfnd fs(size_s);
		std::string w_s = fs.next("x");
		std::string h_s = fs.next("");
		int w = atoi(w_s.c_str());
		int h = atoi(h_s.c_str());
		if(w<=0 || h<=0){
			std::cout<<"invalid size: "<<size_s<<std::endl;
			return -1;
		}
		std::cout<<"shortid \""<<shortid<<"\", size "<<w<<"x"<<h<<std::endl;

		std::vector<int> frame_durations;
		while(!f.atend()){
			std::string duration_string = trim(f.next(" "));
			int duration = atoi(duration_string.c_str());
			if(duration <= 0){
				std::cout<<"invalid frame duration: "<<duration<<std::endl;
				return -1;
			}
			frame_durations.push_back(duration);
		}
		
		unsigned int framecount = frame_durations.size();

		std::cout<<"frame durations: ";
		for(unsigned int i=0; i<framecount; i++){
			std::cout<<frame_durations[i]<<" ";
		}
		std::cout<<std::endl;

		//ladataan kuvatiedosto
		
		std::string wholename = path + "/" + shortid + ".png";
		SDL_Surface *tempimg = IMG_Load((wholename).c_str());
		if(tempimg==NULL){
			std::cout<<"could not load "<<wholename<<std::endl;
			return -1;
		}
		
		int needed_width = w * framecount;
		int needed_height = h * 2; //under+over
		if(tempimg->w != needed_width || tempimg->h != needed_height){
			SDL_FreeSurface(tempimg);
			std::cout<<"Invalid sized image. According to index.txt, needs "
					<<needed_width<<"x"<<needed_height<<std::endl;
			return -1;
		}
		SDL_Surface *img = SDL_DisplayFormat(tempimg);
		SDL_FreeSurface(tempimg);
		
		//animaatioden lataus
		
		Animation under_animation, over_animation;
		
		int y;
		
		y = 0 * h;
		
		{//vc6
		for(unsigned int a = 0; a < framecount; a++){
			int x = a * w;
			AnimationImage anim_image;
			SDL_Surface *temp = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, SCREENBPP,
					rmask, gmask, bmask, amask);
			if(temp == NULL){
				std::cout<<"failed to create surface"<<std::endl;
				return -1;
			}
			GDrawIMG(temp, img, 0, 0, w, h, x, y);

			SDL_SetColorKey(temp, SDL_SRCCOLORKEY, 
					SDL_MapRGB(temp->format, 255, 0, 255));

			anim_image.image = SDL_DisplayFormat(temp);

			anim_image.duration_ms = frame_durations[a];

			under_animation.AddImage(anim_image);
		}
		}//vc6

		y = 1 * h;
		
		{//vc6
		for(unsigned int a = 0; a < framecount; a++){
			int x = a * w;
			AnimationImage anim_image;
			SDL_Surface *temp = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, SCREENBPP,
					rmask, gmask, bmask, amask);
			if(temp == NULL){
				std::cout<<"failed to create surface"<<std::endl;
				return -1;
			}
			GDrawIMG(temp, img, 0, 0, w, h, x, y);

			SDL_SetColorKey(temp, SDL_SRCCOLORKEY, 
					SDL_MapRGB(temp->format, 255, 0, 255));

			anim_image.image = SDL_DisplayFormat(temp);

			anim_image.duration_ms = frame_durations[a];

			over_animation.AddImage(anim_image);
		}
		}//vc6
	
		//tehdään mapimage ja tungetaan sinne kaikki
		MapImage mapimage(shortid, under_animation, over_animation,
				backswapy);

		mapimages.push_back(mapimage);

		SDL_FreeSurface(img);
	}
	
	return 0;
}

void MapImageLib::Clear()
{
	for(unsigned int i=0; i<mapimages.size(); i++){
		mapimages[i].FreeAll();
	}
	mapimages.clear();
}

std::vector<std::string> MapImageLib::ListIDs()
{
	std::vector<std::string> shortids;
	for(unsigned int i=0; i<mapimages.size(); i++){
		shortids.push_back(mapimages[i].GetShortID());
	}
	return shortids;
}

MapImage* MapImageLib::GetImage(std::string shortid)
{
	for(unsigned int i=0; i<mapimages.size(); i++){
		if(mapimages[i].GetShortID() == shortid){
			return &mapimages[i];
		}
	}
	std::cout<<"MapImageLib::GetImage(): \""<<shortid<<"\" not found?"<<std::endl;
	return NULL;
}

void MapImageLib::RollAnims(int timenow_ms)
{
	//std::cout<<"MapImageLib::RollAnims("<<timenow_ms<<")"<<std::endl;
	for(unsigned int i=0; i<mapimages.size(); i++){
		mapimages[i].RollAnims(timenow_ms);
	}
}

