#include "objectimagelib.h"
#include "commonfunctions.h"
#include "graphics.h"
#include <fstream>
#include <iostream>
#include <SDL/SDL_image.h>

ObjectImageLib::~ObjectImageLib()
{
	Clear();
}

int ObjectImageLib::Load(std::string path)
{
	std::cout<<"ObjectImageLib::Load(\""<<path<<"\")"<<std::endl;
	std::string indexfname = path + "/index.txt";
	std::fstream ifile(indexfname.c_str());
	if(!ifile.good()){
		std::cout<<"failed to open "<<indexfname<<std::endl;
		return -1;
	}
	while(!ifile.eof()){
		char cline[1024];
		ifile.getline(cline, 1024);
		std::string line = trim(cline);
		if(line == "") continue;
		if(line[0] == '#') continue;
		
		//haetaan riviltä kaikki tiedot
		
		std::cout<<"line: \""<<line<<"\""<<std::endl;
		
		Strfnd f(line);
		std::string shortid = trim(f.next(":"));
		std::string size_s = trim(f.next(":"));
		std::string yoffset_s = trim(f.next(":"));

		Strfnd fs(size_s);
		std::string w_s = fs.next("x");
		std::string h_s = fs.next("");
		int w = atoi(w_s.c_str());
		int h = atoi(h_s.c_str());
		if(w<=0 || h<=0){
			std::cout<<"invalid size: "<<size_s<<std::endl;
			return -1;
		}

		int yoffset = atoi(yoffset_s.c_str());
		
		//laskeskellaan montako animaatioita on kaikkiaan
		unsigned int wherewas = f.where();
		int all_animations_count = 0;
		while(f.next(":") != "") all_animations_count++;
		f.to(wherewas);

		//ladataan kuvatiedosto
		
		std::string wholename = path + "/" + shortid + ".png";
		SDL_Surface *tempimg = IMG_Load((wholename).c_str());
		SDL_SetColorKey(tempimg, SDL_SRCCOLORKEY, 
				SDL_MapRGB(tempimg->format, 255, 0, 255));
		if(tempimg==NULL){
			std::cout<<"could not load "<<wholename<<std::endl;
			return -1;
		}

		//tarkistellaan kuvan koko
		
		//käytetään varsinaisesti vasta liikkumisanimaation latauksessa,
		//mutta seuraavan takia jo tässä
		Strfnd durf = trim(f.next(":"));
		
		bool no_move_animations = false;
		if(durf.what()=="nomove"){
			no_move_animations = true;
			all_animations_count--;
		}
		int other_animations_count;
		if(no_move_animations) other_animations_count = all_animations_count;
		else other_animations_count = all_animations_count - 1;
		
		int needed_width = 4 * w;
		int needed_height = (1 + (no_move_animations?0:4) + other_animations_count) * h;
		if(tempimg->w < needed_width || tempimg->h != needed_height){
			SDL_FreeSurface(tempimg);
			std::cout<<"invalid sized image, needs minimum of "
					<<needed_width<<" px width and exactly "
					<<needed_height<<" px height"<<std::endl;
			return -1;
		}
		SDL_Surface *img = SDL_DisplayFormat(tempimg);
		SDL_FreeSurface(tempimg);
		
		//aletaan kaivella kuvatiedostosta kuvia ObjectImageChunkille

		ObjectImageChunk imagechunk;
		imagechunk.shortid = shortid;
		imagechunk.yoffset = yoffset;

		std::cout<<"adding main images"<<std::endl;

		//pääkuvat
		
		SDL_Surface *mainimages[4];
		for(int i=0; i<4; i++){
			mainimages[i]  = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, SCREENBPP,
					rmask, gmask, bmask, amask);
			if(mainimages[i] == NULL){
				std::cout<<"failed to create surface"<<std::endl;
				return -1;
			}
			int x = i * w;
			GDrawIMG(mainimages[i], img, 0, 0, w, h, x, 0);
		}

		imagechunk.main_image.SetImages(mainimages);

		//animaatiot
		
		//maksimi framecountti kuvan koon mukaan
		int max_frames = img->w / w;
		
		//liikkumisanimaatio
		
		std::cout<<"adding move animations"<<std::endl;
		
		Way4Animation move_animation;

		if(!no_move_animations){
			for(int a = 0; a < max_frames; a++){
				int x = a * w;
				if(IsBlockSingleColoured(img, a * w, h, w, h * 4, 0,255,0)){
					break;
				}

				//std::cout<<"loading images for all headings for frame "<<a<<std::endl;

				SDL_Surface *images[4];
				for(int i=0; i<4; i++){
					int y = (1 + i) * h;
					images[i]  = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, SCREENBPP,
							rmask, gmask, bmask, amask);
					if(images[i] == NULL){
						std::cout<<"failed to create surface"<<std::endl;
						return -1;
					}
					GDrawIMG(images[i], img, 0, 0, w, h, x, y);
				}

				std::string frameduration_ms_s = trim(durf.next(","));
				if(frameduration_ms_s==""){
					std::cout<<"invalid frame duration for move animation frame "<<a<<std::endl;
					return -1;
				}
				int frameduration_ms = atoi(frameduration_ms_s.c_str());

				AnimationWay4Image anim_w4image;
				anim_w4image.image.SetImages(images);
				anim_w4image.duration_ms = frameduration_ms;
				move_animation.AddImage(anim_w4image);
			}
			
			imagechunk.move_animation = move_animation;
		}

		//muut animaatiot
		
		std::cout<<"adding other animations"<<std::endl;
		
		{//vc6
		for(int i=0; i<other_animations_count; i++){
			Animation other_animation;
			
			Strfnd durf = f.next(":");
		
			int y = (1 + (no_move_animations?0:4) + i) * h;
			
			for(int a = 0; a < max_frames; a++){
				int x = a * w;
				if(IsBlockSingleColoured(img, a * w, h, w, h * 4, 0,255,0)){
					break;
				}
				AnimationImage anim_image;
				anim_image.image = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, SCREENBPP,
						rmask, gmask, bmask, amask);
				if(anim_image.image == NULL){
					std::cout<<"failed to create surface"<<std::endl;
					return -1;
				}
				GDrawIMG(anim_image.image, img, 0, 0, w, h, x, y);

				std::string frameduration_ms_s = trim(durf.next(","));
				if(frameduration_ms_s==""){
					std::cout<<"invalid frame duration for other animation "
							<<i<<" frame "<<a<<std::endl;
					return -1;
				}
				int frameduration_ms = atoi(frameduration_ms_s.c_str());

				anim_image.duration_ms = frameduration_ms;

				other_animation.AddImage(anim_image);
			}

			imagechunk.other_animations.push_back(other_animation);
		}
		}//vc6

		SDL_FreeSurface(img);
		
		chunks.push_back(imagechunk);
	}
	
	return 0;
}

void ObjectImageLib::Clear()
{
	for(unsigned int i=0; i<chunks.size(); i++){
		chunks[i].main_image.FreeImages();
		chunks[i].move_animation.FreeImages();
		for(unsigned int a=0; a<chunks[i].other_animations.size(); a++){
			chunks[i].other_animations[a].FreeImages();
		}
		chunks[i].other_animations.clear();
	}
	chunks.clear();
}

ObjectImageChunk ObjectImageLib::GetChunk(std::string shortid)
{
	for(unsigned int i=0; i<chunks.size(); i++){
		if(chunks[i].shortid == shortid){
			//std::cout<<"ObjectImageLib::GetChunk(): "<<(int)chunks[i].main_image.GetImage(HEADING_S)<<std::endl;
			return chunks[i];
		}
	}
	ObjectImageChunk b;
	return b;
}

