#include "script.h"
#include "commonfunctions.h"
#include "superlaskin.h"
#include <iostream>
#include <fstream>
#include "ctype.h"

void ScriptInstance::SetStringVariable(std::string name, std::string value)
{
	if(name[0] == '$'){
		// Get the value and put things into gvariables
		VariableNode & var = vars.FindVar(name);
		if(var.name == "") {
			VariableNode addvar;
			addvar.SetString(value);
			addvar.name = name;
			vars.AddVar(addvar);
		}
		else {
			var.SetString(value);
		}
	}
	else if(name[0] == '@'){
		// Get the value and put things into gvariables
		VariableNode & var = script->vars.FindVar(name);
		if(var.name == "") {
			VariableNode addvar;
			addvar.SetString(value);
			addvar.name = name;
			script->vars.AddVar(addvar);
		}
		else {
			var.SetString(value);
		}
	}
	else if(name[0] == '%'){
		name = name.substr(1);
		// Get the value and put things into gvariables
		VariableNode & var = script->gvariables->FindVar(name);
		if(var.name == "") {
			VariableNode addvar;
			addvar.SetString(value);
			addvar.name = name;
			script->gvariables->AddVar(addvar);
		}
		else {
			var.SetString(value);
		}
	}
	else{
		std::cout<<"SCRIPT: invalid variable name \""<<name
				<<"\", variable name has to start with $ or @"<<std::endl;
		return;
	}
}

void ScriptInstance::SetIntVariable(std::string name, int value)
{
	if(name[0] == '$'){
		// Get the value and put things into gvariables
		VariableNode & var = vars.FindVar(name);
		VariableNode addvar;
		addvar.SetInt(value);
		if(var.name == "") {
			addvar.name = name;
			vars.AddVar(addvar);
		}
		else {
			var.SetString(addvar.GetString());
		}
	}
	else if(name[0] == '@'){
		// Get the value and put things into gvariables
		VariableNode & var = script->vars.FindVar(name);
		VariableNode addvar;
		addvar.SetInt(value);
		if(var.name == "") {
			addvar.name = name;
			script->vars.AddVar(addvar);
		}
		else {
			var.SetString(addvar.GetString());
		}
	}
	else if(name[0] == '%'){
		name = name.substr(1);
		// Get the value and put things into gvariables
		VariableNode & var = script->gvariables->FindVar(name);
		VariableNode addvar;
		addvar.SetInt(value);
		if(var.name == "") {
			addvar.name = name;
			script->gvariables->AddVar(addvar);
		}
		else {
			var.SetString(addvar.GetString());
		}
	}
	else{
		std::cout<<"SCRIPT: invalid variable name \""<<name
				<<"\", variable name has to start with $ or @"<<std::endl;
		return;
	}
}

std::string ScriptInstance::GetStringVariable(std::string name)
{
	if(name[0] == '$'){
		VariableNode & var = vars.FindVar(name);
		if(var.name == "") return "";
		else return var.GetString();
	}
	else if(name[0] == '@'){
		VariableNode & var = script->vars.FindVar(name);
		if(var.name == "") return "";
		else return var.GetString();
	}
	else if(name[0] == '%'){
		name = name.substr(1);
		VariableNode & var = script->gvariables->FindVar(name);
		if(var.name == "") return "";
		else return var.GetString();
	}
	else return "";
}

int ScriptInstance::GetIntVariable(std::string name)
{
	if(name[0] == '$'){
		VariableNode & var = vars.FindVar(name);
		if(var.name == "") return 0;
		else return var.GetInt();
	}
	else if(name[0] == '@'){
		VariableNode & var = script->vars.FindVar(name);
		if(var.name == "") return 0;
		else return var.GetInt();
	}
	else if(name[0] == '%'){
		name = name.substr(1);
		VariableNode & var = script->gvariables->FindVar(name);
		if(var.name == "") return 0;
		else return var.GetInt();
	}
	return 0;
}

std::string ScriptInstance::GetStringValue(std::string arg)
{
	std::string wholestring;
	unsigned int place = 0;
	do{
		bool quotes = false;
		unsigned int oldplace = place;
		for(; place < arg.size(); place++){
			if(arg[place] == '\"') quotes = !quotes;
			else if(arg[place] == '+' && quotes == false) break;
		}

		std::string argpart = trim(arg.substr(oldplace, place - oldplace));

		if(argpart[0] == '$' || argpart[0] == '@' || argpart[0] == '%'){
			wholestring += GetStringVariable(argpart);
		}
		else{
			if(argpart[0] == '\"' && argpart[argpart.size()-1] == '\"'){
				wholestring += argpart.substr(1, argpart.size()-2);
			}
			else{
				wholestring += argpart;
			}
		}
		place++;
	} while(place < arg.size());
	return wholestring;
}

int ScriptInstance::GetSingleIntValue(std::string arg)
{
	if(arg[0] == '$' || arg[0] == '@' || arg[0] == '%'){
		return GetIntVariable(arg);
	}
	else{
		return atoi(arg.c_str());
	}
}

int ScriptInstance::GetIntValue(std::string arg)
{
	//std::cout<<"GetIntValue(\""<<arg<<"\")"<<std::endl;
	unsigned int place = 0;
	do{
		//std::cout<<"arg=\""<<arg<<"\""<<std::endl;
		int varstart=-1, varend=-1;
		for(; place < arg.size(); place++){
			if(arg[place] == '$' || arg[place] == '@' || arg[place] == '%'){
				varstart = place;
				break;
			}
		}
		place++;
		for(; place < arg.size(); place++){
			if(!isalnum(arg[place])){
				varend = place;
				break;
			}
		}
		if(varstart != -1){
			if(varend == -1) varend = arg.size();
			int varlength = varend - varstart;
			//std::cout<<"varstart "<<varstart<<" varlength "<<varlength<<std::endl;
			std::string varname = arg.substr(varstart, varlength);
			//std::cout<<"varname="<<varname<<std::endl;
			std::string varvalue = GetStringVariable(varname);
			arg.erase(varstart, varlength);
			arg.insert(varstart, varvalue);
			place -= varlength - varvalue.size();
		}
	} while(place < arg.size());
	//std::cout<<"-> \""<<arg<<"\""<<std::endl;
	return (int)LaskeIsoLasq(arg);
}

bool ScriptInstance::DoCondition(std::string condition)
{
	std::string left_s, oper, right_s;
	unsigned int i;
	for(i=0; i<condition.size(); i++){
		if(condition[i]=='<' || condition[i]=='>'
			|| condition[i]=='=' || condition[i]=='!') break;
		left_s += condition[i];
	}
	for(; i<condition.size(); i++){
		if(!(condition[i]=='<' || condition[i]=='>'
			|| condition[i]=='=' || condition[i]=='!')) break;
		oper += condition[i];
	}
	for(; i<condition.size(); i++){
		right_s += condition[i];
	}
	int left = GetIntValue(trim(left_s));
	int right = GetIntValue(trim(right_s));
	left_s = GetStringValue(trim(left_s));
	right_s = GetStringValue(trim(right_s));
	oper = trim(oper);
	if(oper == "<"){
		return left < right;
	}
	if(oper == ">"){
		return left > right;
	}
	if(oper == "<="){
		return left <= right;
	}
	if(oper == ">="){
		return left >= right;
	}
	if(oper == "=="){
		return left == right;
	}
	if(oper == "!="){
		return left != right;
	}
	if(oper == ".=="){
		return left_s == right_s;
	}
	if(oper == ".!="){
		return left_s != right_s;
	}
	std::cout<<"invalid condition: \""<<condition<<"\""<<std::endl;
	return false;
}

//////////////////////////

Script::Script()
{
	file_loaded = false;
	init_done = false;
}

Script::~Script()
{
}

int Script::LoadFile(std::string fname)
{
	if(file_loaded) return 1;

	std::fstream ifile(fname.c_str());
	if(!ifile.good()){
		std::cout<<"failed to open "<<fname<<std::endl;
		return 1;
	}

	bool no_errors = true;
	std::string operated_string;
	int rownum = 0;
	std::string block;
	for(; !ifile.eof();){
		block = "";
		bool quotes = false;
		int brackets = 0;
		for(;;){
			if(operated_string == ""){
				if(ifile.eof()){
					if(block != ""){
						std::cout<<"could not find last ';'"<<std::endl;
						no_errors = false;
					}
					break;
				}
				char cline[1024];
				ifile.getline(cline, 1024);
				rownum++;
				operated_string += cline;
				//std::cout<<"rownum "<<rownum<<std::endl;
			}
			//std::cout<<"operated_string = \""<<operated_string<<"\""<<std::endl;
			bool gotit = false;
			while(operated_string.size() > 0){
				char thischar = operated_string[0];
				operated_string = operated_string.substr(1);
				if(thischar == '\"') quotes = !quotes;
				else if(!quotes && thischar == '#'){
					operated_string = "";
					break;
				}
				else if(!quotes && thischar == '(') brackets++;
				else if(!quotes && thischar == ')'){
					if(brackets > 0) brackets--;
					else{
						std::cout<<"row "<<rownum<<": too many closing brackets"<<std::endl;
						no_errors = false;
					}
				}
				else if(thischar == ';' && quotes == false && brackets == 0){
					gotit = true;
					break;
				}
				block += thischar;
			}
			if(gotit){
				block = trim(block);
				break;
			}
		}
		
		if(block == "") continue;
		
		//std::cout<<"block ending on row "<<rownum<<": \""<<block<<"\""<<std::endl;

		if(brackets > 0){
			std::cout<<"row "<<rownum<<": "<<brackets<<" brackets missing"<<std::endl;
			no_errors = false;
		}
		if(quotes){
			std::cout<<"row "<<rownum<<": quotes should end somewhere before ';'"<<std::endl;
			no_errors = false;
		}

		ScriptRow row;

		row.real_rownum = rownum;
		
		Strfnd f(block);

		if(block[0] == ':'){
			f.next(":");
			row.rowid = trim(f.next(":"));
		}
		row.command = trim(f.next("("));
		{//vc6
		for(unsigned int i=0; i<row.rowid.size(); i++){
			if(!isalnum(row.rowid[i]) && row.rowid[i] != '_'){
				std::cout<<"row "<<rownum<<": charactes a-z, 0-9, _ allowed in rowid"
						<<std::endl;
				no_errors = false;
				break;
			}
		}
		}//vc6

		{//vc6
		for(unsigned int i=0; i<row.command.size(); i++){
			if(!isalnum(row.command[i]) && row.command[i] != '_'){
				std::cout<<"row "<<rownum<<": charactes a-z, 0-9, _ allowed in command"
						<<std::endl;
				no_errors = false;
				break;
			}
		}
		}//vc6

		//std::cout<<"rowid=\""<<row.rowid<<"\" command=\""<<row.command<<"\""<<std::endl;
		
		WiseStrfnd wf = f.next("");
		
		std::string arguments_s = trim(wf.next(')'));
		if(!wf.atend()){
			std::cout<<"row "<<rownum<<": what crap do you have after the command?"
					" missing ';'?"<<std::endl;
			no_errors = false;
		}

		WiseStrfnd wfa(arguments_s);
		while(!wfa.atend()){
			std::string arg_s = trim(wfa.next(','));
			row.args.push_back(arg_s);
		}

		rows.push_back(row);
	}

	ifile.close();

	if(no_errors == false){
		std::cout<<"There were errors in the script"<<std::endl;
		std::cout<<"-------------------------------"<<std::endl;
		return 1;
	}

	file_loaded = true;
	filename = fname;
	return 0;
}

int Script::Init(Object * obj, Variables *g, Map *map_, ObjectImageLib * objimglib)
{
	if(file_loaded) {
		gvariables = g;
		map = map_;
		imagelib = objimglib;
		object = obj;
		std::vector<std::string> args;
		if(StartInstanceAtRowID("start", args)){
			std::cout<<"Script error: no start found"<<std::endl;
			return 1;
		}
		int i = RunInstance(); //ajellaan siihen asti että tulee joku taski
		if(i==0){
			BadError("Start instance should never get completed!\n"
					"please use do_nothing_forever() or an infinite loop.");
		}
		else if(i==1){
			//instance jäi kesken, hyvä juttu.
			init_done = true;
			return 0;
		}
		else if(i==2){
			BadError("no instance? we just started an instance!");
		}
		else if(i==3){
		}
		else{
			BadError("invalid return value from RunInstance");
		}
	}
	return 1;
}

std::string Script::GetFilename()
{
	return filename;
}

int Script::FindRowID(std::string id)
{
	for(unsigned int i=0; i<rows.size(); i++){
		if(rows[i].rowid == id) return i;
	}
	return -1;
}

void Script::BadError(int row, std::string descr)
{
	std::cout<<"SCRIPT ERROR on row "<<rows[row].real_rownum<<": "<<descr<<std::endl;
	instances.clear();
	object->GetTask()->type = TASKTYPE_NO_TASK;
}

void Script::BadError(std::string descr)
{
	std::cout<<"SCRIPT ERROR: "<<descr<<std::endl;
	instances.clear();
	object->GetTask()->type = TASKTYPE_NO_TASK;
}

int Script::StartInstanceAtRowID(std::string id, std::vector<std::string> args)
{
	ScriptInstance instance;
	instance.current_row = FindRowID(id);
	if(instance.current_row == -1) return 1;

	instance.script = this;

	instance.SetIntVariable("$argc", args.size());

	for(unsigned int i=0; i<args.size(); i++){
		char varname[20];
#ifdef _MSC_VER
		sprintf(varname, "$argv%i", i);
#else
		snprintf(varname, 20, "$argv%i", i);
#endif
		instance.SetStringVariable(varname, args[i]);
	}

	instance.command_continue = true;
	instance.command_counter = 0;
	instances.push_front(instance);

	return 0;
}

std::vector<std::string> Script::GetLastReturnValues()
{
	return lastreturnvalues;
}

std::vector<std::string> Script::RunSub(std::string rowid, std::vector<std::string> args)
{
	if(StartInstanceAtRowID(rowid, args) == 0){
		int i = RunInstance();
		if(i==0){
			std::cout<<"instance completed. good. returning."<<std::endl;
			return lastreturnvalues;
		}
		else if(i==1){
			BadError("instance not completed. this is very bad.");
		}
		else if(i==2){
			BadError("no instance? we just started an instance!");
		}
		else if(i==3){
			//virhe. joku taho on jo kertonut siitä, eli ei sanota mitään
		}
		else{
			BadError("invalid return value from RunInstance");
		}
	}
	else{
		BadError("could not start rowid \""+rowid+"\"");
	}
	std::cout<<"returning empty"<<std::endl;
	std::vector<std::string> empty;
	return empty;
}

//palauttaa 0 jos instanssi loppui, 1 jos se jäi kesken,
//2 jos ei ole instansseja, 3 jos tuli errori
int Script::RunInstance()
{
	//std::cout<<"Script::Run()"<<std::endl;
	if(instances.empty()){
		return 2;
	}
	for(;;){
		ScriptInstance *inst = &instances.front();
		/*std::cout<<"real_rownum: "<<rows[inst->current_row].real_rownum
				<<" instances.size(): "<<instances.size()<<std::endl;*/
		//jos komento on ajossa, suoritellaan sitä
		if(inst->command_continue){
			inst->command_continue = false;
			ScriptRow row = rows[inst->current_row];
			//std::cout<<"row.command=\""<<row.command<<"\""<<std::endl;
			if(row.command == ""){
				//varmaankin joku pelkkä rowid
			}
			//endif
			else if(row.command == "endif"){}
			//else
			else if(row.command == "else"){
				//hypätään oikeaan endiffiin
				int if_count = 1;
				unsigned int i;
				for(i = inst->current_row+1; i<rows.size(); i++){
					if(rows[i].command == "if") if_count++;
					else if(rows[i].command == "endif") if_count--;
					if(if_count == 0) break;
				}
				if(i == rows.size()){
					BadError(inst->current_row, "can't find matching endif");
					continue;
				}
				int nextrow = i + 1;
				inst->current_row = nextrow;
				inst->command_continue = true;
			}
			//end
			else if(row.command == "end"){
				instances.pop_front();
				return 0;
				//continue;
			}
			//return(value)
			else if(row.command == "return"){
				/*if(instances.size()==1){
					BadError(inst->current_row, "return needs an instance where to return");
					continue;
				}*/

				std::vector<std::string> retvals;
				{//vc6
				for(unsigned int i=0; i<row.args.size(); i++){
					retvals.push_back(inst->GetStringValue(row.args[i]));
				}
				}//vc6

				instances.pop_front();

				if(instances.size() > 0){
					instances.front().SetIntVariable("$retvalc", retvals.size());

					for(unsigned int i=0; i<retvals.size(); i++){
						char varname[20];
#ifdef _MSC_VER
						sprintf(varname, "$retval%i", i);
#else
						snprintf(varname, 20, "$retval%i", i);
#endif
						instances.front().SetStringVariable(varname, retvals[i]);
					}
				}

				lastreturnvalues.clear();
				{//vc6
				for(unsigned int i=0; i<retvals.size(); i++){
					lastreturnvalues.push_back(retvals[i]);
				}
				}//vc6
				
				return 0;
			}
			//goto(rowid)
			//rowid:t on niitä :omglol:-juttuja joita voi laittaa rivin
			//alkuun tai omille riveilleen
			else if(row.command == "goto"){
				if(row.args.size()!=1){
					BadError(inst->current_row, "goto needs an argument");
					return 3;
				}
				int nextrow = FindRowID(inst->GetStringValue(row.args[0]));
				if(nextrow == -1){
					BadError(inst->current_row, "no such rowid");
					return 3;
				}
				inst->current_row = nextrow;
				inst->command_continue = true;
			}
			//gosub(rowid, ...)
			else if(row.command == "gosub"){
				if(row.args.size()<1){
					BadError(inst->current_row, "gosub needs at least one argument");
					return 3;
				}
				//argumentit
				std::vector<std::string> args;
				for(unsigned int i=1; i<row.args.size(); i++){
					args.push_back(inst->GetStringValue(row.args[i]));
				}

				if(StartInstanceAtRowID(row.args[0], args)){
					BadError(inst->current_row, "no such rowid (\""+row.args[0]+"\")");
					return 3;
				}
			}
			//setint(mikä, miksi)
			else if(row.command == "setint"){
				if(row.args.size()!=2){
					BadError(inst->current_row, "setint needs 2 arguments");
					return 3;
				}
				int i = inst->GetIntValue(row.args[1]);
				inst->SetIntVariable(row.args[0], i);
			}
			//setstring(mikä, miksi)
			else if(row.command == "setstring"){
				if(row.args.size()!=2){
					BadError(inst->current_row, "setstring needs 2 arguments");
					return 3;
				}
				std::string s = inst->GetStringValue(row.args[1]);
				inst->SetStringVariable(row.args[0], s);
			}
			//if(ehto)
			//monta komentoa
			//endif
			else if(row.command == "if"){
				if(row.args.size()!=1){
					BadError(inst->current_row, "if needs an argument");
					return 3;
				}
				if(inst->DoCondition(row.args[0])){
					//suoritetaan seuraavakin komento
				}
				else{
					//hypätään oikeaan endiffiin tai elseen
					int if_count = 1;
					unsigned int i;
					for(i = inst->current_row+1; i<rows.size(); i++){
						if(rows[i].command == "if") if_count++;
						else if(rows[i].command == "endif") if_count--;
						if(if_count == 0) break;
						else if(if_count == 1 && rows[i].command == "else") break;
					}
					if(i == rows.size()){
						BadError(inst->current_row, "can't find matching endif or else");
						continue;
					}
					int nextrow = i + 1;
					inst->current_row = nextrow;
					inst->command_continue = true;
				}
			}
			//singleif(ehto)
			//yksi komento
			else if(row.command == "singleif"){
				if(row.args.size()!=1){
					BadError(inst->current_row, "singleif needs an argument");
					return 3;
				}
				if(inst->DoCondition(row.args[0])){
					//suoritetaan seuraavakin komento
				}
				else{
					if(inst->current_row + 1 >= (int)rows.size()){
						BadError(inst->current_row, "singleif needs a command after it");
						return 3;
					}
					inst->current_row+=2; //seuraavan komennon
					inst->command_continue = true;
				}
			}
			//do_nothing_forever()
			else if(row.command == "do_nothing_forever"){
				Task *task = object->GetTask();
				task->type = TASKTYPE_DO_NOTHING_FOREVER;
				return 1;
			}
			//print(message)
			else if(row.command == "print"){
				if(row.args.size()<1){
					BadError(inst->current_row, "print needs an argument");
					return 3;
				}
				std::cout<<"[script] "<<inst->GetStringValue(row.args[0])<<std::endl;
			}
			//wait(ms)
			else if(row.command == "wait"){
				if(inst->command_counter == 0){
					if(row.args.size()!=1){
						BadError(inst->current_row, "wait needs an argument");
						return 3;
					}
					Task *task = object->GetTask();
					task->type = TASKTYPE_WAIT;
					task->length = inst->GetIntValue(row.args[0]);

					inst->command_counter = 1;
					inst->command_continue = true;
					return 1;
				}
			}
			//setplace(x, y, floor, heading(=0-7))
			else if(row.command == "setplace"){
				if(row.args.size() != 4){
					BadError(inst->current_row, "needs 4 arguments");
					return 3;
				}
				PlaceInfo p;
				p.xy.x = inst->GetIntValue(row.args[0]);
				p.xy.y = inst->GetIntValue(row.args[1]);
				p.floor = inst->GetIntValue(row.args[2]);
				p.heading = inst->GetIntValue(row.args[3]);
				object->SetPlaceInfo(p);
			}
			//setheading(heading(=0-7))
			else if(row.command == "setheading"){
				if(row.args.size() != 1){
					BadError(inst->current_row, "needs 1 argument");
					return 3;
				}
				object->GetPlaceInfo()->heading = inst->GetIntValue(row.args[0]);
			}
			//getplace(x, y, floor, heading(=0-7))
			else if(row.command == "getplace"){
				if(row.args.size() != 4){
					BadError(inst->current_row, "needs 4 arguments");
					return 3;
				}
				PlaceInfo *p = object->GetPlaceInfo();
				inst->SetIntVariable(row.args[0], p->xy.x);
				inst->SetIntVariable(row.args[1], p->xy.y);
				inst->SetIntVariable(row.args[2], p->floor);
				inst->SetIntVariable(row.args[3], p->heading.GetVal());
			}
			//setimageid(id)
			else if(row.command == "setimageid"){
				if(row.args.size() != 1){
					BadError(inst->current_row, "setimageid needs 1 argument");
					return 3;
				}
				object->SetCurrentImageID(inst->GetIntValue(row.args[0]));
			}
			//setimagechunk(chunkname)
			else if(row.command == "setimagechunk"){
				if(row.args.size() != 1){
					BadError(inst->current_row, "needs one argument");
					return 3;
				}
				std::string shortid = inst->GetStringValue(row.args[0]);
				ObjectImageChunk c = imagelib->GetChunk(shortid);
				if(c.shortid == ""){
					std::cout<<"could not get imagechunk \""<<shortid<<"\""<<std::endl;
				}
				else{
					object->SetImageChunk(c);
				}
			}
			//move(x, y, speed)
			else if(row.command == "move"){
				if(inst->command_counter == 0){
					if(row.args.size()!=3){
						BadError(inst->current_row, "move needs 3 arguments");
						return 3;
					}
					Task *task = object->GetTask();
					task->type = TASKTYPE_MOVE;
					task->targetplace.x = inst->GetIntValue(row.args[0]);
					task->targetplace.y = inst->GetIntValue(row.args[1]);
					task->speed = inst->GetIntValue(row.args[2]);
					/*std::cout<<"move("<<task->targetplace.x<<","
							<<task->targetplace.y<<","<<task->speed<<")"<<std::endl;*/

					inst->command_counter = 1;
					inst->command_continue = true;
					return 1;
				}
			}
			
			//joku outo komento
			else{
				BadError(inst->current_row, "invalid command \""+row.command+"\"");
				return 3;
			}
		}
		//jos komentoa ei ole ajossa, katsotaan josko olisi seuraava ja
		//heitetään se ajoon
		else{
			inst->current_row++;
			if(inst->current_row >= (int)rows.size()){
				/*std::cout<<"end of script without end command. using end recommended."
						<<std::endl;
				instances.pop_front();*/
				BadError(inst->current_row - 1, "end of script, end needed");
				return 3;
			}
			else{
				inst->command_continue = true;
				inst->command_counter = 0;
			}
		}
	}
	BadError("RunInstance() went to end");
	return 3;
}




