#include "tilelib.h"
#include "graphics.h"
#include "filesys.h"
#include <SDL/SDL_image.h>
#include <iostream>

TileLib::TileLib()
{
}

TileLib::~TileLib()
{
	Clear();
}

int TileLib::Load(std::string path)
{
	std::cout<<"TileLib::Load(\""<<path<<"\")"<<std::endl;
	
	std::vector<DirListNode> listing = GetDirListing(path);

	if(listing.empty()) return 1;
	
	for(unsigned int i=0; i<listing.size(); i++){
		if(!listing[i].dir){
			std::string filename = listing[i].name;
			std::string shortid = filename.substr(0, filename.size()-4); //pääte pois = shortid
			//std::cout<<shortid<<std::endl;
			std::string wholename = path+"/"+filename;
			SDL_Surface *tempimg = IMG_Load((wholename).c_str());
			if(tempimg==NULL){
				std::cout<<"TileLib::Load(): WARNING: could not load filename \""
						<<wholename<<"\""<<std::endl;
			}
			SDL_SetColorKey(tempimg, SDL_SRCCOLORKEY, 
					SDL_MapRGB(tempimg->format, 255, 0, 255));
			SDL_Surface *img = SDL_DisplayFormat(tempimg);
			SDL_FreeSurface(tempimg);
			Tile tile;
			tile.image = img;
			tile.shortid = shortid;
			tiles.push_back(tile);
			std::cout<<shortid<<", ";
		}
	}
	std::cout<<std::endl;

	return 0;
}

void TileLib::Clear()
{
	for(unsigned int i=0; i<tiles.size(); i++){
		SDL_FreeSurface(tiles[i].image);
	}
	tiles.clear();
}

std::vector<std::string> TileLib::ListIDs()
{
	std::vector<std::string> shortids;
	for(unsigned int i=0; i<tiles.size(); i++){
		shortids.push_back(tiles[i].shortid);
	}
	return shortids;
}

SDL_Surface* TileLib::GetImage(std::string shortid)
{
	for(unsigned int i=0; i<tiles.size(); i++){
		if(tiles[i].shortid == shortid){
			return tiles[i].image;
		}
	}
	std::cout<<shortid<<" not found?"<<std::endl;
	return NULL;
}

